@@ -1,23 +1,22 @@
 """Flow integrations package."""
 
 # Monarch integration exports
-from .monarch import (
-    ComputeRequirements,
-    ProcessHandle,
-    ProcessLifecycleEvents,
+from flow._internal.integrations.monarch import (
+    AllocationError,
     ComputeAllocator,
-    FlowComputeAllocator,
     ComputeAllocatorFactory,
+    ComputeRequirements,
+    FlowComputeAllocator,
     MonarchFlowConfig,
     MonarchFlowError,
-    AllocationError,
     NetworkError,
+    ProcessHandle,
+    ProcessLifecycleEvents,
 )
-
-from .monarch_adapter import (
+from flow._internal.integrations.monarch_adapter import (
+    FlowProcMesh,
     MonarchAllocatorAdapter,
     MonarchFlowBackend,
-    FlowProcMesh,
     create_monarch_backend,
 )
 
