@@ -9,7 +9,7 @@ import time
 import uuid
 from dataclasses import dataclass
 from datetime import datetime, timezone
-from typing import Any, Dict, List, Optional
+from typing import Any
 
 from flow import Flow, TaskConfig
 from flow._internal.integrations.jupyter_persistence import PersistenceManager
@@ -31,10 +31,10 @@ class JupyterConnection:
     ssh_command: str  # SSH tunnel command for secure connection
 
     # For resume connections
-    last_active: Optional[str] = None
-    checkpoint_size: Optional[str] = None
-    variables_restored: Optional[int] = None
-    restore_time: Optional[float] = None
+    last_active: str | None = None
+    checkpoint_size: str | None = None
+    variables_restored: int | None = None
+    restore_time: float | None = None
 
 
 @dataclass
@@ -42,13 +42,13 @@ class LaunchOperation:
     """Tracks a detached launch operation."""
 
     session_id: str
-    task_id: Optional[str]
+    task_id: str | None
     instance_type: str
     stage: str  # 'launching', 'ready', 'failed'
     start_time: datetime
     message: str
-    error: Optional[str] = None
-    connection: Optional[JupyterConnection] = None
+    error: str | None = None
+    connection: JupyterConnection | None = None
 
 
 class JupyterIntegration:
@@ -63,7 +63,10 @@ set -euo pipefail
 # Install dependencies in venv
 python3 -m venv /opt/flow-kernel-env
 source /opt/flow-kernel-env/bin/activate
-pip install --no-cache-dir ipykernel jupyter-client pyzmq
+pip install --no-cache-dir \
+  ipykernel~=6.29 \
+  jupyter-client~=8.6 \
+  pyzmq~=26.0
 
 # Start kernel on localhost (SSH tunnel provides security)
 python -m ipykernel_launcher \
@@ -104,7 +107,11 @@ fi
 # Install in venv
 python3 -m venv /opt/flow-kernel-env
 source /opt/flow-kernel-env/bin/activate
-pip install --no-cache-dir ipykernel jupyter-client pyzmq dill
+pip install --no-cache-dir \
+  ipykernel~=6.29 \
+  jupyter-client~=8.6 \
+  pyzmq~=26.0 \
+  dill
 
 # Start kernel with persistence wrapper
 cat > /tmp/start_kernel.py << 'EOF'
@@ -133,14 +140,14 @@ echo "KERNEL_PID=$KERNEL_PID"
         self.session_manager = SessionManager()
         self.persistence_manager = PersistenceManager(flow_client)
         # Track detached operations in memory
-        self._operations: Dict[str, LaunchOperation] = {}
+        self._operations: dict[str, LaunchOperation] = {}
 
     def launch(
         self,
-        instance_type: Optional[str] = None,
+        instance_type: str | None = None,
         hours: float = 4.0,
-        session_id: Optional[str] = None,
-        min_gpu_memory_gb: Optional[int] = None,
+        session_id: str | None = None,
+        min_gpu_memory_gb: int | None = None,
     ) -> JupyterConnection:
         """Launch a new Colab session with GPU backend.
 
@@ -219,7 +226,7 @@ echo "KERNEL_PID=$KERNEL_PID"
             ssh_command=ssh_command,
         )
 
-    def resume(self, notebook_name: str, instance_type: Optional[str] = None) -> JupyterConnection:
+    def resume(self, notebook_name: str, instance_type: str | None = None) -> JupyterConnection:
         """Resume a notebook with its saved state.
 
         Args:
@@ -250,7 +257,7 @@ echo "KERNEL_PID=$KERNEL_PID"
 
         return connection
 
-    def list_sessions(self) -> List[FlowJupyterSession]:
+    def list_sessions(self) -> list[FlowJupyterSession]:
         """List all Colab sessions."""
         return self.session_manager.list_sessions()
 
@@ -266,7 +273,7 @@ echo "KERNEL_PID=$KERNEL_PID"
         # Mark session as stopped
         self.session_manager.stop_session(session_id)
 
-    def _wait_for_kernel(self, task: Task, timeout: int = 120) -> Dict[str, Any]:
+    def _wait_for_kernel(self, task: Task, timeout: int = 120) -> dict[str, Any]:
         """Wait for Jupyter kernel to be ready.
 
         Returns:
@@ -353,10 +360,10 @@ echo "KERNEL_PID=$KERNEL_PID"
 
     def launch_async(
         self,
-        instance_type: Optional[str] = None,
+        instance_type: str | None = None,
         hours: float = 4.0,
-        session_id: Optional[str] = None,
-        min_gpu_memory_gb: Optional[int] = None,
+        session_id: str | None = None,
+        min_gpu_memory_gb: int | None = None,
     ) -> str:
         """Launch a Colab session asynchronously.
 
@@ -414,7 +421,7 @@ echo "KERNEL_PID=$KERNEL_PID"
 
         return session_id
 
-    def get_launch_status(self, session_id: str) -> Dict[str, Any]:
+    def get_launch_status(self, session_id: str) -> dict[str, Any]:
         """Get the status of a launch operation.
 
         Args:
