@@ -1,104 +1,73 @@
-"""Decorator-based interface for remote function execution.
+"""Decorator-based remote execution.
 
-This module provides a clean decorator pattern for executing Python functions
-on remote GPU infrastructure. Functions remain pure Python, infrastructure
-concerns are separated, and the API provides clear boundaries.
+Annotation-first API to run Python functions on remote GPUs while keeping user
+code framework-free. Local calls run locally; `.remote()` runs on GPU.
 
-The decorator pattern complements the invoker pattern, providing an alternative
-interface for users who prefer annotation-based configuration over explicit
-function calls.
+Examples:
+    Basic usage:
+        >>> from flow import FlowApp
+        >>> app = FlowApp()
+        >>> @app.function(gpu="a100")
+        ... def train(path: str) -> dict:
+        ...     return {"ok": True}
+        >>> train.remote("s3://bucket/data.csv")
 
-Example:
-    Basic usage with GPU specification::
-
-        from flow import FlowApp
-
-        app = FlowApp()
-
-        @app.function(gpu="a100")
-        def train_model(data_path: str, epochs: int = 100):
-            import torch
-            # Training logic here
-            return {"loss": 0.01, "accuracy": 0.99}
-
-        # Execute remotely
-        result = train_model.remote("s3://bucket/data.csv", epochs=50)
-
-        # Execute locally for testing
-        local_result = train_model("./local_data.csv")
-
-    Advanced usage with full resource specification::
-
-        @app.function(
-            gpu="h100:8",  # 8x H100 GPUs
-            memory=65536,  # 64GB RAM
-            cpu=16.0,      # 16 CPU cores
-            image="pytorch/pytorch:2.0.0",
-            volumes={"/models": "model-cache"},
-            environment={"WANDB_API_KEY": "..."},
-        )
-        def distributed_training(config_path: str):
-            # Multi-GPU training code
-            return {"status": "completed"}
-
-    Handling complex data types::
-
-        @app.function(gpu="a100")
-        def process_embeddings(embeddings_path: str, output_path: str):
-            import numpy as np
-            embeddings = np.load(embeddings_path)
-            # Process embeddings...
-            results = embeddings @ embeddings.T
-            np.save(output_path, results)
-            return {"shape": list(results.shape), "output": output_path}
-
-        # Usage
-        embeddings = np.random.randn(1000, 768)
-        np.save("/tmp/embeddings.npy", embeddings)
-        result = process_embeddings.remote("/tmp/embeddings.npy", "/tmp/results.npy")
+    Advanced resources and secrets:
+        >>> from flow.api.secrets import Secret
+        >>> @app.function(
+        ...     gpu="h100:8", cpu=32.0, memory=131072,
+        ...     image="nvcr.io/nvidia/pytorch:23.10-py3",
+        ...     volumes={"/models": "model-cache"},
+        ...     environment={"NCCL_DEBUG": "INFO"},
+        ...     secrets=[Secret.from_env("WANDB_API_KEY")],
+        ... )
+        ... def train_large(config_path: str) -> dict:
+        ...     return {"status": "done"}
+        >>> task = train_large.spawn("config.yaml")
+        >>> task.wait()
 """
 
+from __future__ import annotations
+
 import functools
 import inspect
 import json
+from collections.abc import Callable
 from pathlib import Path
-from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar, Union
+from typing import Any, Generic, TypeVar
 
 from typing_extensions import ParamSpec
 
 from flow.api.client import Flow
-from flow.api.models import TaskConfig, Retries
+from flow.api.models import Retries, Task, TaskConfig
 from flow.api.secrets import Secret, validate_secrets
-from flow.errors import FlowError
 
 P = ParamSpec("P")
 R = TypeVar("R")
 
 
 class RemoteFunction(Generic[P, R]):
-    """A function wrapper that enables remote GPU execution.
+    """Wraps a Python function to support `.remote()` GPU execution.
 
-    This class wraps a Python function to allow both local and remote execution.
-    When called normally, the function executes locally. When called via .remote(),
-    it executes on cloud GPU infrastructure.
-
-    The design ensures zero coupling between user code and infrastructure code.
-    Functions remain testable, portable, and free of framework dependencies.
+    Local calls execute the function as-is; `.remote()` runs it remotely with
+    an isolated environment and returns the JSON-serializable result.
     """
 
     def __init__(
         self,
         func: Callable[P, R],
         flow_client: Flow,
-        gpu: Optional[str] = None,
-        cpu: Optional[Union[float, tuple[float, float]]] = None,
-        memory: Optional[Union[int, tuple[int, int]]] = None,
-        image: Optional[str] = None,
-        retries: Union[int, Retries] = 0,
-        timeout: Optional[int] = None,
-        volumes: Optional[Dict[str, Any]] = None,
-        environment: Optional[Dict[str, str]] = None,
-        secrets: Optional[List[Secret]] = None,
+        gpu: str | None = None,
+        cpu: float | tuple[float, float] | None = None,
+        memory: int | tuple[int, int] | None = None,
+        gpu_memory_gb: int | None = None,
+        image: str | None = None,
+        retries: int | Retries = 0,
+        timeout: int | None = None,
+        volumes: dict[str, Any] | None = None,
+        environment: dict[str, str] | None = None,
+        secrets: list[Secret] | None = None,
+        max_result_size: int | None = 10 * 1024 * 1024,
         **kwargs,
     ):
         """Initialize a RemoteFunction.
@@ -130,6 +99,8 @@ class RemoteFunction(Generic[P, R]):
         self.environment = environment or {}
         self.env = self.environment  # Backward compatibility
         self.secrets = secrets or []
+        self.gpu_memory_gb = gpu_memory_gb
+        self.max_result_size = max_result_size
         self.kwargs = kwargs
 
         # Validate secrets if provided
@@ -150,52 +121,35 @@ class RemoteFunction(Generic[P, R]):
         self.func_name = func.__name__
 
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
-        """Execute the function locally.
-
-        This allows the decorated function to be called normally for local
-        testing and development. The function executes in the current process
-        with no infrastructure overhead.
-
-        Args:
-            *args: Positional arguments for the function.
-            **kwargs: Keyword arguments for the function.
-
-        Returns:
-            The function's return value.
-
-        Example:
-            >>> @app.function(gpu="a100")
-            ... def process(x: int) -> int:
-            ...     return x * 2
-            >>> result = process(5)  # Runs locally
-            >>> assert result == 10
-        """
+        """Execute locally (no infrastructure involved)."""
         return self.func(*args, **kwargs)
 
     def remote(self, *args: P.args, **kwargs: P.kwargs) -> R:
-        """Execute the function remotely on GPU infrastructure.
-
-        Submits the function for execution on cloud GPUs with the configured
-        resources. Blocks until execution completes and returns the result.
+        """Execute remotely on GPU and return the JSON-serializable result.
 
         Args:
-            *args: Positional arguments for the function.
-            **kwargs: Keyword arguments for the function.
+            *args: Positional arguments for the wrapped function.
+            **kwargs: Keyword arguments for the wrapped function.
 
         Returns:
-            The function's return value, deserialized from the remote execution.
+            R: The function's return value.
 
         Raises:
-            FlowError: If remote execution fails or times out.
+            TypeError: If arguments cannot be JSON-serialized.
+            Exception: Propagated remote error if the function fails.
 
-        Example:
-            >>> @app.function(gpu="h100", memory=32768)
-            ... def train(data: str, lr: float = 0.001):
-            ...     # ML training code
-            ...     return {"loss": 0.15}
-            >>> result = train.remote("s3://data/train.csv", lr=0.0001)
-            >>> print(result["loss"])
-            0.15
+        Examples:
+            Simple remote call:
+                >>> @app.function(gpu="a100")
+                ... def add(x: int, y: int) -> int:
+                ...     return x + y
+                >>> add.remote(2, 3)
+
+            Configure retries and timeout:
+                >>> @app.function(gpu="a100", retries=3, timeout=600)
+                ... def train(cfg_path: str) -> dict:
+                ...     return {"ok": True}
+                >>> train.remote("config.yaml")
         """
         # Prepare the execution script
         wrapper_script = self._create_wrapper_script(args, kwargs)
@@ -208,36 +162,22 @@ class RemoteFunction(Generic[P, R]):
         # Use Task.result() to surface rich error information if the function failed
         return self._extract_result(task)
 
-    def spawn(self, *args: P.args, **kwargs: P.kwargs) -> str:
-        """Execute the function remotely without waiting.
-
-        Submits the function for asynchronous execution and returns immediately
-        with a task ID. Use this for fire-and-forget operations or when you
-        need to manage multiple concurrent executions.
+    def spawn(self, *args: P.args, **kwargs: P.kwargs) -> Task:
+        """Submit for async execution and return the `Task` immediately.
 
         Args:
-            *args: Positional arguments for the function.
-            **kwargs: Keyword arguments for the function.
+            *args: Positional arguments for the wrapped function.
+            **kwargs: Keyword arguments for the wrapped function.
 
         Returns:
-            Task ID that can be used to check status or retrieve results.
+            Task: Handle that can be used to monitor execution.
 
-        Example:
-            >>> @app.function(gpu="a100")
-            ... def process_batch(batch_id: int):
-            ...     # Process data batch
-            ...     return {"processed": batch_id}
-            >>>
-            >>> # Launch multiple tasks
-            >>> task_ids = []
-            >>> for i in range(10):
-            ...     task_id = process_batch.spawn(i)
-            ...     task_ids.append(task_id)
-            >>>
-            >>> # Check status later
-            >>> from flow import status
-            >>> for task_id in task_ids:
-            ...     print(f"{task_id}: {status(task_id)}")
+        Examples:
+            Launch N parallel tasks and stream their last logs:
+                >>> tasks = [process.spawn(i) for i in range(8)]
+                >>> for t in tasks:
+                ...     t.wait()
+                ...     print(t.logs(tail=10))
         """
         # Prepare the execution script
         wrapper_script = self._create_wrapper_script(args, kwargs)
@@ -247,23 +187,10 @@ class RemoteFunction(Generic[P, R]):
 
         # Submit task without waiting
         task = self.flow_client.run(config, wait=False)
-        return task.id
+        return task
 
-    def _create_wrapper_script(self, args: tuple, kwargs: dict) -> str:
-        """Create a Python script for remote execution.
-
-        Generates a self-contained Python script that imports the target
-        function, deserializes arguments, executes the function, and saves
-        the result. This ensures clean separation between user code and
-        infrastructure code.
-
-        Args:
-            args: Positional arguments for the function.
-            kwargs: Keyword arguments for the function.
-
-        Returns:
-            Python script as a string.
-        """
+    def _create_wrapper_script(self, args: tuple[Any, ...], kwargs: dict[str, Any]) -> str:
+        """Build the remote wrapper script that runs the function and writes JSON."""
         # Serialize arguments with helpful errors
         try:
             args_json = json.dumps({"args": list(args), "kwargs": kwargs})
@@ -291,20 +218,22 @@ sys.path.insert(0, str(Path.cwd()))
 
 from {self.module_name} import {self.func_name}
 
-args_data = json.loads({json.dumps(args_json)!r})
+MAX_RESULT_SIZE = {int(self.max_result_size) if self.max_result_size else 0}
+
+args_data = json.loads({args_json!r})
 args = args_data["args"]
 kwargs = args_data["kwargs"]
 
 try:
     func = {self.func_name}
     if inspect.iscoroutinefunction(func):
-        result = asyncio.run(func(*args, **kwargs))
+        result = asyncio.run({self.func_name}(*args, **kwargs))
     else:
-        result = func(*args, **kwargs)
+        result = {self.func_name}(*args, **kwargs)
     
     # Result must be JSON-serializable
     try:
-        json.dumps(result)
+        _serialized = json.dumps(result)
     except (TypeError, ValueError) as e:
         # Provide guidance if result is not JSON-serializable
         raise TypeError(
@@ -322,6 +251,17 @@ try:
             f"      return {{'result_path': '/outputs/result.npy', 'metrics': {{...}}}}\\n"
         ) from e
     
+    # Enforce maximum result size if configured
+    try:
+        result_size = len(_serialized.encode('utf-8'))
+        if MAX_RESULT_SIZE and result_size > MAX_RESULT_SIZE:
+            raise ValueError(
+                f"Function returned result too large: {{result_size}} bytes > {{MAX_RESULT_SIZE}} bytes. "
+                f"Save large outputs to disk and return the path instead."
+            )
+    except Exception:
+        pass
+    
     with open("/tmp/flow_result.json", "w") as f:
         json.dump({{"success": True, "result": result}}, f)
         
@@ -331,7 +271,10 @@ except Exception as e:
     error_obj = {{
         "type": type(e).__name__,
         "message": str(e),
-        "traceback": tb
+        "traceback": tb,
+        # Backward-compat aliases expected by some tests/tools
+        "error_type": type(e).__name__,
+        "error_message": str(e),
     }}
     
     with open("/tmp/flow_result.json", "w") as f:
@@ -343,13 +286,9 @@ except Exception as e:
         return wrapper
 
     def _get_serialization_error_message(
-        self, args: tuple, kwargs: dict, original_error: Exception
+        self, args: tuple[Any, ...], kwargs: dict[str, Any], original_error: Exception
     ) -> str:
-        """Generate helpful error messages for non-serializable types.
-
-        Detects common ML/scientific types and provides specific guidance
-        on how to save and load them properly.
-        """
+        """Return a clear, type-aware message for non-JSON-serializable inputs."""
         import re
 
         # Try to detect common non-serializable types
@@ -361,32 +300,32 @@ except Exception as e:
         torch_tensor_type = None
         sklearn_model_types = []
 
-        try:
-            import numpy as np
+        try:  # pragma: no cover - optional dependency
+            import numpy as np  # type: ignore
 
             numpy_array_type = np.ndarray
-        except ImportError:
+        except Exception:
             pass
 
-        try:
-            import pandas as pd
+        try:  # pragma: no cover - optional dependency
+            import pandas as pd  # type: ignore
 
             pandas_dataframe_type = pd.DataFrame
-        except ImportError:
+        except Exception:
             pass
 
-        try:
-            import torch
+        try:  # pragma: no cover - optional dependency
+            import torch  # type: ignore
 
             torch_tensor_type = torch.Tensor
-        except ImportError:
+        except Exception:
             pass
 
-        try:
-            from sklearn.base import BaseEstimator
+        try:  # pragma: no cover - optional dependency
+            from sklearn.base import BaseEstimator  # type: ignore
 
             sklearn_model_types.append(BaseEstimator)
-        except ImportError:
+        except Exception:
             pass
 
         # Check each argument for known types
@@ -460,7 +399,7 @@ except Exception as e:
                 )
 
             # PyTorch models (have state_dict method)
-            elif hasattr(arg, "state_dict") and callable(getattr(arg, "state_dict")):
+            elif hasattr(arg, "state_dict") and callable(arg.state_dict):
                 return (
                     f"Cannot serialize PyTorch model ({arg_name}) to JSON.\n"
                     f"Flow uses JSON serialization to ensure reproducibility across environments.\n"
@@ -528,35 +467,15 @@ except Exception as e:
         )
 
     def _build_task_config(self, command: str) -> TaskConfig:
-        """Build TaskConfig from decorator parameters.
-
-        Translates the decorator arguments into a TaskConfig object,
-        handling resource specifications, volume mounts, and other
-        configuration options.
-
-        Args:
-            command: Python command to execute.
-
-        Returns:
-            Configured TaskConfig ready for submission.
-        """
+        """Translate decorator settings into a `TaskConfig`."""
         # Parse resource requirements
         instance_type = None
         min_gpu_memory_gb = None
 
         if self.gpu:
             instance_type = self.gpu
-
-        # For now, CPU and memory constraints are stored but not used
-        # TaskConfig doesn't support these fields yet
-        # They could be used in future for instance selection logic
-
-        if self.memory and not self.gpu:
-            # Only use memory for GPU selection if no explicit GPU specified
-            if isinstance(self.memory, tuple):
-                min_gpu_memory_gb = self.memory[0] // 1024  # Convert MB to GB
-            else:
-                min_gpu_memory_gb = self.memory // 1024
+        elif self.gpu_memory_gb:
+            min_gpu_memory_gb = int(self.gpu_memory_gb)
 
         # Merge environment variables from secrets
         env = self.environment.copy()
@@ -572,6 +491,7 @@ except Exception as e:
                     )
             env.update(secret_env)
 
+        # Base TaskConfig fields
         config_dict = {
             "name": f"{self.func_name}-remote",
             "command": ["python", "-c", command],
@@ -604,125 +524,80 @@ except Exception as e:
                 # Ignore invalid values silently; validation will happen downstream if needed
                 pass
 
+        # Map retries (int -> Retries) if provided
+        if self.retries:
+            if isinstance(self.retries, int):
+                config_dict["retries"] = Retries.fixed(retries=self.retries)
+            else:
+                config_dict["retries"] = self.retries
+
         return TaskConfig(**config_dict)
 
-    def _extract_result(self, task) -> Any:
-        """Extract result from completed task.
-
-        Retrieves and deserializes the function's return value from the
-        completed task using SSH to fetch the result file.
-
-        Args:
-            task: Completed Task object.
-
-        Returns:
-            The function's return value.
-
-        Raises:
-            FlowError: If result cannot be retrieved or function failed.
-        """
+    def _extract_result(self, task: Task) -> Any:
+        """Fetch and return the remote function result from the completed task."""
         # Use the Task.result() method to fetch results
         return task.result()
 
 
 class FlowApp(Flow):
-    """Flow client extended with decorator support.
-
-    Provides a decorator-based interface for configuring remote function
-    execution. This class extends the base Flow client with the @function
-    decorator pattern for users who prefer declarative configuration.
-
-    Example:
-        >>> from flow import FlowApp
-        >>>
-        >>> app = FlowApp()
-        >>>
-        >>> @app.function(gpu="a100")
-        >>> def compute(n: int) -> int:
-        ...     return n ** 2
-        >>>
-        >>> result = compute.remote(10)  # Runs on GPU
-        >>> local = compute(10)          # Runs locally
-    """
+    """`Flow` with an `@function` decorator for remote execution."""
 
     def function(
         self,
-        gpu: Optional[str] = None,
-        cpu: Optional[Union[float, tuple[float, float]]] = None,
-        memory: Optional[Union[int, tuple[int, int]]] = None,
-        image: Optional[str] = None,
-        retries: Union[int, Retries] = 0,
-        timeout: Optional[int] = None,
-        volumes: Optional[Dict[str, Any]] = None,
-        environment: Optional[Dict[str, str]] = None,
-        env: Optional[Dict[str, str]] = None,  # Allow both env and environment
-        secrets: Optional[List[Secret]] = None,
+        gpu: str | None = None,
+        cpu: float | tuple[float, float] | None = None,
+        memory: int | tuple[int, int] | None = None,
+        gpu_memory_gb: int | None = None,
+        image: str | None = None,
+        retries: int | Retries = 0,
+        timeout: int | None = None,
+        volumes: dict[str, Any] | None = None,
+        environment: dict[str, str] | None = None,
+        env: dict[str, str] | None = None,  # Allow both env and environment
+        secrets: list[Secret] | None = None,
+        max_result_size: int | None = 10 * 1024 * 1024,
         **kwargs,
     ) -> Callable[[Callable[P, R]], RemoteFunction[P, R]]:
-        """Decorator to create a remote function.
-
-        Configures a function for remote execution with specified resources.
-        The decorated function can be called normally for local execution
-        or via .remote() for GPU execution.
+        """Decorator that returns a `RemoteFunction` configured with resources.
 
         Args:
-            gpu: GPU specification. Examples:
-                - "a100": Single A100 GPU
-                - "h100:4": Four H100 GPUs
-                - "l40s:8": Eight L40S GPUs
-            cpu: CPU cores. Either a float for cores or (request, limit) tuple.
-            memory: Memory in MB. Either an int or (request, limit) tuple.
-            image: Docker image name. Defaults to "python:3.11".
-            retries: Retry configuration. Either an int for simple retries
-                or a Retries object for advanced configuration.
-            timeout: Maximum execution time in seconds.
-            volumes: Volume mount specifications. Dict mapping mount paths to
-                volume references. Example: {"/data": "training-data"}.
-            environment: Environment variables for execution context.
-            secrets: List of Secret objects for secure credential injection.
-                Example: [Secret.from_name("api-key"), Secret.from_env("TOKEN")]
-            **kwargs: Additional TaskConfig parameters.
+            gpu: GPU spec, for example "a100" or "h100:8". Alternatively set
+                `gpu_memory_gb` to request by memory.
+            cpu: CPU cores as float or (request, limit).
+            memory: Memory in MB as int or (request, limit).
+            image: Container image (default: "python:3.11").
+            retries: Retry policy (int or `Retries`).
+            timeout: Maximum runtime in seconds.
+            volumes: Mapping of mount_path to volume reference. Each value can be
+                a volume name or a dict with `volume_id` and other fields.
+            environment | env: Environment variables to set.
+            secrets: Secrets to inject as environment variables.
+            max_result_size: Maximum allowed size of serialized result (bytes).
+            **kwargs: Additional `TaskConfig` fields to merge.
 
         Returns:
-            Decorator function that wraps the target function.
+            Callable[[Callable[P, R]], RemoteFunction[P, R]]: A decorator that wraps the function.
 
-        Example:
-            Simple GPU function::
+        Examples:
+            Minimal configuration:
+                >>> @app.function(gpu="a100")
+                ... def f(x: int) -> int:
+                ...     return x * 2
 
-                @app.function(gpu="a100")
-                def inference(text: str) -> dict:
-                    # Run inference
-                    return {"sentiment": "positive"}
-
-            Multi-GPU with resources::
-
-                @app.function(
-                    gpu="h100:8",
-                    cpu=32.0,
-                    memory=131072,  # 128GB
-                    image="nvcr.io/nvidia/pytorch:23.10-py3",
-                    volumes={
-                        "/data": "imagenet",
-                        "/models": "model-cache",
-                    },
-                    environment={
-                        "NCCL_DEBUG": "INFO",
-                        "CUDA_VISIBLE_DEVICES": "0,1,2,3,4,5,6,7"
-                    },
-                )
-                def train_large_model(config_path: str) -> dict:
-                    # Distributed training logic
-                    return {"final_loss": 0.023}
-
-            Resource limits::
-
-                @app.function(
-                    cpu=(4.0, 8.0),      # Request 4, limit 8 cores
-                    memory=(16384, 32768),  # Request 16GB, limit 32GB
-                )
-                def cpu_intensive(data: list) -> list:
-                    # CPU-bound processing
-                    return processed_data
+            Full configuration with volumes and env:
+                >>> @app.function(
+                ...     gpu="h100:8",
+                ...     cpu=32.0,
+                ...     memory=131072,
+                ...     image="nvcr.io/nvidia/pytorch:23.10-py3",
+                ...     volumes={
+                ...         "/data": {"name": "datasets", "size_gb": 500},
+                ...         "/ckpts": "model-checkpoints",
+                ...     },
+                ...     environment={"NCCL_DEBUG": "INFO"},
+                ... )
+                ... def train(cfg: str) -> dict:
+                ...     return {"status": "ok"}
         """
 
         def decorator(func: Callable[P, R]) -> RemoteFunction[P, R]:
@@ -734,12 +609,14 @@ class FlowApp(Flow):
                 gpu=gpu,
                 cpu=cpu,
                 memory=memory,
+                gpu_memory_gb=gpu_memory_gb,
                 image=image,
                 retries=retries,
                 timeout=timeout,
                 volumes=volumes,
                 environment=actual_env,
                 secrets=secrets,
+                max_result_size=max_result_size,
                 **kwargs,
             )
 
@@ -754,8 +631,8 @@ _app = None
 _app_lock = threading.Lock()
 
 
-def _get_app():
-    """Get or create the default app instance (thread-safe)."""
+def _get_app() -> FlowApp:
+    """Return the singleton `FlowApp` (thread-safe)."""
     global _app
     if _app is None:
         with _app_lock:
@@ -766,25 +643,25 @@ def _get_app():
 
 
 class _LazyApp:
-    """Lazy proxy for FlowApp that initializes on first use.
+    """Lazy proxy that instantiates `FlowApp` on first use."""
 
-    This proxy ensures that FlowApp is only instantiated when actually used,
-    preventing authentication checks during module import.
-    """
-
-    def __getattr__(self, name):
-        """Delegate attribute access to the actual FlowApp instance."""
+    def __getattr__(self, name: str) -> Any:
+        """Delegate attribute access to the underlying `FlowApp`."""
         return getattr(_get_app(), name)
 
-    def __call__(self):
-        """Allow calling app() to get the actual FlowApp instance."""
+    def __call__(self) -> FlowApp:
+        """Return the underlying `FlowApp` instance."""
         return _get_app()
 
-    def __repr__(self):
-        """Represent as the underlying FlowApp for debugging."""
+    def __repr__(self) -> str:
+        """Represent using the underlying `FlowApp`."""
         return repr(_get_app())
 
 
 # Create singleton instances for module-level use
 app = _LazyApp()
-function = lambda *args, **kwargs: _get_app().function(*args, **kwargs)
+
+
+def function(*args, **kwargs):
+    """Module-level helper: `function(...)` == `_get_app().function(...)`."""
+    return _get_app().function(*args, **kwargs)
