@@ -1,7 +1,7 @@
 """Base class for bridge adapters."""
 
 from abc import ABC, abstractmethod
-from typing import Any, Dict
+from typing import Any
 
 
 class BridgeAdapter(ABC):
@@ -13,7 +13,7 @@ class BridgeAdapter(ABC):
     """
 
     @abstractmethod
-    def get_capabilities(self) -> Dict[str, Any]:
+    def get_capabilities(self) -> dict[str, Any]:
         """Return a dictionary describing the adapter's capabilities.
 
         This should include:
