@@ -19,9 +19,10 @@ def main():
         raise SystemExit(1)
 
     # Import after version check
-    from .app import cli
+    from flow.cli.app import create_cli
 
-    cli(prog_name="flow")
+    cli_group = create_cli()
+    cli_group(prog_name="flow")
 
 
 __all__ = ["main"]
