@@ -12,19 +12,45 @@ import os
 import time
 import webbrowser
 from pathlib import Path
-from typing import Dict, List, Optional, Any
+from typing import Any
 
 from rich.console import Console
 from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
 from rich.prompt import Confirm, Prompt
 
 from flow._internal.io.http import HttpClient
-from flow.errors import AuthenticationError, NetworkError
 from flow.cli.utils.config_validator import ConfigValidator, ValidationStatus
 from flow.cli.utils.interactive_selector import InteractiveSelector, SelectionItem
+from flow.cli.utils.mask_utils import mask_api_key
 from flow.cli.utils.visual_constants import get_status_display
 
 
+def _run_async(awaitable):
+    """Run an awaitable safely regardless of loop state.
+
+    Uses asyncio.run() when no loop is running; otherwise falls back to a
+    loop-agnostic execution by creating a new task on the existing loop.
+    """
+    try:
+        loop = asyncio.get_running_loop()
+    except RuntimeError:
+        loop = None
+    if loop and loop.is_running():
+        # Best-effort inline execution in existing loop
+        try:
+            import nest_asyncio  # type: ignore
+
+            try:
+                nest_asyncio.apply()
+            except Exception:
+                pass
+        except Exception:
+            pass
+        return loop.run_until_complete(awaitable)
+    else:
+        return asyncio.run(awaitable)
+
+
 class AnimatedDots:
     """Minimal animated dots implementation for progress messages."""
 
@@ -45,12 +71,15 @@ class AnimatedDots:
 
 def select_from_options(
     console: Console,
-    options: List[Dict[str, Any]],
+    options: list[dict[str, Any]],
     name_key: str = "name",
     id_key: str = "id",
     title: str = "Select an option",
     show_ssh_table: bool = False,
-) -> Optional[Dict[str, Any]]:
+    extra_header_html: str | None = None,
+    breadcrumbs: list[str] | None = None,
+    preferred_viewport_size: int | None = None,
+) -> dict[str, Any] | None:
     """Intelligently select from options using appropriate UI pattern.
 
     Uses simple prompt for few options, interactive selector for many.
@@ -72,7 +101,7 @@ def select_from_options(
     if len(options) == 1:
         option = options[0]
         name = option.get(name_key, "Unknown")
-        console.print(f"{get_status_display('configured', f'Using: {name}')}")
+        console.print(f"{get_status_display('configured', f'Selected project: {name}')}")
         return option
 
     # Show SSH keys in a table format if requested
@@ -98,7 +127,7 @@ def select_from_options(
             )
 
         # Use interactive selector with custom formatting
-        def ssh_option_to_selection(option: Dict[str, Any]) -> SelectionItem[Dict[str, Any]]:
+        def ssh_option_to_selection(option: dict[str, Any]) -> SelectionItem[dict[str, Any]]:
             if option.get(id_key, "").startswith("GENERATE_"):
                 # Action item
                 return SelectionItem(
@@ -125,15 +154,22 @@ def select_from_options(
             item_to_selection=ssh_option_to_selection,
             title=title,  # Use the provided title
             allow_multiple=False,
+            allow_back=True,
+            show_preview=False,
+            breadcrumbs=breadcrumbs,
+            extra_header_html=extra_header_html,
+            preferred_viewport_size=preferred_viewport_size,
         )
 
         result = selector.select()
+        if result is InteractiveSelector.BACK_SENTINEL:
+            return None
         return result if result else None
 
     # Always try interactive selector first for better UX
     try:
 
-        def option_to_selection(option: Dict[str, Any]) -> SelectionItem[Dict[str, Any]]:
+        def option_to_selection(option: dict[str, Any]) -> SelectionItem[dict[str, Any]]:
             name = option.get(name_key, "Unknown")
             option_id = option.get(id_key, str(hash(str(option))))
 
@@ -163,10 +199,20 @@ def select_from_options(
                 )
 
         selector = InteractiveSelector(
-            items=options, item_to_selection=option_to_selection, title=title, allow_multiple=False
+            items=options,
+            item_to_selection=option_to_selection,
+            title=title,
+            allow_multiple=False,
+            allow_back=True,
+            show_preview=False,
+            breadcrumbs=breadcrumbs,
+            extra_header_html=extra_header_html,
+            preferred_viewport_size=preferred_viewport_size,
         )
 
         result = selector.select()
+        if result is InteractiveSelector.BACK_SENTINEL:
+            return None
         if result is not None:
             return result
 
@@ -181,7 +227,7 @@ def select_from_options(
         console.print(f"  {i}. {name}")
 
     while True:
-        choice = Prompt.ask(f"\nSelect number", default="1")
+        choice = Prompt.ask("\nSelect number", default="1")
         try:
             idx = int(choice) - 1
             if 0 <= idx < len(options):
@@ -204,7 +250,7 @@ class ApiKeySetup:
         self.console = console
         self.validator = ConfigValidator()
 
-    def setup(self, existing_config: Dict[str, Any]) -> Optional[str]:
+    def setup(self, existing_config: dict[str, Any]) -> str | None:
         """Configure and validate API key.
 
         Args:
@@ -227,14 +273,16 @@ class ApiKeySetup:
         # Show instructions
         self.console.print("\nFlow SDK requires an API key from Mithril.")
         self.console.print("\n[bold]Get your API key:[/bold]")
-        self.console.print("1. Visit: [link]https://app.mithril.ai/account/apikeys[/link]")
+        from flow.links import WebLinks
+        self.console.print(f"1. Visit: [link]{WebLinks.api_keys()}[/link]")
         self.console.print("2. Sign in or create an account")
         self.console.print("3. Create a new key or copy an existing one")
 
         # Offer to open browser
         if Confirm.ask("\n[bold]Open browser?[/bold]", default=True):
             self.console.print("[dim]Opening browser...[/dim]")
-            webbrowser.open("https://app.mithril.ai/account/apikeys")
+            from flow.links import WebLinks
+            webbrowser.open(WebLinks.api_keys())
             time.sleep(2)
 
         # Get API key with validation
@@ -252,12 +300,7 @@ class ApiKeySetup:
                     continue
 
             # Show masked feedback
-            masked_key = (
-                f"{api_key[:5]}{'*' * (len(api_key) - 9)}{api_key[-4:]}"
-                if len(api_key) > 9
-                else api_key
-            )
-            self.console.print(f"[dim]Received: {masked_key}[/dim]")
+            self.console.print(f"[dim]Received: {mask_api_key(api_key)}[/dim]")
 
             # Verify the key works
             if self._verify_api_key(api_key):
@@ -287,10 +330,15 @@ class ApiKeySetup:
             task = progress.add_task(f"Verifying API key{dots.next()}", total=None)
 
             try:
-                result = asyncio.run(self.validator.verify_api_key(api_key))
+                result = _run_async(self.validator.verify_api_key(api_key))
 
                 if result.status == ValidationStatus.VALID:
-                    progress.update(task, description="[green]✓ API key verified![/green]")
+                    from flow.cli.utils.theme_manager import theme_manager as _tm
+
+                    success_color = _tm.get_color("success")
+                    progress.update(
+                        task, description=f"[{success_color}]✓ API key verified![/{success_color}]"
+                    )
                     if result.details and result.details.get("project_count"):
                         self.console.print(
                             f"\n[dim]Found {result.details['project_count']} project(s)[/dim]"
@@ -302,7 +350,9 @@ class ApiKeySetup:
 
             except Exception as e:
                 progress.update(task, description="[red]✗ Verification failed[/red]")
-                self.console.print(f"\n[red]Error:[/red] {e}")
+                from rich.markup import escape
+
+                self.console.print(f"\n[red]Error:[/red] {escape(str(e))}")
                 return False
 
 
@@ -318,7 +368,7 @@ class ProjectSetup:
         self.console = console
         self.validator = ConfigValidator()
 
-    def setup(self, api_key: str) -> Optional[str]:
+    def setup(self, api_key: str) -> str | None:
         """Configure project selection.
 
         Args:
@@ -335,7 +385,10 @@ class ProjectSetup:
             self.console.print("\n[yellow]Could not fetch projects from API.[/yellow]")
             self.console.print("\nEnter your project name manually.")
             self.console.print(
-                "[dim]To create projects: https://app.mithril.ai/settings/projects[/dim]"
+                (
+                    f"[dim]To create projects: "
+                    f"{__import__('flow.links', fromlist=['WebLinks']).WebLinks.projects_settings()}[/dim]"
+                )
             )
 
             project = Prompt.ask("\nProject name", default="default")
@@ -355,7 +408,9 @@ class ProjectSetup:
             options=projects,
             name_key="name",
             id_key="fid",
-            title="Available projects",
+            title="Available project",
+            breadcrumbs=["Flow Setup", "Configuration", "Project"],
+            preferred_viewport_size=5,
         )
 
         if selected_project:
@@ -363,7 +418,7 @@ class ProjectSetup:
 
         return None
 
-    def _fetch_projects(self, api_key: str) -> List[Dict[str, Any]]:
+    def _fetch_projects(self, api_key: str) -> list[dict[str, Any]]:
         """Fetch user's projects from API.
 
         Args:
@@ -374,13 +429,14 @@ class ProjectSetup:
         """
         try:
             client = HttpClient(
-                base_url=os.environ.get("FLOW_API_URL", "https://api.mithril.ai"),
+                base_url=os.environ.get("MITHRIL_API_URL", os.environ.get("FLOW_API_URL", "https://api.mithril.ai")),
                 headers={"Authorization": f"Bearer {api_key}"},
             )
 
             response = client.request(
                 method="GET",
                 url="/v2/projects",
+                timeout_seconds=10.0,
             )
 
             return response if isinstance(response, list) else []
@@ -401,7 +457,7 @@ class SshKeySetup:
         self.console = console
         self.validator = ConfigValidator()
 
-    def setup(self, api_key: str, project: Optional[str] = None) -> Optional[str]:
+    def setup(self, api_key: str, project: str | None = None) -> str | None:
         """Configure SSH keys with platform integration.
 
         Args:
@@ -413,18 +469,17 @@ class SshKeySetup:
         """
         self.console.print(
             """
-SSH keys in Flow allow you to access your GPU instances via SSH.
+SSH keys let you connect to GPU instances via SSH.
 
-[bold]Important:[/bold] Flow uses SSH keys managed in your Mithril account.
-You need to upload your SSH public key to the platform first.
+[bold]Important:[/bold] Flow uses SSH keys from your Mithril account. Upload your
+SSH public key before selecting it here.
 
-[bold]How it works:[/bold]
-1. Upload your public key at: https://app.mithril.ai/account/sshkeys
-2. The platform assigns it an ID like 'sshkey_ABC123'
+Steps:
+1. Upload a public key: {__import__('flow.links', fromlist=['WebLinks']).WebLinks.ssh_keys()}
+2. Copy its ID (e.g., sshkey_ABC123)
 3. Use that ID in your Flow configuration
-4. Flow automatically provisions this key on your instances
 
-[dim]Note: SSH access is optional. You can configure this later if needed.[/dim]
+[dim]SSH access is optional; you can configure it later.[/dim]
         """
         )
 
@@ -464,7 +519,9 @@ You need to upload your SSH public key to the platform first.
                 options=enhanced_keys,
                 name_key="display_name",
                 id_key="fid",
-                title="Available SSH keys",
+                title="Available SSH key",
+                breadcrumbs=["Flow Setup", "Configuration", "Default SSH Key"],
+                preferred_viewport_size=5,
             )
 
             if selected_key:
@@ -492,16 +549,21 @@ You need to upload your SSH public key to the platform first.
 
         # Guide user through upload
         self.console.print("\n[bold]To use SSH keys:[/bold]")
-        self.console.print("1. Go to: https://app.mithril.ai/ssh-keys")
-        self.console.print("2. Click 'Add SSH Key'")
-        if local_keys:
-            self.console.print(f"3. Upload your public key (e.g., {local_keys[0]})")
-        else:
-            self.console.print("3. Upload your SSH public key")
-        self.console.print("4. Copy the assigned key ID")
+        self.console.print(
+            (
+                "1. Upload your public key at: [link]"
+                f"{__import__('flow.links', fromlist=['WebLinks']).WebLinks.ssh_keys()}[/link]"
+            )
+        )
+        self.console.print(
+            "   Or upload locally via CLI: [accent]flow ssh-keys upload ~/.ssh/id_ed25519.pub[/accent]"
+        )
+        self.console.print("   (Private keys are never uploaded; Flow only reads your .pub)")
+        self.console.print("2. Copy the assigned key ID (sshkey_…)")
 
         if Confirm.ask("\nOpen SSH key management page?", default=True):
-            webbrowser.open("https://app.mithril.ai/ssh-keys")
+            from flow.links import WebLinks
+            webbrowser.open(WebLinks.ssh_keys())
             time.sleep(2)
 
         # Manual entry
@@ -519,7 +581,7 @@ You need to upload your SSH public key to the platform first.
 
         return None
 
-    def _fetch_ssh_keys(self, api_key: str, project: Optional[str] = None) -> List[Dict[str, Any]]:
+    def _fetch_ssh_keys(self, api_key: str, project: str | None = None) -> list[dict[str, Any]]:
         """Fetch user's SSH keys from API.
 
         Args:
@@ -535,13 +597,13 @@ You need to upload your SSH public key to the platform first.
                 return []
 
             client = HttpClient(
-                base_url=os.environ.get("FLOW_API_URL", "https://api.mithril.ai"),
+                base_url=os.environ.get("MITHRIL_API_URL", os.environ.get("FLOW_API_URL", "https://api.mithril.ai")),
                 headers={"Authorization": f"Bearer {api_key}"},
             )
 
             # Get all projects to find the ID for our project name
             # This is Mithril-specific: the /v2/projects endpoint and fid field
-            projects = client.request("GET", "/v2/projects")
+            projects = client.request("GET", "/v2/projects", timeout_seconds=10.0)
             project_id = None
 
             for proj in projects:
@@ -559,6 +621,7 @@ You need to upload your SSH public key to the platform first.
                 method="GET",
                 url="/v2/ssh-keys",
                 params=params,
+                timeout_seconds=10.0,
             )
 
             return response if isinstance(response, list) else []
@@ -566,7 +629,7 @@ You need to upload your SSH public key to the platform first.
         except Exception:
             return []
 
-    def _find_local_ssh_keys(self) -> List[Path]:
+    def _find_local_ssh_keys(self) -> list[Path]:
         """Find SSH public keys in user's .ssh directory.
 
         Returns:
