@@ -3,34 +3,30 @@
 Focused GPU resource display for rapid allocation assessment.
 """
 
-from datetime import datetime, timezone
-from typing import List, Optional, Tuple
-import time
 import sys
 import termios
+import time
 import tty
+from datetime import datetime, timezone
 
 import click
+from rich import box
+from rich.align import Align
 from rich.console import Console, Group, RenderableType
 from rich.live import Live
 from rich.panel import Panel
 from rich.table import Table
 from rich.text import Text
-from rich import box
-from rich.align import Align
-from rich.layout import Layout
-from rich.columns import Columns
 
 from flow import Flow
-from flow.api.models import Task, TaskStatus
+from flow.api.models import Task
+from flow.cli.commands.base import BaseCommand, console
+from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
 from flow.errors import AuthenticationError
 
-from .base import BaseCommand, console
-from ..utils.animated_progress import AnimatedEllipsisProgress
-
 
 class BeautifulTaskRenderer:
-    """Renders GPU task allocations with hierarchical organization.
+    """Render GPU task allocations with hierarchical organization.
 
     Implements a state-based rendering system that groups tasks by execution
     status and provides progressive detail levels. Optimized for terminal
@@ -70,7 +66,7 @@ class BeautifulTaskRenderer:
         self.show_details = False
         self.selected_task = None
 
-    def render_interactive_view(self, tasks: List[Task]) -> RenderableType:
+    def render_interactive_view(self, tasks: list[Task]) -> RenderableType:
         """Render task list with keyboard navigation support.
 
         Args:
@@ -96,7 +92,7 @@ class BeautifulTaskRenderer:
         else:
             return main_panel
 
-    def render_allocation_view(self, tasks: List[Task]) -> Panel:
+    def render_allocation_view(self, tasks: list[Task]) -> Panel:
         """Render hierarchical task display grouped by execution state.
 
         Tasks are organized into three primary groups: active (running/starting),
@@ -130,6 +126,15 @@ class BeautifulTaskRenderer:
         groups = []
         task_index = 0
 
+        # Upcoming reservation hint (top-of-view)
+        try:
+            hint = self._format_next_reservation_hint(tasks)
+            if hint:
+                groups.append(hint)
+                groups.append(Text(""))
+        except Exception:
+            pass
+
         # Active tasks section
         if active_tasks:
             active_group = self._render_task_group(
@@ -164,7 +169,7 @@ class BeautifulTaskRenderer:
         # Combine all groups
         content = Group(*groups) if groups else Text("Initializing...", style="dim")
 
-        # Construct final panel with rounded borders
+        # Construct final panel with rounded borders (do not stretch to full width)
         return Panel(
             Align.center(content, vertical="middle"),
             title="[bold]GPU Resources[/bold]",
@@ -172,10 +177,46 @@ class BeautifulTaskRenderer:
             border_style="bright_black",
             box=box.ROUNDED,
             padding=(1, 3),
-            expand=True,
+            expand=False,
         )
 
-    def _render_task_list_interactive(self, tasks: List[Task]) -> Panel:
+    def _format_next_reservation_hint(self, tasks: list[Task]) -> Text | None:
+        """Show the nearest upcoming reservation window with countdown.
+
+        Minimal, non-intrusive hint to increase awareness of scheduled capacity.
+        """
+        from datetime import datetime, timezone
+
+        upcoming = []
+        for t in tasks:
+            try:
+                meta = getattr(t, "provider_metadata", {}) or {}
+                res = meta.get("reservation")
+                if not res:
+                    continue
+                start = res.get("start_time") or res.get("start_time_utc")
+                if not start:
+                    continue
+                # Normalize ISO
+                s = str(start).replace("Z", "+00:00")
+                dt = datetime.fromisoformat(s)
+                if dt > datetime.now(timezone.utc):
+                    upcoming.append((dt, t))
+            except Exception:
+                continue
+        if not upcoming:
+            return None
+        dt, t = sorted(upcoming, key=lambda x: x[0])[0]
+        remaining = int((dt - datetime.now(timezone.utc)).total_seconds())
+        minutes = max(0, remaining // 60)
+        name = t.name or t.task_id
+        txt = Text(justify="center")
+        txt.append("Reserved window: ")
+        txt.append(name, style="bold")
+        txt.append(f" starts in {minutes} min", style="dim")
+        return txt
+
+    def _render_task_list_interactive(self, tasks: list[Task]) -> Panel:
         """Render interactive task list with selection indicator.
 
         Args:
@@ -200,7 +241,7 @@ class BeautifulTaskRenderer:
             # Row selection state
             if idx == self.selected_index:
                 selector = "▶"
-                row_style = "bold cyan"
+                row_style = "bold accent"
             else:
                 selector = " "
                 row_style = None
@@ -226,11 +267,14 @@ class BeautifulTaskRenderer:
 
         content = Group(table, help_text)
 
+        # Resolve accent color for border dynamically
+        from flow.cli.utils.theme_manager import theme_manager as _tm
+        border = _tm.get_color("accent") if self.selected_index >= 0 else "bright_black"
         return Panel(
             content,
             title="[bold]GPU Resources[/bold] - Interactive Mode",
             title_align="center",
-            border_style="cyan" if self.selected_index >= 0 else "bright_black",
+            border_style=border,
             box=box.ROUNDED,
             padding=(1, 2),
         )
@@ -256,7 +300,7 @@ class BeautifulTaskRenderer:
 
         # Hardware resource allocation
         if task.instance_type:
-            from ..utils.gpu_formatter import GPUFormatter
+            from flow.cli.utils.gpu_formatter import GPUFormatter
 
             gpu_display = GPUFormatter.format_ultra_compact(
                 task.instance_type, getattr(task, "num_instances", 1)
@@ -271,7 +315,7 @@ class BeautifulTaskRenderer:
 
         # SSH connection parameters for active tasks
         if task.status.value == "running" and task.ssh_host:
-            lines.append("[bold cyan]Connection:[/bold cyan]")
+            lines.append("[bold accent]Connection:[/bold accent]")
             lines.append(f"  IP: {task.ssh_host}")
             if task.ssh_port:
                 lines.append(f"  Port: {task.ssh_port}")
@@ -303,7 +347,7 @@ class BeautifulTaskRenderer:
 
     def _render_task_group(
         self,
-        tasks: List[Task],
+        tasks: list[Task],
         title: str,
         show_details: bool = True,
         dim: bool = False,
@@ -366,7 +410,7 @@ class BeautifulTaskRenderer:
                 gpu_info = self._format_gpu_elegant(
                     task.instance_type, getattr(task, "num_instances", 1)
                 )
-                row.append(f"[dim cyan]{gpu_info}[/dim cyan]")
+                row.append(f"[dim accent]{gpu_info}[/dim accent]")
 
                 # Duration formatting
                 time_info = self._format_time_elegant(task)
@@ -392,7 +436,7 @@ class BeautifulTaskRenderer:
             "",
             "[dim]No active allocations[/dim]",
             "",
-            f"[dim cyan]{gradient_chars[frame]}  {gradient_chars[frame]}  {gradient_chars[frame]}[/dim cyan]",
+            f"[dim accent]{gradient_chars[frame]}  {gradient_chars[frame]}  {gradient_chars[frame]}[/dim accent]",
             "",
             "[white]flow run task.yaml[/white]",
             "[dim]or[/dim]",
@@ -419,7 +463,7 @@ class BeautifulTaskRenderer:
             return "starting"
         return task.status.value
 
-    def _format_gpu_elegant(self, instance_type: Optional[str], num_instances: int = 1) -> str:
+    def _format_gpu_elegant(self, instance_type: str | None, num_instances: int = 1) -> str:
         """Format GPU instance type for compact display.
 
         For multi-node clusters, shows total GPU count (e.g., '16×H100' for
@@ -433,7 +477,7 @@ class BeautifulTaskRenderer:
         Returns:
             str: Formatted instance specification showing total GPUs.
         """
-        from ..utils.gpu_formatter import GPUFormatter
+        from flow.cli.utils.gpu_formatter import GPUFormatter
 
         return GPUFormatter.format_ultra_compact(instance_type, num_instances)
 
@@ -480,7 +524,7 @@ class BeautifulTaskRenderer:
         """
         return dt.strftime("%Y-%m-%d %H:%M:%S UTC")
 
-    def _format_duration_detailed(self, task: Task) -> Optional[str]:
+    def _format_duration_detailed(self, task: Task) -> str | None:
         """Calculate and format task execution duration.
 
         Computes duration from creation to completion (or current time if running).
@@ -588,7 +632,7 @@ class AllocCommand(BaseCommand):
             type=float,
             help="Update interval in seconds (default: 2.0)",
         )
-        def alloc(watch: bool, gpus: Optional[int], gpu_type: Optional[str], refresh_rate: float):
+        def alloc(watch: bool, gpus: int | None, gpu_type: str | None, refresh_rate: float):
             """Streamlined GPU allocation view - abbreviated version of `flow status`.
 
             Focused display optimized for rapid resource assessment.
@@ -608,8 +652,8 @@ class AllocCommand(BaseCommand):
         self,
         interactive: bool,
         watch: bool,
-        gpus: Optional[int],
-        gpu_type: Optional[str],
+        gpus: int | None,
+        gpu_type: str | None,
         refresh_rate: float,
     ) -> None:
         """Execute allocation command in specified mode.
@@ -643,7 +687,7 @@ class AllocCommand(BaseCommand):
                 )
 
                 with progress:
-                    from ..utils.task_fetcher import TaskFetcher
+                    from flow.cli.utils.task_fetcher import TaskFetcher
 
                     fetcher = TaskFetcher(flow_client)
                     tasks = fetcher.fetch_for_display(show_all=False, limit=30)
@@ -652,6 +696,28 @@ class AllocCommand(BaseCommand):
 
                 console.print(panel)
 
+                # Context-aware next steps
+                try:
+                    has_active = any(
+                        getattr(t, "status", None) and getattr(t.status, "value", str(t.status)) in ("running", "starting")
+                        for t in tasks
+                    )
+                    actions: list[str] = []
+                    if has_active:
+                        actions = [
+                            "Inspect tasks: [accent]flow status[/accent]",
+                            "SSH into a task: [accent]flow ssh :1[/accent]",
+                            "Stream logs: [accent]flow logs :1 -f[/accent]",
+                        ]
+                    else:
+                        actions = [
+                            "Submit a job: [accent]flow run task.yaml[/accent]",
+                            "Start a dev VM: [accent]flow dev[/accent]",
+                        ]
+                    self.show_next_actions(actions)
+                except Exception:
+                    pass
+
         except AuthenticationError:
             self.handle_auth_error()
         except Exception as e:
@@ -671,7 +737,7 @@ class AllocCommand(BaseCommand):
             console.print("[red]Error:[/red] Interactive mode requires a terminal")
             return
 
-        from ..utils.task_fetcher import TaskFetcher
+        from flow.cli.utils.task_fetcher import TaskFetcher
 
         fetcher = TaskFetcher(flow_client)
 
@@ -747,7 +813,9 @@ class AllocCommand(BaseCommand):
         except KeyboardInterrupt:
             console.print("\n[dim]Interrupted[/dim]")
         except Exception as e:
-            console.print(f"[red]Error:[/red] {e}")
+            from rich.markup import escape
+
+            console.print(f"[red]Error:[/red] {escape(str(e))}")
 
     def _run_watch_mode(self, flow_client: Flow, refresh_rate: float) -> None:
         """Execute continuous monitoring mode with periodic updates.
@@ -764,7 +832,7 @@ class AllocCommand(BaseCommand):
             console.print("[red]Error:[/red] Watch mode requires an interactive terminal")
             return
 
-        from ..utils.task_fetcher import TaskFetcher
+        from flow.cli.utils.task_fetcher import TaskFetcher
 
         fetcher = TaskFetcher(flow_client)
 
@@ -779,7 +847,9 @@ class AllocCommand(BaseCommand):
                 self.renderer.advance_animation()
                 return self.renderer.render_allocation_view(tasks)
             except Exception as e:
-                return Panel(f"[red]Error: {e}[/red]", border_style="red")
+                from rich.markup import escape
+
+                return Panel(f"[red]Error: {escape(str(e))}[/red]", border_style="red")
 
         try:
             # Initial data fetch with progress indicator
