@@ -25,39 +25,42 @@ Examples:
 
 from __future__ import annotations
 
+import json
+import logging
+import os
 import re
 import shlex
 import socket
-from urllib.parse import urlparse, urlunparse, parse_qs
-from pathlib import Path
-import json
-import os
 from dataclasses import dataclass
-import logging
-from typing import Optional
+from pathlib import Path
+from urllib.parse import parse_qs, urlparse, urlunparse
 
 import click
 
 from flow import Flow
 from flow.api.models import Task, TaskStatus
+from flow.cli.commands.base import BaseCommand, console
+from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
+from flow.cli.utils.shell_completion import complete_task_ids
+from flow.cli.utils.step_progress import (
+    AllocationProgressAdapter,
+    SSHWaitProgressAdapter,
+    StepTimeline,
+)
+from flow.cli.utils.task_formatter import TaskFormatter
+from flow.cli.utils.task_resolver import resolve_task_identifier
 from flow.errors import AuthenticationError, FlowError
 
-from .base import BaseCommand, console
-from ..utils.animated_progress import AnimatedEllipsisProgress
-from ..utils.shell_completion import complete_task_ids
-from ..utils.task_resolver import resolve_task_identifier
-from ..utils.task_formatter import TaskFormatter
-
-
 # Lazy import to avoid heavy deps at CLI import time
 
+
 def _get_colab_integration(flow_client: Flow):
     from flow._internal.integrations.google_colab import GoogleColabIntegration
 
     return GoogleColabIntegration(flow_client)
 
 
-def _extract_token_from_logs(flow_client: Flow, task_id: str) -> Optional[str]:
+def _extract_token_from_logs(flow_client: Flow, task_id: str) -> str | None:
     """Legacy fallback: try to locate token from any printed URL or token=… snippet.
 
     Avoids printing tokens; use only when metadata/control channel isn't available.
@@ -81,7 +84,9 @@ def _extract_token_from_logs(flow_client: Flow, task_id: str) -> Optional[str]:
     return m2.group(1) if m2 else None
 
 
-def _build_ssh_tunnel_command(task: Task, local_port: int = 8888, remote_port: int = 8888) -> Optional[str]:
+def _build_ssh_tunnel_command(
+    task: Task, local_port: int = 8888, remote_port: int = 8888
+) -> str | None:
     if not task.ssh_host or not task.ssh_user:
         return None
     return (
@@ -129,6 +134,7 @@ def _load_local_store() -> dict:
 def _save_local_store(data: dict) -> None:
     try:
         import os
+
         _LOCAL_STORE_PATH.parent.mkdir(parents=True, exist_ok=True)
         fd = os.open(_LOCAL_STORE_PATH, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0o600)
         with os.fdopen(fd, "w") as f:
@@ -147,17 +153,45 @@ def _persist_connection(task_id: str, token: str, base_url: str, remote_port: in
     _save_local_store(store)
 
 
-def _get_persisted(task_id: str) -> Optional[dict]:
+def _get_persisted(task_id: str) -> dict | None:
     return _load_local_store().get(task_id)
 
 
-def _fetch_token_via_remote(flow_client: Flow, task_id: str) -> Optional[str]:
+def _gc_local_store(flow_client: Flow) -> None:
+    """Garbage-collect stale entries from the local colab store.
+
+    Removes tokens for tasks that are no longer running/pending or not found.
+    """
+    store = _load_local_store()
+    if not store:
+        return
+    changed = False
+    for task_id in list(store.keys()):
+        try:
+            task = flow_client.get_task(task_id)
+            if task.status not in {TaskStatus.RUNNING, TaskStatus.PENDING}:
+                del store[task_id]
+                changed = True
+        except Exception:
+            # Not found or inaccessible – remove
+            del store[task_id]
+            changed = True
+    if changed:
+        _save_local_store(store)
+
+
+def _fetch_token_via_remote(flow_client: Flow, task_id: str) -> str | None:
     try:
         remote_ops = flow_client.get_remote_operations()
-        cmd = "awk -F\"'\" '/^c.NotebookApp.token/ {print $2}' ~/.jupyter/jupyter_notebook_config.py"
+        cmd = (
+            "awk -F\"'\" '/^c.NotebookApp.token/ {print $2}' ~/.jupyter/jupyter_notebook_config.py"
+        )
         token_output = remote_ops.execute_command(task_id, cmd)
         token = (token_output or "").strip()
         return token or None
+    except (AttributeError, NotImplementedError):
+        # No remote access in this provider (e.g., demo/mock)
+        return None
     except Exception:
         return None
 
@@ -165,8 +199,8 @@ def _fetch_token_via_remote(flow_client: Flow, task_id: str) -> Optional[str]:
 @dataclass
 class ColabSessionInfo:
     task: Task
-    url: Optional[str]
-    tunnel_cmd: Optional[str]
+    url: str | None
+    tunnel_cmd: str | None
 
 
 class ColabCommand(BaseCommand):
@@ -186,12 +220,18 @@ class ColabCommand(BaseCommand):
 
     def get_command(self) -> click.Group:
         @click.group(name=self.name, help=self.help)
-        @click.option("--verbose", "verbose", is_flag=True, help="Show integration details and tips")
+        @click.option(
+            "--verbose", "verbose", is_flag=True, help="Show integration details and tips"
+        )
         def colab(verbose: bool):
             if verbose:
                 console.print("\n[bold]Colab Local Runtime with Flow[/bold]\n")
-                console.print("This connects Colab's UI to a Jupyter server running on your GPU instance.")
-                console.print("Security: server binds to 127.0.0.1 on the instance; you tunnel via SSH.\n")
+                console.print(
+                    "This connects Colab's UI to a Jupyter server running on your GPU instance."
+                )
+                console.print(
+                    "Security: server binds to 127.0.0.1 on the instance; you tunnel via SSH.\n"
+                )
 
         colab.add_command(self._up())
         colab.add_command(self._list())
@@ -238,13 +278,13 @@ class ColabCommand(BaseCommand):
             help="Optional workspace volume name (reuse to persist across sessions)",
         )
         def up(
-            instance_type: Optional[str],
-            hours: Optional[float],
-            name: Optional[str],
+            instance_type: str | None,
+            hours: float | None,
+            name: str | None,
             local_port: int,
             workspace: bool,
             workspace_size: int,
-            workspace_name: Optional[str],
+            workspace_name: str | None,
         ):
             """Launch a Colab-ready instance and print connection details."""
             try:
@@ -262,12 +302,14 @@ class ColabCommand(BaseCommand):
                         or "h100"
                     )
                 # Launch via internal integration (Jupyter on host)
-                # DRY with dev: show the same provisioning UX
+                # Use the same provisioning UX as dev
                 integration = _get_colab_integration(flow_client)
                 # Submit and then wait with shared progress UI
                 from flow import TaskConfig
+
                 cfg = TaskConfig(
                     name=name or f"colab-{instance_type}",
+                    unique_name=False,
                     instance_type=instance_type,
                     command=["bash", "-c", integration.JUPYTER_STARTUP_SCRIPT],
                     upload_code=False,
@@ -282,43 +324,91 @@ class ColabCommand(BaseCommand):
                         workspace_name or f"colab-ws-{instance_type}"
                     )
                     from flow.api.models import VolumeSpec
+
                     cfg.volumes = [
                         VolumeSpec(name=vol_name, size_gb=workspace_size, mount_path="/workspace")
                     ]
 
+                # Pre-select a region to keep volumes and task in the same region
+                # This avoids creating a volume in a region different from where the task runs
+                try:
+                    instances = flow_client.find_instances(
+                        {"instance_type": instance_type}, limit=20
+                    )
+                    if instances:
+                        # Prefer highest availability, then lowest price
+                        def _avail(i):
+                            return i.available_quantity if i.available_quantity is not None else 0
+
+                        instances.sort(key=lambda i: (-_avail(i), i.price_per_hour))
+                        cfg.region = instances[0].region
+                except Exception:
+                    # If pre-selection fails, let provider choose and fallback logic will handle mismatch
+                    pass
+
                 # Temporarily silence noisy 5xx retry warnings during provisioning
                 http_logger = logging.getLogger("flow._internal.io.http")
                 prev_level = http_logger.level
                 http_logger.setLevel(logging.ERROR)
                 try:
-                    # Brief submission spinner to avoid dead air before allocation progress
-                    with AnimatedEllipsisProgress(
-                        console,
-                        message=f"Submitting {instance_type} request",
-                        start_immediately=True,
-                        transient=True,
-                        show_progress_bar=False,
-                    ):
-                        task = flow_client.run(cfg)
-
-                    # Wait with the same animated UX used elsewhere
-                    from .utils import wait_for_task
-                    final_status = wait_for_task(
-                        flow_client, task.task_id, watch=False, task_name=task.name
+                    # Unified timeline for submission → allocation → SSH
+                    timeline = StepTimeline(console, title="flow colab", title_animation="auto")
+                    timeline.start()
+                    submit_idx = timeline.add_step(
+                        f"Submitting {instance_type} request", show_bar=False
                     )
+                    timeline.start_step(submit_idx)
+                    task = flow_client.run(cfg)
+                    timeline.complete_step()
+
+                    # Allocation
+                    from flow.cli.commands.utils import wait_for_task
+
+                    alloc_idx = timeline.add_step(
+                        "Allocating instance", show_bar=True, estimated_seconds=120
+                    )
+                    alloc = AllocationProgressAdapter(timeline, alloc_idx, estimated_seconds=120)
+                    with alloc:
+                        final_status = wait_for_task(
+                            flow_client,
+                            task.task_id,
+                            watch=False,
+                            task_name=task.name,
+                            progress_adapter=alloc,
+                        )
                     if final_status != "running":
+                        timeline.fail_step(f"Status: {final_status}")
+                        timeline.finish()
                         raise FlowError(f"Failed to start instance (status: {final_status})")
 
-                    # After running, wait for SSH readiness with the same progress UX as dev
+                    # SSH readiness
+                    from flow.api.ssh_utils import DEFAULT_PROVISION_MINUTES
+
+                    baseline = 0
                     try:
-                        from flow.api.ssh_utils import DEFAULT_PROVISION_MINUTES
+                        baseline = int(getattr(task, "instance_age_seconds", None) or 0)
+                    except Exception:
+                        baseline = 0
+                    ssh_idx = timeline.add_step(
+                        "Provisioning & SSH",
+                        show_bar=True,
+                        estimated_seconds=DEFAULT_PROVISION_MINUTES * 60,
+                        baseline_elapsed_seconds=baseline,
+                    )
+                    ssh = SSHWaitProgressAdapter(
+                        timeline,
+                        ssh_idx,
+                        DEFAULT_PROVISION_MINUTES * 60,
+                        baseline_elapsed_seconds=baseline,
+                    )
+                    with ssh:
                         task = flow_client.wait_for_ssh(
                             task_id=task.task_id,
                             timeout=DEFAULT_PROVISION_MINUTES * 60 * 2,
-                            show_progress=True,
+                            show_progress=False,
+                            progress_adapter=ssh,
                         )
-                    except Exception as e:
-                        raise FlowError(f"SSH not ready: {e}")
+                    timeline.finish()
                 except Exception as submit_err:
                     # If workspace volume creation fails, retry without workspace
                     if workspace and getattr(cfg, "volumes", None):
@@ -328,13 +418,14 @@ class ColabCommand(BaseCommand):
                             # Show a quick retry submission spinner as well
                             with AnimatedEllipsisProgress(
                                 console,
-                                message=f"Retrying submission (no workspace)",
+                                message="Retrying submission (no workspace)",
                                 start_immediately=True,
                                 transient=True,
                                 show_progress_bar=False,
                             ):
                                 task = flow_client.run(cfg)
-                            from .utils import wait_for_task
+                            from flow.cli.commands.utils import wait_for_task
+
                             final_status = wait_for_task(
                                 flow_client, task.task_id, watch=False, task_name=task.name
                             )
@@ -343,6 +434,7 @@ class ColabCommand(BaseCommand):
                                     f"Failed to start instance (status: {final_status})"
                                 )
                             from flow.api.ssh_utils import DEFAULT_PROVISION_MINUTES
+
                             task = flow_client.wait_for_ssh(
                                 task_id=task.task_id,
                                 timeout=DEFAULT_PROVISION_MINUTES * 60 * 2,
@@ -360,18 +452,10 @@ class ColabCommand(BaseCommand):
                 finally:
                     http_logger.setLevel(prev_level)
 
-                # Once SSH is ready, use the same short spinner style as dev
-                # to keep UX consistent
-                with AnimatedEllipsisProgress(
-                    console,
-                    message="Initializing SSH connection",
-                    start_immediately=True,
-                    transient=True,
-                    show_progress_bar=False,
-                ):
-                    connection = integration._wait_for_instance_ready(
-                        task, session_id=f"colab-{task.task_id[:6]}", quiet=True
-                    )
+                # Once SSH is ready, connect
+                connection = integration._wait_for_instance_ready(
+                    task, session_id=f"colab-{task.task_id[:6]}", quiet=True
+                )
 
                 # Fetch task to build SSH cmd
                 task = flow_client.get_task(connection.task_id)
@@ -379,14 +463,18 @@ class ColabCommand(BaseCommand):
                 if local_port == 0:
                     local_port = _pick_free_port()
                 tunnel_cmd = _build_ssh_tunnel_command(
-                    task, local_port=local_port, remote_port=getattr(connection, "remote_port", 8888)
+                    task,
+                    local_port=local_port,
+                    remote_port=getattr(connection, "remote_port", 8888),
                 )
 
                 console.print("\n[green]✓[/green] Instance ready for Colab")
                 if tunnel_cmd:
                     console.print("\n[bold]SSH tunnel (run on your laptop):[/bold]")
                     console.print(f"  {tunnel_cmd}")
-                console.print("\n[bold]Colab connection URL (paste in Colab → Connect to local runtime…):[/bold]")
+                console.print(
+                    "\n[bold]Colab connection URL (paste in Colab → Connect to local runtime…):[/bold]"
+                )
                 # Persist connection details locally for future url/tunnel retrieval without logs
                 try:
                     # Try to extract token from connection URL query param
@@ -409,9 +497,13 @@ class ColabCommand(BaseCommand):
 
                 # Workspace guidance
                 if workspace:
-                    console.print("\n[dim]Workspace mounted at /workspace. Notebooks will persist at /workspace/notebooks.[/dim]")
+                    console.print(
+                        "\n[dim]Workspace mounted at /workspace. Notebooks will persist at /workspace/notebooks.[/dim]"
+                    )
                 else:
-                    console.print("\n[yellow]No workspace volume attached. Files will be ephemeral unless you download or push them.[/yellow]")
+                    console.print(
+                        "\n[yellow]No workspace volume attached. Files will be ephemeral unless you download or push them.[/yellow]"
+                    )
 
                 # Next steps
                 self.show_next_actions(
@@ -440,6 +532,12 @@ class ColabCommand(BaseCommand):
                 self.handle_auth_error()
                 return
 
+            # Clean up stale local entries quietly
+            try:
+                _gc_local_store(flow_client)
+            except Exception:
+                pass
+
             with AnimatedEllipsisProgress(console, "Fetching tasks"):
                 tasks = flow_client.list_tasks(limit=200)
 
@@ -456,10 +554,16 @@ class ColabCommand(BaseCommand):
                     token = persisted["token"]
                     remote_port = int(persisted.get("remote_port", 8888))
                     url = f"http://localhost:{local_port}/?token={token}"
-                    tunnel_cmd = _build_ssh_tunnel_command(task, local_port=local_port, remote_port=remote_port)
+                    tunnel_cmd = _build_ssh_tunnel_command(
+                        task, local_port=local_port, remote_port=remote_port
+                    )
                 else:
                     # Legacy fallback only if we detect a colab-* name
-                    token = _extract_token_from_logs(flow_client, task.task_id) if is_colab_name else None
+                    token = (
+                        _extract_token_from_logs(flow_client, task.task_id)
+                        if is_colab_name
+                        else None
+                    )
                     url = f"http://localhost:{local_port}/?token={token}" if token else None
                     tunnel_cmd = _build_ssh_tunnel_command(task, local_port=local_port)
                 if is_colab_name or token:
@@ -493,13 +597,19 @@ class ColabCommand(BaseCommand):
                 self.handle_auth_error()
                 return
 
+            # Clean old entries
+            try:
+                _gc_local_store(flow_client)
+            except Exception:
+                pass
+
             task, error = resolve_task_identifier(flow_client, task_identifier)
             if error:
                 console.print(f"[red]Error:[/red] {error}")
                 return
 
             # Prefer local persisted token; fallback to remote fetch; last resort: logs
-            token: Optional[str] = None
+            token: str | None = None
             persisted = _get_persisted(task.task_id)
             if persisted and persisted.get("token"):
                 token = persisted["token"]
@@ -539,7 +649,9 @@ class ColabCommand(BaseCommand):
     def _tunnel(self) -> click.Command:
         @click.command(name="tunnel")
         @click.argument("task_identifier", required=True, shell_complete=complete_task_ids)
-        @click.option("--local-port", type=int, default=8888, show_default=True, help="Local port (0 = auto)")
+        @click.option(
+            "--local-port", type=int, default=8888, show_default=True, help="Local port (0 = auto)"
+        )
         @click.option("--print-only", is_flag=True, help="Only print SSH command; do not execute")
         def tunnel(task_identifier: str, local_port: int, print_only: bool):
             """Start or print the SSH port-forward command (-N -L)."""
@@ -557,14 +669,29 @@ class ColabCommand(BaseCommand):
             # Auto-pick local port if requested
             if local_port == 0:
                 local_port = _pick_free_port()
-            # Try to infer remote port from metadata via integration URL (fallback 8888)
+            # Try to infer remote port from persisted data or by querying remote file
             remote_port = 8888
             try:
-                # If the task has provider metadata with endpoints or similar, that could be used.
-                pass
+                persisted = _get_persisted(task.task_id)
+                if persisted and persisted.get("remote_port"):
+                    remote_port = int(persisted["remote_port"]) or 8888
+                else:
+                    # Query remote port file (best-effort)
+                    try:
+                        remote_ops = flow_client.get_remote_operations()
+                        port_output = remote_ops.execute_command(
+                            task.task_id, "cat ~/.jupyter/colab_port || true"
+                        )
+                        port_str = (port_output or "").strip()
+                        if port_str.isdigit():
+                            remote_port = int(port_str)
+                    except Exception:
+                        pass
             except Exception:
                 pass
-            ssh_cmd = _build_ssh_tunnel_command(task, local_port=local_port, remote_port=remote_port)
+            ssh_cmd = _build_ssh_tunnel_command(
+                task, local_port=local_port, remote_port=remote_port
+            )
             if not ssh_cmd:
                 console.print("SSH not ready yet; wait for provisioning.")
                 return
