@@ -1,8 +1,7 @@
-"""Grab command - Quickly acquire GPU resources for interactive use.
+"""Grab command - quickly acquire GPU resources for interactive use.
 
-This module implements the grab command for the Flow CLI. It provides a
-simplified interface for acquiring GPU resources without the task abstraction,
-perfect for interactive development and exploration.
+Simplified interface for acquiring GPU resources without the full task
+abstraction.
 
 Examples:
     # Grab 8 GPUs of any type
@@ -32,16 +31,18 @@ Note:
     Default duration is 1 hour to prevent accidental long-running allocations.
 """
 
+from __future__ import annotations
+
 import json
 
 import click
 
 from flow import Flow, TaskConfig
+from flow.cli.commands.base import BaseCommand, console
+from flow.cli.commands.utils import wait_for_task
+from flow.cli.utils.step_progress import AllocationProgressAdapter, StepTimeline
 from flow.errors import AuthenticationError, ValidationError
 
-from .base import BaseCommand, console
-from .utils import wait_for_task
-
 
 class GrabCommand(BaseCommand):
     """Quickly grab GPU resources for interactive use."""
@@ -57,6 +58,8 @@ class GrabCommand(BaseCommand):
         Example: flow grab 256 h100"""
 
     def get_command(self) -> click.Command:
+        from flow.cli.utils.mode import demo_aware_command
+
         @click.command(name=self.name, help=self.help)
         @click.argument("count", type=int)
         @click.argument("gpu_type", required=False, default=None)
@@ -126,6 +129,7 @@ class GrabCommand(BaseCommand):
             is_flag=True,
             help="Show detailed usage patterns and pricing info",
         )
+        @demo_aware_command()
         def grab(
             count: int,
             gpu_type: str | None,
@@ -239,17 +243,11 @@ class GrabCommand(BaseCommand):
         output_json: bool,
     ) -> None:
         """Execute the grab command."""
-        # Start progress animation
-        progress = None
+        # Unified timeline
+        timeline: StepTimeline | None = None
         if not output_json:
-            from ..utils.animated_progress import AnimatedEllipsisProgress
-
-            progress = AnimatedEllipsisProgress(
-                console,
-                f"Acquiring {count} GPU{'s' if count != 1 else ''}",
-                start_immediately=True,
-                transient=True,
-            )
+            timeline = StepTimeline(console, title="flow grab", title_animation="auto")
+            timeline.start()
 
         try:
             # Validate inputs
@@ -274,7 +272,7 @@ class GrabCommand(BaseCommand):
 
             # Generate name if not provided
             if not name:
-                from ..utils.name_generator import generate_unique_name
+                from flow.cli.utils.name_generator import generate_unique_name
 
                 name = generate_unique_name(prefix="grab", base_name=None, add_unique=not no_unique)
             # If name is provided, use it as-is
@@ -287,6 +285,8 @@ class GrabCommand(BaseCommand):
                 "num_instances": num_instances,
                 "command": run if run else ["sleep", "infinity"],
                 "image": "nvidia/cuda:12.1.0-runtime-ubuntu22.04",
+                # Grab is an interactive devbox; skip code packaging for speed
+                "upload_code": False,
                 "env": {
                     "FLOW_GRAB": "true",
                     "FLOW_GPU_COUNT": str(count),
@@ -311,7 +311,7 @@ class GrabCommand(BaseCommand):
             # Initialize Flow client and submit
             flow_client = Flow()
 
-            if not output_json:
+            if not output_json and timeline:
                 # Show what we're doing
                 console.print("\n[bold]Configuration:[/bold]")
                 console.print(f"  GPUs: {count}x {gpu_type or 'any'}")
@@ -324,10 +324,12 @@ class GrabCommand(BaseCommand):
                     )
                 console.print()
 
-                # Update progress
-                progress.update_message(f"Provisioning {num_instances}x {instance_type}")
-                with progress:
-                    task = flow_client.run(config)
+                submit_idx = timeline.add_step(
+                    f"Provisioning {num_instances}x {instance_type}", show_bar=False
+                )
+                timeline.start_step(submit_idx)
+                task = flow_client.run(config)
+                timeline.complete_step()
             else:
                 task = flow_client.run(config)
 
@@ -344,16 +346,36 @@ class GrabCommand(BaseCommand):
                 return
 
             # Wait for resources to be ready
-            status = wait_for_task(
-                flow_client, task.task_id, watch=False, json_output=False, task_name=task.name
-            )
+            if timeline:
+                alloc_idx = timeline.add_step(
+                    "Allocating instances", show_bar=True, estimated_seconds=120
+                )
+                alloc = AllocationProgressAdapter(timeline, alloc_idx, estimated_seconds=120)
+                with alloc:
+                    status = wait_for_task(
+                        flow_client,
+                        task.task_id,
+                        watch=False,
+                        json_output=False,
+                        task_name=task.name,
+                        progress_adapter=alloc,
+                    )
+            else:
+                status = wait_for_task(
+                    flow_client, task.task_id, watch=False, json_output=False, task_name=task.name
+                )
 
             if status == "running":
                 # Get updated task info
                 task = flow_client.get_task(task.task_id)
 
-                console.print("\n[green]✓[/green] Resources acquired successfully!")
-                console.print(f"\nCluster: [cyan]{name}[/cyan]")
+                from flow.cli.utils.theme_manager import theme_manager as _tm
+
+                success_color = _tm.get_color("success")
+                console.print(
+                    f"\n[{success_color}]✓[/{success_color}] Resources acquired successfully!"
+                )
+                console.print(f"\nCluster: [accent]{name}[/accent]")
                 console.print(
                     f"GPUs: {count}x {gpu_type or 'any'} ({num_instances} node{'s' if num_instances > 1 else ''})"
                 )
@@ -372,10 +394,10 @@ class GrabCommand(BaseCommand):
                         f"Price limit: ${max_price:.2f}/GPU/hour (${max_price * count:.2f}/hour total)"
                     )
 
-                console.print(f"\nSSH access: [cyan]flow ssh {name}[/cyan]")
+                console.print(f"\nSSH access: [accent]flow ssh {name}[/accent]")
                 if num_instances > 1:
                     console.print(
-                        f"Multi-node access: [cyan]flow ssh {name} --node <0-{num_instances - 1}>[/cyan]"
+                        f"Multi-node access: [accent]flow ssh {name} --node <0-{num_instances - 1}>[/accent]"
                     )
 
                 if total_hours is not None:
@@ -386,7 +408,7 @@ class GrabCommand(BaseCommand):
                     console.print(
                         "\n[yellow]Resources will remain allocated until manually released.[/yellow]"
                     )
-                console.print(f"To release: [cyan]flow release {name}[/cyan]")
+                console.print(f"To release: [accent]flow release {name}[/accent]")
 
                 if run:
                     console.print(f"\n[dim]Running command: {run}[/dim]")
@@ -394,17 +416,32 @@ class GrabCommand(BaseCommand):
                 console.print("\n[red]✗[/red] Failed to acquire resources")
                 console.print("Try adjusting your requirements or check availability")
             else:
-                console.print(f"\n[yellow]![/yellow] Unexpected status: {status}")
+                # Still pending or preparing after soft timeout — continue in background
+                console.print("\n[yellow]⏳ Allocation in progress[/yellow]")
+                console.print(
+                    f"Resources are still {status}. This is normal during bidding/queueing."
+                )
+                console.print(
+                    f"We'll keep provisioning in the background. Check progress with: [accent]flow status {name}[/accent]"
+                )
+                console.print(
+                    f"Connect when ready: [accent]flow ssh {name}[/accent]  |  Release: [accent]flow release {name}[/accent]"
+                )
 
         except AuthenticationError:
             self.handle_auth_error()
         except ValidationError as e:
-            self.handle_error(f"Invalid configuration: {e}")
+            from rich.markup import escape
+
+            self.handle_error(f"Invalid configuration: {escape(str(e))}")
         except Exception as e:
             self.handle_error(str(e))
         finally:
-            if progress and progress._active:
-                progress.__exit__(None, None, None)
+            if timeline:
+                try:
+                    timeline.finish()
+                except Exception:
+                    pass
 
     def _calculate_optimal_config(self, gpu_count: int, gpu_type: str | None) -> tuple[str, int]:
         """Calculate optimal instance type and count for requested GPUs.
