@@ -17,13 +17,15 @@ import asyncio
 import logging
 import os
 from pathlib import Path
-from typing import Optional
 
 import click
 import yaml
+from rich.markup import escape
+
 from flow.cli.commands.base import BaseCommand
-from ..utils.theme_manager import theme_manager
-from flow.cli.utils.config_validator import ConfigValidator, ValidationStatus
+from flow.cli.utils.config_validator import ConfigValidator
+from flow.cli.utils.mask_utils import mask_api_key, mask_config_for_display
+from flow.cli.utils.theme_manager import theme_manager
 
 # Import private components
 from flow.core.setup_registry import SetupRegistry, register_providers
@@ -34,7 +36,7 @@ logger = logging.getLogger(__name__)
 console = theme_manager.create_console()
 
 
-def run_setup_wizard(provider: Optional[str] = None) -> bool:
+def run_setup_wizard(provider: str | None = None) -> bool:
     """Run the setup wizard for the resolved provider.
 
     If provider is None, attempt to detect from existing config or prompt.
@@ -70,7 +72,13 @@ def run_setup_wizard(provider: Optional[str] = None) -> bool:
                     SelectionItem(value=p, id=p, title=p.title(), subtitle="", status="")
                     for p in providers
                 ]
-                selector = InteractiveSelector(options, lambda x: x, title="Select provider")
+                selector = InteractiveSelector(
+                    options,
+                    lambda x: x,
+                    title="Select provider",
+                    breadcrumbs=["Flow Setup", "Provider"],
+                    preferred_viewport_size=5,
+                )
                 choice = selector.select()
                 resolved_provider = choice if isinstance(choice, str) else None
             except Exception:
@@ -89,7 +97,7 @@ def run_setup_wizard(provider: Optional[str] = None) -> bool:
         console.print("\n\n[yellow]Setup cancelled[/yellow]")
         return False
     except Exception as e:
-        console.print(f"\n\n[red]Setup error:[/red] {e}")
+        console.print(f"\n\n[red]Setup error:[/red] {escape(str(e))}")
         logger.exception("Setup wizard error")
         return False
 
@@ -115,8 +123,15 @@ class InitCommand(BaseCommand):
         return "Configure Flow SDK credentials and provider settings"
 
     def get_command(self) -> click.Command:
+        from flow.cli.utils.mode import demo_aware_command
+
         @click.command(name=self.name, help=self.help)
         @click.option("--provider", envvar="FLOW_PROVIDER", help="Provider to use")
+        @click.option(
+            "--demo",
+            is_flag=True,
+            help="Enable demo mode: configure mock provider (no real provisioning)",
+        )
         @click.option("--api-key", help="API key for authentication")
         @click.option("--project", help="Project name")
         @click.option("--region", help="Default region")
@@ -131,14 +146,16 @@ class InitCommand(BaseCommand):
         @click.option("--reset", is_flag=True, help="Reset configuration to start fresh")
         @click.option("--show", is_flag=True, help="Print current resolved configuration and exit")
         @click.option("--yes", is_flag=True, help="Non-interactive; answer yes to prompts (CI)")
+        @demo_aware_command(flag_param="demo")
         def init(
-            provider: Optional[str],
-            api_key: Optional[str],
-            project: Optional[str],
-            region: Optional[str],
-            api_url: Optional[str],
+            provider: str | None,
+            demo: bool,
+            api_key: str | None,
+            project: str | None,
+            region: str | None,
+            api_url: str | None,
             dry_run: bool,
-            output: Optional[str],
+            output: str | None,
             verbose: bool,
             reset: bool,
             show: bool,
@@ -154,11 +171,21 @@ class InitCommand(BaseCommand):
 
             Use 'flow init --verbose' for detailed configuration options.
             """
-            if verbose and not any([provider, api_key, project, region, api_url, dry_run]):
+            if demo:
+                # Force mock provider and clear sensitive fields
+                provider = "mock"
+                api_key = None
+                os.environ["FLOW_DEMO_MODE"] = "1"
+                os.environ["FLOW_PROVIDER"] = "mock"
+
+            if verbose and not any([provider, api_key, project, region, api_url, dry_run, demo]):
                 console.print("\n[bold]Flow SDK Configuration Guide:[/bold]\n")
                 console.print("Interactive setup:")
                 console.print("  flow init                         # Step-by-step wizard")
-                console.print("  flow init --dry-run               # Preview without saving\n")
+                console.print("  flow init --dry-run               # Preview without saving")
+                console.print(
+                    "  flow init --demo                  # Configure mock provider (no real instances)\n"
+                )
 
                 console.print("Direct configuration:")
                 console.print(
@@ -167,10 +194,10 @@ class InitCommand(BaseCommand):
                 console.print("  flow init --region us-west-2      # Set default region")
                 console.print("  flow init --api-url https://custom.api.com  # Custom endpoint\n")
 
-                console.print("Environment variables:")
-                console.print("  export FLOW_API_KEY=fkey_xxx      # Set API key")
-                console.print("  export FLOW_PROJECT=myproject     # Set project")
-                console.print("  export FLOW_REGION=us-west-2      # Set region\n")
+                console.print("Environment variables (canonical):")
+                console.print("  export MITHRIL_API_KEY=fkey_xxx   # Set API key")
+                console.print("  export MITHRIL_PROJECT=myproject  # Set project")
+                console.print("  export MITHRIL_REGION=us-west-2   # Set region\n")
 
                 console.print("Configuration files:")
                 console.print("  ~/.flow/config.yaml               # User configuration")
@@ -187,47 +214,81 @@ class InitCommand(BaseCommand):
             # Handle --show
             if show:
                 try:
-                    from flow._internal.config_loader import ConfigLoader
-
-                    loader = ConfigLoader()
-                    sources = loader.load_all_sources()
+                    from flow._internal.config_manager import ConfigManager
                     import yaml as _yaml
 
-                    console.print(_yaml.safe_dump(sources.as_dict(), default_flow_style=False))
+                    manager = ConfigManager()
+                    sources = manager.load_sources()
+                    # Build a user-facing view and mask sensitive values
+                    show_dict = {
+                        "provider": sources.provider,
+                        "api_key": mask_api_key(sources.api_key),
+                        "mithril": sources.get_mithril_config(),
+                    }
+                    console.print(_yaml.safe_dump(show_dict, default_flow_style=False))
                 except Exception as e:
-                    console.print(f"[red]Error loading configuration:[/red] {e}")
+                    console.print(f"[red]Error loading configuration:[/red] {escape(str(e))}")
                     raise click.exceptions.Exit(1)
                 return
 
-            # Run async function
-            asyncio.run(
-                self._init_async(
-                    provider,
-                    api_key,
-                    project,
-                    region,
-                    api_url,
-                    dry_run,
-                    verbose,
-                    reset,
-                    output_path=output,
-                    assume_yes=yes,
+            # Run async function safely in or out of an existing loop
+            try:
+                loop = asyncio.get_running_loop()
+            except RuntimeError:
+                loop = None
+            if loop and loop.is_running():
+                try:
+                    import nest_asyncio  # type: ignore
+
+                    try:
+                        nest_asyncio.apply()
+                    except Exception:
+                        pass
+                except Exception:
+                    pass
+                loop.run_until_complete(
+                    self._init_async(
+                        provider,
+                        api_key,
+                        project,
+                        region,
+                        api_url,
+                        dry_run,
+                        verbose,
+                        reset,
+                        output_path=output,
+                        assume_yes=yes,
+                    )
+                )
+            else:
+                asyncio.run(
+                    self._init_async(
+                        provider,
+                        api_key,
+                        project,
+                        region,
+                        api_url,
+                        dry_run,
+                        verbose,
+                        reset,
+                        output_path=output,
+                        assume_yes=yes,
+                    )
                 )
-            )
 
         return init
 
     async def _init_async(
         self,
-        provider: Optional[str],
-        api_key: Optional[str],
-        project: Optional[str],
-        region: Optional[str],
-        api_url: Optional[str],
+        provider: str | None,
+        api_key: str | None,
+        project: str | None,
+        region: str | None,
+        api_url: str | None,
         dry_run: bool,
         verbose: bool = False,
         reset: bool = False,
-        output_path: Optional[str] = None,
+        output_path: str | None = None,
         assume_yes: bool = False,
     ):
         """Execute init command.
@@ -244,35 +305,111 @@ class InitCommand(BaseCommand):
         # Handle reset flag first
         if reset:
             if await self._reset_configuration():
-                console.print("[green]✓[/green] Configuration reset successfully")
+                from flow.cli.utils.theme_manager import theme_manager as _tm
+
+                success_color = _tm.get_color("success")
+                console.print(
+                    f"[{success_color}]✓[/{success_color}] Configuration reset successfully"
+                )
                 if not (provider or api_key or project or region or api_url):
                     console.print("\nStarting fresh setup...")
                     console.print("")  # Add blank line before wizard
             else:
                 return  # User cancelled or error occurred
 
-        if provider or api_key or project or region or api_url or dry_run:
+        # In demo mode with provider 'mock' and no explicit options, fast-path to a friendly UX
+        try:
+            from flow.cli.utils.mode import is_demo_active
+
+            demo_on = is_demo_active()
+        except Exception:
+            demo_on = False
+
+        # Honor explicit provider choice: if user set --provider mithril (or any non-mock),
+        # do NOT force demo/mock wizard even if demo env is active.
+        if provider and provider != "mock":
+            demo_on = False
+
+        if (((provider is None) and demo_on) or (provider == "mock")) and not any(
+            [api_key, project, region, api_url, dry_run]
+        ):
+            # Run the full setup wizard for the mock provider so users can reconfigure demo settings
+            ok = run_setup_wizard("mock")
+            if not ok:
+                raise click.exceptions.Exit(1)
+            # Enable shell completion after a successful wizard run
+            self._setup_shell_completion()
+            return True
+
+        # Treat as non-interactive only when actual configuration flags are provided.
+        # Having only a provider value (e.g., from demo mode env default) should not
+        # force non-interactive mode; users expect the interactive wizard in that case.
+        explicit_non_interactive = bool(api_key or project or region or api_url or dry_run)
+
+        if explicit_non_interactive:
             # Non-interactive mode with provided options
             success = await self._configure_with_options(
                 provider, api_key, project, region, api_url, dry_run, output_path
             )
         else:
-            # Interactive mode - use refactored wizard
+            # Interactive mode - offer fast demo mode escape hatch only on first run
             if assume_yes:
                 console.print(
                     "[red]Error:[/red] --yes requires non-interactive options. Provide --provider and related flags."
                 )
                 return False
+            # If provider not specified, offer demo mode only on the very first init run
+            # (i.e., when no user config file exists yet). Subsequent runs bypass this confirmation.
+            from pathlib import Path as _Path
+
+            first_run = not (_Path.home() / ".flow" / "config.yaml").exists()
+            if not provider and first_run:
+                try:
+                    from rich.prompt import Confirm
+
+                    use_demo = Confirm.ask(
+                        "Start in demo mode (no real provisioning, mock provider)?",
+                        default=True,
+                    )
+                    if use_demo:
+                        # Persist provider: mock and return
+                        try:
+                            from flow._internal.config_manager import ConfigManager
+
+                            manager = ConfigManager()
+                            manager.save({"provider": "mock"})
+                            from flow.cli.utils.theme_manager import theme_manager as _tm0
+
+                            success_color = _tm0.get_color("success")
+                            console.print(
+                                f"[{success_color}]✓[/{success_color}] Demo mode enabled (provider: mock)."
+                            )
+                            console.print(
+                                "[dim]Tip:[/dim] Run with --no-demo or re-run 'flow init' to configure real Mithril integration."
+                            )
+                            self.show_next_actions(
+                                [
+                                    "Explore safely: [accent]flow tutorial[/accent]",
+                                    "Submit a demo run: [accent]flow run --demo 'nvidia-smi' -i h100[/accent]",
+                                    "Switch to real: [accent]flow init --provider mithril[/accent]",
+                                ]
+                            )
+                            return True
+                        except Exception:
+                            # Fallback to wizard if writing fails
+                            pass
+                except Exception:
+                    pass
             # Pass provider if given; otherwise let wizard resolve
             success = run_setup_wizard(provider)
             if success:
-                # Show next actions after successful setup
-                console.print("")  # Add blank line for spacing
+                # The wizard already displays a provider-specific completion panel.
+                # Show only next steps here to avoid duplicate success banners.
                 self.show_next_actions(
                     [
-                        "Run an example: [cyan]flow example minimal[/cyan]",
-                        "View all examples: [cyan]flow example[/cyan]",
-                        "Submit your first task: [cyan]flow run task.yaml[/cyan]",
+                        "Run your first GPU test: [accent]flow example gpu-test[/accent]",
+                        "Submit a job: [accent]flow run examples/configs/basic.yaml[/accent]",
+                        "Monitor tasks: [accent]flow status[/accent]",
                     ]
                 )
 
@@ -285,13 +422,13 @@ class InitCommand(BaseCommand):
 
     async def _configure_with_options(
         self,
-        provider: Optional[str],
-        api_key: Optional[str],
-        project: Optional[str],
-        region: Optional[str],
-        api_url: Optional[str],
+        provider: str | None,
+        api_key: str | None,
+        project: str | None,
+        region: str | None,
+        api_url: str | None,
         dry_run: bool,
-        output_path: Optional[str],
+        output_path: str | None,
     ) -> bool:
         """Configure using command-line options.
 
@@ -304,14 +441,24 @@ class InitCommand(BaseCommand):
         # Register providers
         register_providers()
 
-        # Resolve adapter-only path
+        # Resolve provider for non-interactive path with sensible defaults
         if not provider:
-            console.print("[red]Error:[/red] Provider must be specified with --provider in non-interactive mode")
-            return False
+            adapters = SetupRegistry.list_adapters()
+            if len(adapters) == 1:
+                provider = adapters[0]
+            elif "mithril" in adapters:
+                provider = "mithril"
+            else:
+                console.print(
+                    "[red]Error:[/red] Provider must be specified with --provider in non-interactive mode"
+                )
+                return False
 
         adapter = SetupRegistry.get_adapter(provider)
         if not adapter:
-            console.print(f"[red]Error:[/red] Unknown or unavailable provider: {provider}")
+            console.print(
+                f"[red]Error:[/red] Unknown or unavailable provider: {escape(str(provider))}"
+            )
             return False
 
         # Build config from provided options only (no prompts in non-interactive path)
@@ -320,19 +467,34 @@ class InitCommand(BaseCommand):
             # Validate via adapter
             vr = adapter.validate_field("api_key", api_key)
             if not vr.is_valid:
-                console.print(f"[red]Invalid API key:[/red] {vr.message}")
+                console.print(f"[red]Invalid API key:[/red] {escape(str(vr.message))}")
                 return False
             config["api_key"] = api_key
+            # Show masked key for feedback
+            from flow.cli.utils.theme_manager import theme_manager as _tm2
+
+            success_color = _tm2.get_color("success")
+            console.print(
+                f"[{success_color}]✓[/{success_color}] API key validated: {mask_api_key(api_key)}"
+            )
+            # Cost awareness warning and demo hint
+            if provider == "mithril":
+                console.print(
+                    "\n[dim]Note:[/dim] Running tasks provisions real infrastructure and may incur costs."
+                )
+                console.print(
+                    "Use [accent]--dry-run[/accent] to preview, or [accent]flow init --demo[/accent] / [accent]flow tutorial --demo[/accent] for a safe mock mode."
+                )
         if project:
             vr = adapter.validate_field("project", project, {"api_key": config.get("api_key")})
             if not vr.is_valid:
-                console.print(f"[red]Invalid project:[/red] {vr.message}")
+                console.print(f"[red]Invalid project:[/red] {escape(str(vr.message))}")
                 return False
             config["project"] = project
         if region:
             vr = adapter.validate_field("region", region)
             if not vr.is_valid:
-                console.print(f"[red]Invalid region:[/red] {vr.message}")
+                console.print(f"[red]Invalid region:[/red] {escape(str(vr.message))}")
                 return False
             config["region"] = region
         if api_url:
@@ -350,38 +512,48 @@ class InitCommand(BaseCommand):
         if dry_run:
             console.print("\n[bold]Configuration (dry run)[/bold]")
             console.print("─" * 50)
-            console.print(yaml.safe_dump(config, default_flow_style=False))
+            # Create masked config for display based on field specs
+            display_config = mask_config_for_display(config, adapter.get_configuration_fields())
+            console.print(yaml.safe_dump(display_config, default_flow_style=False))
             if output_path:
                 try:
                     with open(output_path, "w") as f:
-                        yaml.safe_dump(config, f, default_flow_style=False)
-                    console.print(f"\n[green]✓[/green] Wrote preview to {output_path}")
+                        yaml.safe_dump(display_config, f, default_flow_style=False)
+                    from flow.cli.utils.theme_manager import theme_manager as _tm3
+
+                    success_color = _tm3.get_color("success")
+                    console.print(
+                        f"\n[{success_color}]✓[/{success_color}] Wrote masked preview to {output_path}"
+                    )
                 except Exception as e:
-                    console.print(f"[red]Error writing output file:[/red] {e}")
+                    console.print(f"[red]Error writing output file:[/red] {escape(str(e))}")
                     return False
             return True
 
-        # Save via adapter to ensure consistent behavior
+        # Save via adapter (which uses ConfigManager) to ensure consistent behavior
         saved = adapter.save_configuration(config)
         if not saved:
             console.print("[red]Failed to save configuration[/red]")
             return False
 
-        console.print("\n[green]✓[/green] Configuration saved")
+        from flow.cli.utils.theme_manager import theme_manager as _tm4
+
+        success_color = _tm4.get_color("success")
+        console.print(f"\n[{success_color}]✓[/{success_color}] Configuration saved")
         self.show_next_actions(
             [
-                "Test your setup: [cyan]flow health[/cyan]",
-                "Run an example: [cyan]flow example minimal[/cyan]",
-                "View all examples: [cyan]flow example[/cyan]",
-                "Submit your first task: [cyan]flow run task.yaml[/cyan]",
-                "(Optional) Upload existing SSH key: [cyan]flow ssh-keys upload ~/.ssh/id_ed25519.pub[/cyan]",
+                "Test your setup: [accent]flow health[/accent]",
+                "Run GPU test: [accent]flow example gpu-test[/accent]",
+                "View examples: [accent]flow example[/accent]",
+                "Submit your first task: [accent]flow run task.yaml[/accent]",
+                "(Optional) Upload existing SSH key: [accent]flow ssh-keys upload ~/.ssh/id_ed25519.pub[/accent]",
             ]
         )
         return True
 
     async def _prompt_for_value(
-        self, name: str, password: bool = False, default: Optional[str] = None
-    ) -> Optional[str]:
+        self, name: str, password: bool = False, default: str | None = None
+    ) -> str | None:
         """Prompt user for configuration value.
 
         Args:
@@ -398,29 +570,17 @@ class InitCommand(BaseCommand):
             None, lambda: Prompt.ask(name, password=password, default=default)
         )
 
-    def _mask_api_key(self, api_key: str) -> str:
-        """Mask API key for safe display.
-
-        Example:
-            'fkey_abcd1234efgh5678' -> 'fkey_abc...5678'
-        """
-        if len(api_key) > 10:
-            return f"{api_key[:8]}...{api_key[-4:]}"
-        return "[CONFIGURED]"
+    # Legacy method removed in favor of mask_utils.mask_api_key
 
     async def _reset_configuration(self) -> bool:
         """Reset Flow SDK configuration to initial state.
 
-        Surgical removal of configuration files with safety prompt.
-        Following principles:
-        - Single responsibility: only resets config
-        - User safety: shows what will be deleted
-        - No surprises: requires confirmation
+        Removes configuration files with a safety prompt that lists what will be
+        deleted and asks for confirmation.
 
         Returns:
             bool: True if reset, False if cancelled
         """
-        from pathlib import Path
         from rich.prompt import Confirm
 
         flow_dir = Path.home() / ".flow"
@@ -445,7 +605,7 @@ class InitCommand(BaseCommand):
         for file in files_to_clear:
             console.print(f"  • {file}")
 
-        # Safety prompt (80/20: most users want confirmation)
+        # Safety prompt before deletion
         console.print("")
         confirm = await asyncio.get_event_loop().run_in_executor(
             None,
@@ -461,16 +621,16 @@ class InitCommand(BaseCommand):
             try:
                 file.unlink()
             except Exception as e:
-                console.print(f"[red]Error deleting {file}: {e}[/red]")
+                console.print(f"[red]Error deleting {file}: {escape(str(e))}[/red]")
                 return False
 
         return True
 
     def _setup_shell_completion(self):
-        """Automatically set up shell completion after successful init.
+        """Set up shell completion after successful init.
 
-        This follows the 80/20 principle - works for most users automatically,
-        while advanced users can still customize if needed.
+        Adds a guarded completion line to the user's shell config when
+        possible. Skips silently if the shell cannot be detected.
         """
         try:
             import shutil
@@ -520,26 +680,37 @@ class InitCommand(BaseCommand):
                 if completion_marker in content:
                     return  # Already installed
 
-            # Generate appropriate completion line
-            if shell_name == "bash":
-                completion_line = 'eval "$(_FLOW_COMPLETE=bash_source flow)"'
-            elif shell_name == "zsh":
-                completion_line = 'eval "$(_FLOW_COMPLETE=zsh_source flow)"'
-            elif shell_name == "fish":
-                completion_line = "_FLOW_COMPLETE=fish_source flow | source"
-            else:
-                return
+            # Generate appropriate, shell-guarded completion line
+            try:
+                from flow.cli.utils.shell_completion import CompletionCommand as _CompletionCommand
+
+                completion_line = _CompletionCommand()._get_completion_line(shell_name)
+            except Exception:
+                # Fallback to conservative guarded lines
+                if shell_name == "bash":
+                    completion_line = 'if [ -n "${BASH_VERSION-}" ]; then eval "$(_FLOW_COMPLETE=bash_source flow)"; fi'
+                elif shell_name == "zsh":
+                    completion_line = 'if [ -n "${ZSH_VERSION-}" ]; then eval "$(_FLOW_COMPLETE=zsh_source flow)"; fi'
+                elif shell_name == "fish":
+                    completion_line = (
+                        'if test -n "$FISH_VERSION"; _FLOW_COMPLETE=fish_source flow | source; end'
+                    )
+                else:
+                    return
 
             # Add completion to shell config
             with open(config_file, "a") as f:
                 f.write(f"\n{completion_marker}\n{completion_line}\n")
 
-            console.print(f"\n[green]✓ Shell completion enabled for {shell_name}[/green]")
-            # Offer an immediate, one-shot activation for the current shell session
-            console.print(f"  Enable now in this shell: [cyan]{completion_line}[/cyan]")
+            from flow.cli.utils.theme_manager import theme_manager as _tm5
+
+            success_color = _tm5.get_color("success")
             console.print(
-                f"  Or restart your shell, or run: [cyan]source {config_file}[/cyan]"
+                f"\n[{success_color}]✓ Shell completion enabled for {shell_name}[/{success_color}]"
             )
+            # Offer an immediate, one-shot activation for the current shell session
+            console.print(f"  Enable now in this shell: [accent]{completion_line}[/accent]")
+            console.print(f"  Or restart your shell, or run: [accent]source {config_file}[/accent]")
 
         except Exception:
             # Silently skip if completion setup fails
