@@ -1,7 +1,8 @@
-"""Mount command - Attach volumes to running tasks.
+"""Mount command - attach volumes to tasks.
 
-This module implements the mount command for dynamically attaching
-storage volumes to already running tasks without requiring restart.
+Dynamically attaches storage volumes to tasks. If the task isn't running yet,
+the volume attachment completes now and the mount finalizes when the task
+starts and is SSH-ready.
 
 Command Usage:
     flow mount VOLUME_ID TASK_ID
@@ -28,34 +29,33 @@ The mount operation:
 5. Volume becomes available at /mnt/{volume_name}
 
 Requirements:
-- Task must be running (not pending or terminated)
+- Task can be pending/starting; mount finalizes when the task is ready (SSH available)
 - Volume and task must be in same region
 - Volume cannot already be attached to the task
-- SSH access must be available to the task
+- SSH access is required only for immediate mount; otherwise completes on startup
 """
 
-import click
 import re
-from typing import Optional, Tuple
+
+import click
 
 from flow import Flow
+from flow.cli.commands.base import BaseCommand, console
+from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
+from flow.cli.utils.task_resolver import resolve_task_identifier
+from flow.cli.utils.volume_resolver import get_volume_display_name, resolve_volume_identifier
 from flow.errors import (
     AuthenticationError,
+    RemoteExecutionError,
     ResourceNotFoundError,
     ValidationError,
 )
-from flow.providers.mithril.remote_operations import RemoteExecutionError
-
-from .base import BaseCommand, console
-from ..utils.task_resolver import resolve_task_identifier
-from ..utils.volume_resolver import resolve_volume_identifier, get_volume_display_name
-from ..utils.animated_progress import AnimatedEllipsisProgress
 
 
 class MountCommand(BaseCommand):
-    """Mount volumes to running tasks."""
+    """Mount volumes to tasks (finalizes when task is ready)."""
 
-    def validate_mount_point(self, mount_point: str) -> Optional[str]:
+    def validate_mount_point(self, mount_point: str) -> str | None:
         """Validate and sanitize a custom mount point.
 
         Args:
@@ -79,7 +79,7 @@ class MountCommand(BaseCommand):
             raise ValidationError("Mount point cannot contain '..' (path traversal)")
 
         # Check allowed prefixes
-        allowed_prefixes = ["/mnt/", "/data/", "/opt/", "/var/"]
+        allowed_prefixes = ["/volumes/", "/mnt/", "/data/", "/opt/", "/var/"]
         if not any(mount_point.startswith(prefix) for prefix in allowed_prefixes):
             raise ValidationError(
                 f"Mount point must start with one of: {', '.join(allowed_prefixes)}"
@@ -103,12 +103,13 @@ class MountCommand(BaseCommand):
 
     @property
     def help(self) -> str:
-        return "Attach storage volumes to running tasks - no restart required"
+        return "Attach storage volumes to tasks (may require machine restart to take effect)"
 
     def get_command(self) -> click.Command:
         """Return the mount command."""
         # Import completion functions
-        from ..utils.shell_completion import complete_volume_ids, complete_task_ids
+        from flow.cli.utils.mode import demo_aware_command
+        from flow.cli.utils.shell_completion import complete_task_ids, complete_volume_ids
 
         @click.command(name=self.name, help=self.help)
         @click.argument("volume_identifier", required=False, shell_complete=complete_volume_ids)
@@ -129,7 +130,7 @@ class MountCommand(BaseCommand):
             "--mount-point",
             "-m",
             type=str,
-            help="Custom mount path on the instance (default: /mnt/{volume_name})",
+            help="Custom mount path on the instance (default: /volumes/{volume_name})",
         )
         @click.option(
             "--dry-run", is_flag=True, help="Preview the mount operation without executing"
@@ -140,17 +141,18 @@ class MountCommand(BaseCommand):
             is_flag=True,
             help="Show detailed mount workflows and troubleshooting",
         )
+        @demo_aware_command()
         def mount(
-            volume_identifier: Optional[str],
-            task_identifier: Optional[str],
-            volume: Optional[str],
-            task: Optional[str],
-            instance: Optional[int],
-            mount_point: Optional[str],
+            volume_identifier: str | None,
+            task_identifier: str | None,
+            volume: str | None,
+            task: str | None,
+            instance: int | None,
+            mount_point: str | None,
             dry_run: bool,
             verbose: bool,
         ):
-            """Mount a volume to a running task.
+            """Mount a volume to a task.
 
             \b
             Examples:
@@ -184,14 +186,14 @@ class MountCommand(BaseCommand):
                 )
 
                 console.print("Mount locations:")
-                console.print("  • Default: /mnt/{volume_name}")
-                console.print("  • Example: volume 'datasets' → /mnt/datasets")
+                console.print("  • Default: /volumes/{volume_name}")
+                console.print("  • Example: volume 'datasets' → /volumes/datasets")
                 console.print("  • Custom: --mount-point /data/my-volume")
-                console.print("  • Allowed prefixes: /mnt/, /data/, /opt/, /var/")
-                console.print("  • Access: cd /mnt/datasets\n")
+                console.print("  • Allowed prefixes: /volumes/, /mnt/, /data/, /opt/, /var/")
+                console.print("  • Access: cd /volumes/datasets\n")
 
                 console.print("Common workflows:")
-                console.print("  # Mount dataset to running training")
+                console.print("  # Mount dataset to a training job")
                 console.print("  flow volumes list                 # Find volume")
                 console.print("  flow status                       # Find task")
                 console.print("  flow mount dataset training-job   # Mount it")
@@ -201,14 +203,14 @@ class MountCommand(BaseCommand):
                 console.print("  flow mount shared-data task2\n")
 
                 console.print("Requirements:")
-                console.print("  • Task must be running (not pending)")
+                console.print("  • Task can be pending; mount finalizes when task starts")
                 console.print("  • Volume and task in same region")
                 console.print("  • Volume not already mounted")
                 console.print("  • SSH access available\n")
 
                 console.print("Troubleshooting:")
                 console.print("  • Permission denied → Check volume exists: flow volumes list")
-                console.print("  • Task not found → Verify running: flow status")
+                console.print("  • Task not found → Verify status: flow status")
                 console.print("  • Region mismatch → Create volume in task's region")
                 console.print("  • Mount failed → Check SSH: flow ssh <task>\n")
                 return
@@ -226,13 +228,13 @@ class MountCommand(BaseCommand):
                 # Interactive selection if arguments are missing
                 if not volume_id:
                     # Get available volumes
-                    from ..utils.interactive_selector import select_volume
+                    from flow.cli.utils.interactive_selector import select_volume
 
                     volumes = flow_client.list_volumes()
                     if not volumes:
                         console.print("[yellow]No volumes available.[/yellow]")
                         console.print(
-                            "\nCreate a volume with: [cyan]flow volumes create --size 100[/cyan]"
+                            "\nCreate a volume with: [accent]flow volumes create --size 100[/accent]"
                         )
                         return
 
@@ -246,17 +248,14 @@ class MountCommand(BaseCommand):
                         console.print(f"[dim]Selected volume ID: {volume_id}[/dim]")
 
                 if not task_id:
-                    # Get running tasks
-                    from ..utils.interactive_selector import select_task
+                    # Get available tasks
+                    from flow.cli.utils.interactive_selector import select_task
+                    from flow.cli.utils.status_utils import is_active_like
 
-                    tasks = [
-                        t
-                        for t in flow_client.list_tasks()
-                        if t.status.lower() in ["running", "active"]
-                    ]
+                    tasks = [t for t in flow_client.list_tasks() if is_active_like(t)]
                     if not tasks:
-                        console.print("[yellow]No running tasks available.[/yellow]")
-                        console.print("\nStart a task with: [cyan]flow run[/cyan]")
+                        console.print("[yellow]No eligible tasks available.[/yellow]")
+                        console.print("\nStart a task with: [accent]flow run[/accent]")
                         return
 
                     selected_task = select_task(tasks, title="Select a task to mount to")
@@ -295,12 +294,14 @@ class MountCommand(BaseCommand):
                     try:
                         validated_mount_point = self.validate_mount_point(mount_point)
                     except ValidationError as e:
-                        console.print(f"[red]Error:[/red] {e}")
+                        from rich.markup import escape
+
+                        console.print(f"[red]Error:[/red] {escape(str(e))}")
                         return
 
                 # Show what we're about to mount
                 console.print(
-                    f"\nMounting volume [cyan]{volume_display}[/cyan] to task [cyan]{task_display}[/cyan]"
+                    f"\nMounting volume [accent]{volume_display}[/accent] to task [accent]{task_display}[/accent]"
                 )
 
                 # Multi-instance check
@@ -325,10 +326,10 @@ class MountCommand(BaseCommand):
                         )
                         console.print("\nOptions:")
                         console.print(
-                            "  • Create a file share volume: [cyan]flow volumes create --interface file --size 100[/cyan]"
+                            "  • Create a file share volume: [accent]flow volumes create --interface file --size 100[/accent]"
                         )
                         console.print(
-                            "  • Use an existing file share: [cyan]flow volumes list --interface file[/cyan]"
+                            "  • Use an existing file share: [accent]flow volumes list --interface file[/accent]"
                         )
                         console.print("  • Mount to a single-instance task instead")
                         return
@@ -336,18 +337,18 @@ class MountCommand(BaseCommand):
                 # Check task status
                 if task.status.lower() not in ["running", "active"]:
                     console.print(
-                        f"[yellow]Warning:[/yellow] Task is {task.status}. Mount may fail if task is not fully running."
+                        f"[yellow]Note:[/yellow] Task is {task.status}. The attachment will complete now; the mount will finalize when the task is ready (SSH available)."
                     )
 
                 # Determine mount path
                 if validated_mount_point:
                     actual_mount_path = validated_mount_point
                 else:
-                    actual_mount_path = f"/mnt/{volume.name or f'volume-{volume.id[-6:]}'}"
+                    actual_mount_path = f"/volumes/{volume.name or f'volume-{volume.id[-6:]}'}"
 
                 # Dry run mode
                 if dry_run:
-                    console.print("\n[cyan]DRY RUN - No changes will be made[/cyan]")
+                    console.print("\n[accent]DRY RUN - No changes will be made[/accent]")
                     console.print(f"Would mount volume {volume.id} to task {task.task_id}")
                     if instance is not None:
                         console.print(f"Target instance: {instance}")
@@ -370,7 +371,7 @@ class MountCommand(BaseCommand):
                         # For now, mount to all instances (current behavior)
                         if instance is not None:
                             console.print(
-                                f"[yellow]Note:[/yellow] Instance-specific mounting not yet implemented. Mounting to all instances."
+                                "[yellow]Note:[/yellow] Instance-specific mounting not yet implemented. Mounting to all instances."
                             )
                         # Pass custom mount point if provided
                         if validated_mount_point:
@@ -380,12 +381,16 @@ class MountCommand(BaseCommand):
                         else:
                             flow_client.mount_volume(volume.id, task.task_id)
                     except ValidationError as e:
-                        console.print(f"[red]Validation Error:[/red] {e}")
+                        from rich.markup import escape
+
+                        console.print(f"[red]Validation Error:[/red] {escape(str(e))}")
                         return
                     except RemoteExecutionError as e:
-                        console.print(f"[red]Mount Failed:[/red] {e}")
+                        from rich.markup import escape
+
+                        console.print(f"[red]Mount Failed:[/red] {escape(str(e))}")
                         console.print("\n[yellow]Troubleshooting:[/yellow]")
-                        console.print("  - Ensure the task is fully running (SSH ready)")
+                        console.print("  - Ensure the task is ready (SSH available)")
                         console.print("  - Check that the volume is not already mounted")
                         console.print("  - Verify region compatibility")
                         return
@@ -396,44 +401,46 @@ class MountCommand(BaseCommand):
                 # Check if task is still starting
                 if task.status.lower() in ["pending", "starting", "initializing"]:
                     console.print(
-                        f"[green]✓[/green] Volume attached to task. Mount will complete when instance is ready."
+                        "[green]✓[/green] Volume attached to task. Mount will complete when the task is ready."
                     )
                     console.print(
                         f"\n[yellow]Note:[/yellow] Task is still starting. The volume will be available at "
-                        f"[cyan]{mount_path}[/cyan] once the instance is running."
+                        f"[accent]{mount_path}[/accent] once the task starts."
                     )
                     console.print(
-                        f"\nMithril instances can take several minutes to start. To check status:"
+                        "\nMithril instances can take several minutes to start. To check status:"
                     )
                     task_ref = task.name or task.task_id
                     self.show_next_actions(
                         [
-                            f"Check task status: [cyan]flow status {task_ref}[/cyan]",
-                            f"Wait for SSH and mount: [cyan]flow ssh {task_ref} -c 'df -h {mount_path}'[/cyan]",
-                            f"Stream startup logs: [cyan]flow logs {task_ref} -f[/cyan]",
+                            f"Check task status: [accent]flow status {task_ref}[/accent]",
+                            f"Wait for SSH and mount: [accent]flow ssh {task_ref} -c 'df -h {mount_path}'[/accent]",
+                            f"Stream startup logs: [accent]flow logs {task_ref} -f[/accent]",
                         ]
                     )
                 else:
-                    # Instance is running, mount should be immediate
+                    # Task is ready; mount should be immediate
                     console.print(
-                        f"[green]✓[/green] Volume mounted successfully at [cyan]{mount_path}[/cyan]"
+                        f"[green]✓[/green] Volume mounted successfully at [accent]{mount_path}[/accent]"
                     )
                     console.print(
-                        f"\n[yellow]Note:[/yellow] In some cases, the instance may need to be restarted for the mount to take effect."
+                        "\n[yellow]Note:[/yellow] In some cases, the task may need to be restarted for the mount to take effect."
                     )
                     task_ref = task.name or task.task_id
                     self.show_next_actions(
                         [
-                            f"SSH into task: [cyan]flow ssh {task_ref}[/cyan]",
-                            f"Verify mount: [cyan]flow ssh {task_ref} -c 'df -h {mount_path}'[/cyan]",
-                            "List all volumes: [cyan]flow volumes list[/cyan]",
+                            f"SSH into task: [accent]flow ssh {task_ref}[/accent]",
+                            f"Verify mount: [accent]flow ssh {task_ref} -c 'df -h {mount_path}'[/accent]",
+                            "List all volumes: [accent]flow volumes list[/accent]",
                         ]
                     )
 
             except AuthenticationError:
                 self.handle_auth_error()
             except ResourceNotFoundError as e:
-                console.print(f"[red]Not Found:[/red] {e}")
+                from rich.markup import escape
+
+                console.print(f"[red]Not Found:[/red] {escape(str(e))}")
             except Exception as e:
                 self.handle_error(str(e))
 
