@@ -1,8 +1,7 @@
-"""Volumes command group - Manage persistent storage volumes for GPU tasks.
+"""Volumes command group - manage persistent storage volumes for GPU tasks.
 
-This module implements the volumes command group for the Flow CLI. It provides
-comprehensive volume management capabilities including creation, deletion,
-listing, and bulk operations for persistent storage used by GPU compute tasks.
+Provides creation, deletion, listing, and bulk operations for volumes used by
+GPU tasks.
 
 Command Usage:
     flow volumes SUBCOMMAND [OPTIONS]
@@ -70,25 +69,19 @@ Note:
     - Partial name matching works if there's only one match
 """
 
-import re
-from concurrent.futures import ThreadPoolExecutor, as_completed
-from typing import Any, Dict, Optional, Tuple
-
 import click
-from rich.table import Table
 
 from flow import Flow
+from flow.cli.commands.base import BaseCommand, console
+from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
+from flow.cli.utils.table_styles import add_centered_column, create_flow_table, wrap_table_in_panel
+from flow.cli.utils.terminal_adapter import TerminalAdapter
+from flow.cli.utils.theme_manager import theme_manager
+from flow.cli.utils.volume_index_cache import VolumeIndexCache
+from flow.cli.utils.volume_operations import VolumeOperations
+from flow.cli.utils.volume_resolver import get_volume_display_name, resolve_volume_identifier
 from flow.errors import AuthenticationError
 
-from .base import BaseCommand, console
-from ..utils.volume_operations import VolumeOperations, BulkOperationResults
-from ..utils.table_styles import create_flow_table, wrap_table_in_panel, add_centered_column
-from ..utils.theme_manager import theme_manager
-from ..utils.volume_resolver import resolve_volume_identifier, get_volume_display_name
-from ..utils.volume_index_cache import VolumeIndexCache
-from ..utils.animated_progress import AnimatedEllipsisProgress
-from ..utils.terminal_adapter import TerminalAdapter
-
 
 class VolumesCommand(BaseCommand):
     """Manage storage volumes."""
@@ -101,7 +94,7 @@ class VolumesCommand(BaseCommand):
     def help(self) -> str:
         return "Manage persistent storage volumes - create, list, delete"
 
-    def _build_instance_task_map(self, flow_client: Flow) -> Dict[str, Tuple[str, str]]:
+    def _build_instance_task_map(self, flow_client: Flow) -> dict[str, tuple[str, str]]:
         """Build mapping of instance_id -> (task_name, task_id).
 
         Returns empty dict on error to gracefully degrade.
@@ -122,8 +115,11 @@ class VolumesCommand(BaseCommand):
     def get_command(self) -> click.Group:
         """Return the volumes command group."""
 
+        from flow.cli.utils.mode import demo_aware_command
+
         @click.group(name=self.name, help=self.help)
         @click.option("--verbose", "-v", is_flag=True, help="Show detailed volume management guide")
+        @demo_aware_command()
         def volumes(verbose: bool):
             """Manage storage volumes.
 
@@ -210,9 +206,9 @@ class VolumesCommand(BaseCommand):
                     console.print("\nNo volumes found.")
                     self.show_next_actions(
                         [
-                            "Create a new volume: [cyan]flow volumes create --size 100[/cyan]",
-                            "Create a named volume: [cyan]flow volumes create --size 50 --name training-data[/cyan]",
-                            "Create file storage: [cyan]flow volumes create --size 200 --interface file[/cyan]",
+                            "Create a new volume: [accent]flow volumes create --size 100[/accent]",
+                            "Create a named volume: [accent]flow volumes create --size 50 --name training-data[/accent]",
+                            "Create file storage: [accent]flow volumes create --size 200 --interface file[/accent]",
                         ]
                     )
                     return
@@ -230,7 +226,8 @@ class VolumesCommand(BaseCommand):
 
                 # Create table with Flow standard styling
                 table = create_flow_table(
-                    show_borders=False
+                    show_borders=False,
+                    expand=False,
                 )  # No borders since we'll wrap in panel
 
                 # Always show core columns
@@ -366,8 +363,8 @@ class VolumesCommand(BaseCommand):
 
                 self.show_next_actions(
                     [
-                        "Create a new volume: [cyan]flow volumes create --size 100[/cyan]",
-                        f"Delete a volume: [cyan]flow volumes delete <volume-name-or-id>[/cyan] or [cyan]flow volumes delete {index_help}[/cyan]",
+                        "Create a new volume: [accent]flow volumes create --size 100[/accent]",
+                        f"Delete a volume: [accent]flow volumes delete <volume-name-or-id>[/accent] or [accent]flow volumes delete {index_help}[/accent]",
                     ]
                 )
 
@@ -389,31 +386,49 @@ class VolumesCommand(BaseCommand):
             default="block",
             help="Storage interface type",
         )
-        def volumes_create(size: int, name: Optional[str], interface: str):
+        def volumes_create(size: int, name: str | None, interface: str):
             """Create a new volume."""
             try:
                 flow_client = Flow(auto_init=True)
 
-                from ..utils.animated_progress import AnimatedEllipsisProgress
+                from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
 
                 with AnimatedEllipsisProgress(
                     console, f"Creating {size}GB {interface} volume"
                 ) as progress:
                     volume = flow_client.create_volume(size_gb=size, name=name, interface=interface)
 
-                console.print(f"[green]✓[/green] Volume created: [cyan]{volume.volume_id}[/cyan]")
+                console.print(
+                    f"[green]✓[/green] Volume created: [accent]{volume.volume_id}[/accent]"
+                )
                 if name:
                     console.print(f"Name: {name}")
 
                 # Show next actions
                 self.show_next_actions(
                     [
-                        "List all volumes: [cyan]flow volumes list[/cyan]",
-                        f"Use in task config: Add to YAML under storage.volumes",
-                        "Submit task with volume: [cyan]flow run task.yaml[/cyan]",
+                        "List all volumes: [accent]flow volumes list[/accent]",
+                        "Use in task config: Add to YAML under storage.volumes",
+                        "Submit task with volume: [accent]flow run task.yaml[/accent]",
                     ]
                 )
 
+                # Invalidate and refresh volumes cache after creation
+                try:
+                    from flow.cli.utils.prefetch import (
+                        invalidate_cache_for_current_context,
+                        refresh_volumes_cache,
+                    )
+
+                    invalidate_cache_for_current_context(
+                        ["volumes_list"]
+                    )  # ensure fresh list on next view
+                    import threading
+
+                    threading.Thread(target=refresh_volumes_cache, daemon=True).start()
+                except Exception:
+                    pass
+
             except AuthenticationError:
                 self.handle_auth_error()
             except Exception as e:
@@ -433,7 +448,7 @@ class VolumesCommand(BaseCommand):
                                     types = ", ".join(cap.get("types", []))
                                     max_gb = cap.get("max_gb", 0)
                                     console.print(
-                                        f"  [cyan]{region}[/cyan]: {types} (up to {max_gb:,}GB)"
+                                        f"  [accent]{region}[/accent]: {types} (up to {max_gb:,}GB)"
                                     )
                     except Exception:
                         pass  # Don't fail if capability discovery fails
@@ -444,7 +459,7 @@ class VolumesCommand(BaseCommand):
 
     def _delete_command(self) -> click.Command:
         # Import completion function
-        from ..utils.shell_completion import complete_volume_ids
+        from flow.cli.utils.shell_completion import complete_volume_ids
 
         @click.command(name="delete")
         @click.argument("volume_identifier", shell_complete=complete_volume_ids)
@@ -482,7 +497,7 @@ class VolumesCommand(BaseCommand):
                             return
 
                     # Delete all volumes
-                    from ..utils.animated_progress import AnimatedEllipsisProgress
+                    from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
 
                     deleted_count = 0
                     with AnimatedEllipsisProgress(
@@ -498,7 +513,11 @@ class VolumesCommand(BaseCommand):
                                 console.print(f"[green]✓[/green] Deleted {volume_name}")
                                 deleted_count += 1
                             except Exception as e:
-                                console.print(f"[red]✗[/red] Failed to delete {volume_name}: {e}")
+                                from rich.markup import escape
+
+                                console.print(
+                                    f"[red]✗[/red] Failed to delete {volume_name}: {escape(str(e))}"
+                                )
 
                     console.print(f"\nDeleted {deleted_count} volume(s)")
                 else:
@@ -517,7 +536,7 @@ class VolumesCommand(BaseCommand):
                             console.print("Cancelled")
                             return
 
-                    from ..utils.animated_progress import AnimatedEllipsisProgress
+                    from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
 
                     with AnimatedEllipsisProgress(
                         console, f"Deleting volume {display_name}"
@@ -528,11 +547,27 @@ class VolumesCommand(BaseCommand):
                 # Show next actions
                 self.show_next_actions(
                     [
-                        "List remaining volumes: [cyan]flow volumes list[/cyan]",
-                        "Create a new volume: [cyan]flow volumes create --size 100[/cyan]",
+                        "List remaining volumes: [accent]flow volumes list[/accent]",
+                        "Create a new volume: [accent]flow volumes create --size 100[/accent]",
                     ]
                 )
 
+                # Invalidate and refresh volumes cache after deletion
+                try:
+                    from flow.cli.utils.prefetch import (
+                        invalidate_cache_for_current_context,
+                        refresh_volumes_cache,
+                    )
+
+                    invalidate_cache_for_current_context(
+                        ["volumes_list"]
+                    )  # ensure fresh list on next view
+                    import threading
+
+                    threading.Thread(target=refresh_volumes_cache, daemon=True).start()
+                except Exception:
+                    pass
+
             except AuthenticationError:
                 self.handle_auth_error()
             except Exception as e:
@@ -545,7 +580,7 @@ class VolumesCommand(BaseCommand):
         @click.option("--pattern", "-p", help="Only delete volumes matching pattern")
         @click.option("--dry-run", is_flag=True, help="Show what would be deleted")
         @click.option("--yes", "-y", is_flag=True, help="Skip confirmation")
-        def volumes_delete_all(pattern: Optional[str], dry_run: bool, yes: bool):
+        def volumes_delete_all(pattern: str | None, dry_run: bool, yes: bool):
             """Delete all volumes (with optional pattern matching)."""
             try:
                 flow_client = Flow(auto_init=True)
@@ -596,11 +631,27 @@ class VolumesCommand(BaseCommand):
                 # Show next actions
                 self.show_next_actions(
                     [
-                        "List remaining volumes: [cyan]flow volumes list[/cyan]",
-                        "Create a new volume: [cyan]flow volumes create --size 100[/cyan]",
+                        "List remaining volumes: [accent]flow volumes list[/accent]",
+                        "Create a new volume: [accent]flow volumes create --size 100[/accent]",
                     ]
                 )
 
+                # Invalidate and refresh volumes cache after bulk deletion
+                try:
+                    from flow.cli.utils.prefetch import (
+                        invalidate_cache_for_current_context,
+                        refresh_volumes_cache,
+                    )
+
+                    invalidate_cache_for_current_context(
+                        ["volumes_list"]
+                    )  # ensure fresh list on next view
+                    import threading
+
+                    threading.Thread(target=refresh_volumes_cache, daemon=True).start()
+                except Exception:
+                    pass
+
             except AuthenticationError:
                 self.handle_auth_error()
             except Exception as e:
