@@ -1,11 +1,8 @@
 """Health status rendering utilities for CLI output."""
 
-import os
-from datetime import datetime, timedelta
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any
 
 from rich import box
-from rich.align import Align
 from rich.console import Console
 from rich.panel import Panel
 from rich.progress import Progress, SpinnerColumn, TextColumn
@@ -17,18 +14,19 @@ from flow.api.health_models import (
     HealthStatus,
     NodeHealthSnapshot,
 )
-
-from .gpu_formatter import GPUFormatter
-from .terminal_adapter import TerminalAdapter, TerminalBreakpoints
-from .time_formatter import TimeFormatter
+from flow.cli.utils.gpu_formatter import GPUFormatter
+from flow.cli.utils.table_styles import create_flow_table, wrap_table_in_panel
+from flow.cli.utils.terminal_adapter import TerminalAdapter, TerminalBreakpoints
+from flow.cli.utils.theme_manager import theme_manager
+from flow.cli.utils.time_formatter import TimeFormatter
 
 
 class HealthRenderer:
     """Renders health status information for Flow tasks."""
 
-    def __init__(self, console: Optional[Console] = None):
+    def __init__(self, console: Console | None = None):
         """Initialize renderer with optional console override."""
-        self.console = console or Console()
+        self.console = console or theme_manager.create_console()
         self.terminal = TerminalAdapter()
         self.time_fmt = TimeFormatter()
         self.gpu_fmt = GPUFormatter()
@@ -47,7 +45,7 @@ class HealthRenderer:
 
         panel = Panel(
             panel_content,
-            title="[bold cyan]Fleet Health Summary[/bold cyan]",
+            title=f"[bold {theme_manager.get_color('accent')}]Fleet Health Summary[/bold {theme_manager.get_color('accent')}]",
             title_align="center",
             border_style=border_style,
             box=box.ROUNDED,
@@ -62,8 +60,8 @@ class HealthRenderer:
 
     def render_node_health_table(
         self,
-        nodes: List[NodeHealthSnapshot],
-        title: Optional[str] = None,
+        nodes: list[NodeHealthSnapshot],
+        title: str | None = None,
         show_details: bool = False,
     ) -> None:
         """Render health status for multiple nodes.
@@ -83,12 +81,7 @@ class HealthRenderer:
         # Create table (no borders when wrapped in panel)
         if title:
             # Simpler table for panel wrapping
-            table = Table(
-                box=None,
-                show_header=True,
-                header_style="bold",
-                expand=False,
-            )
+            table = create_flow_table(show_borders=False, padding=1, expand=False)
         else:
             table = self._create_health_table(nodes, layout, show_details)
 
@@ -102,17 +95,177 @@ class HealthRenderer:
 
         # Wrap in panel with centered title (matches task renderer style)
         if title:
-            panel = Panel(
-                table,
-                title=f"[bold cyan]{title}[/bold cyan]",
-                title_align="center",
-                border_style="cyan",
-                padding=(1, 2),
-            )
-            self.console.print(panel)
+            wrap_table_in_panel(table, title, self.console)
         else:
             self.console.print(table)
 
+    def render_live_table(self, tasks: list[Any], snapshots: list[NodeHealthSnapshot]) -> Table:
+        """Create a live-updating table for health display.
+
+        This replicates the inline table previously built in the command layer,
+        but uses shared theming and proportions.
+        """
+        table = create_flow_table(
+            title="Node GPU Monitoring & Health Status",
+            show_borders=True,
+            padding=1,
+            expand=False,
+        )
+
+        # Columns: compact, set explicit widths and prevent stretching
+        from flow.cli.utils.table_styles import add_centered_column
+
+        table.add_column(
+            "Node",
+            style=theme_manager.get_color("task.name"),
+            no_wrap=True,
+            min_width=18,
+            max_width=24,
+            overflow="ellipsis",
+        )
+        add_centered_column(table, "Monitor", width=7)
+        add_centered_column(table, "GPUs", width=8)
+        add_centered_column(table, "Temp", width=6)
+        add_centered_column(table, "Usage", width=7)
+        add_centered_column(table, "Memory", width=7)
+        table.add_column(
+            "Status",
+            style=theme_manager.get_color("table.row.dim"),
+            no_wrap=True,
+            min_width=10,
+            max_width=14,
+            overflow="fold",
+        )
+
+        # Add rows for completed checks
+        checked_ids = {s.task_id for s in snapshots}
+        for snapshot in snapshots:
+            self._add_live_table_row(table, snapshot)
+
+        # Spinner for pending rows
+        import time
+
+        spinner_frames = ["⠋", "⠙", "⠹", "⠸", "⠼", "⠴", "⠦", "⠧", "⠇", "⠏"]
+        frame_idx = int(time.time() * 10) % len(spinner_frames)
+        spinner = spinner_frames[frame_idx]
+
+        for task in tasks:
+            if task.task_id not in checked_ids:
+                node_label = getattr(task, "name", None) or task.task_id[:12]
+                table.add_row(
+                    node_label,
+                    f"[yellow]{spinner}[/yellow]",
+                    "[dim]—[/dim]",
+                    "[dim]—[/dim]",
+                    "[dim]—[/dim]",
+                    "[dim]—[/dim]",
+                    "[dim italic]Checking...[/dim italic]",
+                )
+
+        return table
+
+    # Public wrapper to add a completed health row to a live table
+    def add_live_table_row(self, table: Table, snapshot: NodeHealthSnapshot) -> None:
+        self._add_live_table_row(table, snapshot)
+
+    def _add_live_table_row(self, table: Table, snapshot: NodeHealthSnapshot) -> None:
+        """Add a completed health check row to the live table (shared)."""
+        # Monitoring
+        if snapshot.gpud_healthy:
+            monitoring = "[green]✓[/green]"
+        elif snapshot.health_status == HealthStatus.UNKNOWN:
+            note = str((snapshot.machine_info or {}).get("note", "")).lower()
+            monitoring = "[dim]Legacy[/dim]" if "legacy" in note else "[yellow]None[/yellow]"
+        else:
+            monitoring = "[red]✗[/red]"
+
+        # GPU count: show cluster total GPUs when multi-node info is available
+        try:
+            nodes = int((snapshot.machine_info or {}).get("nodes", 1) or 1)
+        except Exception:
+            nodes = 1
+        gpn = len(snapshot.gpu_metrics) if snapshot.gpu_metrics else 0
+        if gpn == 0:
+            try:
+                gpn = int((snapshot.machine_info or {}).get("gpus_per_node", 0) or 0)
+            except Exception:
+                gpn = 0
+        total_gpus = nodes * gpn
+        if total_gpus > 0:
+            gpu_count = f"[accent]{total_gpus}[/accent]"
+        elif snapshot.gpud_healthy:
+            gpu_count = "[accent]0[/accent]"
+        else:
+            gpu_count = "[dim]—[/dim]"
+
+        # Temp
+        if snapshot.gpu_metrics:
+            avg_temp = sum(g.temperature_c for g in snapshot.gpu_metrics) / len(
+                snapshot.gpu_metrics
+            )
+            temp_color = self._get_temperature_color(avg_temp)
+            temp = f"[{temp_color}]{avg_temp:.0f}°C[/{temp_color}]"
+        else:
+            temp = "[dim]—[/dim]"
+
+        # Usage
+        if snapshot.gpu_metrics:
+            avg_usage = sum(g.gpu_utilization_pct for g in snapshot.gpu_metrics) / len(
+                snapshot.gpu_metrics
+            )
+            usage_color = self._get_utilization_color(avg_usage)
+            usage = f"[{usage_color}]{avg_usage:.0f}%[/{usage_color}]"
+        else:
+            usage = "[dim]—[/dim]"
+
+        # Memory
+        if snapshot.gpu_metrics:
+            avg_mem = sum(g.memory_utilization_pct for g in snapshot.gpu_metrics) / len(
+                snapshot.gpu_metrics
+            )
+            mem_color = self._get_utilization_color(avg_mem)
+            memory = f"[{mem_color}]{avg_mem:.0f}%[/{mem_color}]"
+        else:
+            memory = "[dim]—[/dim]"
+
+        # Status
+        if not snapshot.gpud_healthy:
+            note = str((snapshot.machine_info or {}).get("note", "")).lower()
+            if "legacy" in note:
+                status = "[dim]Legacy node[/dim]"
+            elif "not installed" in note:
+                from flow.cli.utils.hyperlink_support import (
+                    hyperlink_support,  # local import to avoid cycles
+                )
+
+                install_url = "https://pkg.gpud.dev/install.sh"
+                link = hyperlink_support.create_link("Install GPUd →", install_url)
+                status = f"[yellow]{link}[/yellow]"
+            else:
+                status = "[red]Connection failed[/red]"
+        elif snapshot.health_status == HealthStatus.HEALTHY:
+            status = "[green]● Healthy[/green]"
+        elif snapshot.health_status == HealthStatus.DEGRADED:
+            issues = []
+            for gpu in snapshot.gpu_metrics:
+                if gpu.temperature_c >= 75:
+                    issues.append("Hot")
+                if gpu.memory_utilization_pct >= 90:
+                    issues.append("Mem full")
+            status = f"[yellow]⚠ {' & '.join(issues[:1]) if issues else 'Degraded'}[/yellow]"
+        else:
+            status = "[red]● Critical[/red]"
+
+        table.add_row(
+            snapshot.task_name or snapshot.task_id[:12],
+            monitoring,
+            gpu_count,
+            temp,
+            usage,
+            memory,
+            status,
+        )
+
     def render_node_details(self, node: NodeHealthSnapshot) -> None:
         """Render detailed health information for a single node.
 
@@ -123,6 +276,14 @@ class HealthRenderer:
         header = self._create_node_header(node)
         self.console.print(header)
 
+        # Health score breakdown (if available)
+        try:
+            breakdown = (node.machine_info or {}).get("health_score_breakdown")
+            if breakdown:
+                self._render_health_breakdown(node)
+        except Exception:
+            pass
+
         # GPU metrics table
         if node.gpu_metrics:
             self._render_gpu_metrics(node)
@@ -139,7 +300,7 @@ class HealthRenderer:
         if node.events:
             self._render_recent_events(node)
 
-    def render_checking_health(self, task_names: List[str]) -> Progress:
+    def render_checking_health(self, task_names: list[str]) -> Progress:
         """Render progress indicator for health checking operation.
 
         Args:
@@ -168,7 +329,8 @@ class HealthRenderer:
 
         # Health percentage with color
         health_pct = summary.health_percentage
-        health_color = self._get_percentage_color(health_pct)
+        # Health percentage is higher-is-better
+        health_color = self._get_score_color(health_pct)
 
         table.add_row(
             "[bold]Overall Health[/bold]", f"[{health_color}]{health_pct:.1f}%[/{health_color}]"
@@ -199,42 +361,81 @@ class HealthRenderer:
 
     def _create_health_table(
         self,
-        nodes: List[NodeHealthSnapshot],
-        layout: Dict[str, Any],
+        nodes: list[NodeHealthSnapshot],
+        layout: dict[str, Any],
         show_details: bool,
     ) -> Table:
         """Create health status table with responsive columns."""
-        table = Table(
-            box=box.ROUNDED,
-            show_header=True,
-            header_style="bold",
-            border_style="cyan",
-            title_style="bold cyan",
-            expand=False,
-        )
+        table = create_flow_table(show_borders=True, padding=1, expand=False)
 
         # Base columns
         table.add_column(
-            "Task", style="white", no_wrap=True, header_style="bold white", justify="center"
+            "Task",
+            style=theme_manager.get_color("task.name"),
+            no_wrap=True,
+            header_style=theme_manager.get_color("table.header"),
+            justify="left",
+            min_width=20,
+            max_width=48,
+            overflow="ellipsis",
+        )
+        table.add_column(
+            "Status", justify="center", header_style=theme_manager.get_color("table.header")
+        )
+        table.add_column(
+            "Health", justify="center", header_style=theme_manager.get_color("table.header")
         )
-        table.add_column("Status", justify="center", header_style="bold white")
-        table.add_column("Health", justify="center", header_style="bold white")
 
         # Responsive columns based on width
         width = self.terminal.get_terminal_width()
 
         if width >= TerminalBreakpoints.COMPACT:
-            table.add_column("GPUs", justify="center", header_style="bold white")
-            table.add_column("GPU Temp", justify="center", header_style="bold white")
-            table.add_column("GPU Usage", justify="center", header_style="bold white")
+            table.add_column(
+                "GPUs",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=6,
+                no_wrap=True,
+            )
+            table.add_column(
+                "GPU Temp",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=7,
+                no_wrap=True,
+            )
+            table.add_column(
+                "GPU Usage",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=7,
+                no_wrap=True,
+            )
 
         if width >= TerminalBreakpoints.NORMAL:
-            table.add_column("Memory", justify="center", header_style="bold white")
-            table.add_column("Issues", justify="center", header_style="bold white")
+            table.add_column(
+                "Memory",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=7,
+                no_wrap=True,
+            )
+            table.add_column(
+                "Issues",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=6,
+                no_wrap=True,
+            )
 
         if width >= TerminalBreakpoints.WIDE:
             table.add_column(
-                "Last Check", justify="center", header_style="bold white", style="dim white"
+                "Last Check",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                style=theme_manager.get_color("task.time"),
+                width=12,
+                no_wrap=True,
             )
 
         # Add rows
@@ -247,24 +448,70 @@ class HealthRenderer:
         """Add columns to health table based on terminal width."""
         # Base columns
         table.add_column(
-            "Task", style="white", no_wrap=True, header_style="bold white", justify="center"
+            "Task",
+            style=theme_manager.get_color("task.name"),
+            no_wrap=True,
+            header_style=theme_manager.get_color("table.header"),
+            justify="left",
+            min_width=20,
+            max_width=48,
+            overflow="ellipsis",
+        )
+        table.add_column(
+            "Status", justify="center", header_style=theme_manager.get_color("table.header")
+        )
+        table.add_column(
+            "Health", justify="center", header_style=theme_manager.get_color("table.header")
         )
-        table.add_column("Status", justify="center", header_style="bold white")
-        table.add_column("Health", justify="center", header_style="bold white")
 
         # Responsive columns based on width
         if width >= TerminalBreakpoints.COMPACT:
-            table.add_column("GPUs", justify="center", header_style="bold white")
-            table.add_column("GPU Temp", justify="center", header_style="bold white")
-            table.add_column("GPU Usage", justify="center", header_style="bold white")
+            table.add_column(
+                "GPUs",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=6,
+                no_wrap=True,
+            )
+            table.add_column(
+                "GPU Temp",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=7,
+                no_wrap=True,
+            )
+            table.add_column(
+                "GPU Usage",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=7,
+                no_wrap=True,
+            )
 
         if width >= TerminalBreakpoints.NORMAL:
-            table.add_column("Memory", justify="center", header_style="bold white")
-            table.add_column("Issues", justify="center", header_style="bold white")
+            table.add_column(
+                "Memory",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=7,
+                no_wrap=True,
+            )
+            table.add_column(
+                "Issues",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                width=6,
+                no_wrap=True,
+            )
 
         if width >= TerminalBreakpoints.WIDE:
             table.add_column(
-                "Last Check", justify="center", header_style="bold white", style="dim white"
+                "Last Check",
+                justify="center",
+                header_style=theme_manager.get_color("table.header"),
+                style=theme_manager.get_color("task.time"),
+                width=12,
+                no_wrap=True,
             )
 
     def _add_health_row(
@@ -282,8 +529,8 @@ class HealthRenderer:
         status_text, status_style = self._format_health_status(node.health_status)
         status = Text(status_text, style=status_style)
 
-        # Health score
-        score_color = self._get_percentage_color(node.health_score * 100)
+        # Health score (higher is better => green)
+        score_color = self._get_score_color(node.health_score * 100)
         health = Text(f"{node.health_score * 100:.0f}%", style=score_color)
 
         row = [task_name, status, health]
@@ -307,7 +554,7 @@ class HealthRenderer:
                 avg_util = sum(g.gpu_utilization_pct for g in node.gpu_metrics) / len(
                     node.gpu_metrics
                 )
-                util_color = self._get_percentage_color(avg_util)
+                util_color = self._get_utilization_color(avg_util)
                 row.append(Text(f"{avg_util:.0f}%", style=util_color))
             else:
                 row.append("-")
@@ -318,7 +565,7 @@ class HealthRenderer:
                 avg_mem = sum(g.memory_utilization_pct for g in node.gpu_metrics) / len(
                     node.gpu_metrics
                 )
-                mem_color = self._get_percentage_color(avg_mem)
+                mem_color = self._get_utilization_color(avg_mem)
                 row.append(Text(f"{avg_mem:.0f}%", style=mem_color))
             else:
                 row.append("-")
@@ -339,14 +586,7 @@ class HealthRenderer:
 
     def _render_gpu_metrics(self, node: NodeHealthSnapshot) -> None:
         """Render GPU metrics table."""
-        table = Table(
-            title="[bold cyan]GPU Metrics[/bold cyan]",
-            box=box.ROUNDED,
-            show_header=True,
-            header_style="bold",
-            border_style="cyan",
-            title_style="bold cyan",
-        )
+        table = create_flow_table(title="GPU Metrics", show_borders=True, padding=1, expand=False)
 
         table.add_column("GPU", style="white", header_style="bold white", justify="center")
         table.add_column("Model", style="dim white", header_style="bold white", justify="center")
@@ -365,12 +605,12 @@ class HealthRenderer:
             power = f"{gpu.power_draw_w:.0f}W/{gpu.power_limit_w:.0f}W"
 
             # GPU utilization
-            util_color = self._get_percentage_color(gpu.gpu_utilization_pct)
+            util_color = self._get_utilization_color(gpu.gpu_utilization_pct)
             usage = Text(f"{gpu.gpu_utilization_pct:.0f}%", style=util_color)
 
             # Memory usage
             mem_pct = gpu.memory_utilization_pct
-            mem_color = self._get_percentage_color(mem_pct)
+            mem_color = self._get_utilization_color(mem_pct)
             memory = Text(
                 f"{gpu.memory_used_mb}/{gpu.memory_total_mb}MB ({mem_pct:.0f}%)", style=mem_color
             )
@@ -402,21 +642,21 @@ class HealthRenderer:
         table = Table(show_header=False, box=None, padding=0)
 
         # CPU usage
-        cpu_color = self._get_percentage_color(metrics.cpu_usage_pct)
+        cpu_color = self._get_utilization_color(metrics.cpu_usage_pct)
         table.add_row(
             "[bold]CPU Usage[/bold]", f"[{cpu_color}]{metrics.cpu_usage_pct:.1f}%[/{cpu_color}]"
         )
 
         # Memory usage
         mem_pct = metrics.memory_utilization_pct
-        mem_color = self._get_percentage_color(mem_pct)
+        mem_color = self._get_utilization_color(mem_pct)
         table.add_row(
             "[bold]Memory[/bold]",
             f"[{mem_color}]{metrics.memory_used_gb:.1f}/{metrics.memory_total_gb:.1f}GB ({mem_pct:.0f}%)[/{mem_color}]",
         )
 
         # Disk usage
-        disk_color = self._get_percentage_color(metrics.disk_usage_pct)
+        disk_color = self._get_utilization_color(metrics.disk_usage_pct)
         table.add_row(
             "[bold]Disk Usage[/bold]", f"[{disk_color}]{metrics.disk_usage_pct:.1f}%[/{disk_color}]"
         )
@@ -430,9 +670,58 @@ class HealthRenderer:
 
         panel = Panel(
             table,
-            title="[bold cyan]System Metrics[/bold cyan]",
+            title=f"[bold {theme_manager.get_color('accent')}]System Metrics[/bold {theme_manager.get_color('accent')}]",
             title_align="center",
-            border_style="cyan",
+            border_style=theme_manager.get_color("table.border"),
+            box=box.ROUNDED,
+            padding=(1, 2),
+        )
+
+        self.console.print(panel)
+
+    def _render_health_breakdown(self, node: NodeHealthSnapshot) -> None:
+        """Render per-component health score breakdown if provided."""
+        breakdown = (node.machine_info or {}).get("health_score_breakdown")
+        if not breakdown:
+            return
+
+        def style_for_score(value: float) -> str:
+            if value >= 0.8:
+                return "green"
+            if value >= 0.6:
+                return "yellow"
+            return "red"
+
+        table = Table(show_header=False, box=None, padding=0)
+        table.add_column("Component", style="white")
+        table.add_column("Score", justify="right")
+
+        order = ["gpu", "memory", "interconnect", "host", "software", "confidence"]
+        labels = {
+            "gpu": "GPU",
+            "memory": "Memory",
+            "interconnect": "Interconnect",
+            "host": "Host",
+            "software": "Software",
+            "confidence": "Confidence",
+        }
+
+        for key in order:
+            if key in breakdown:
+                val = breakdown.get(key, 0)
+                try:
+                    val_f = float(val)
+                except Exception:
+                    continue
+                color = style_for_score(val_f)
+                pct = f"{val_f * 100:.0f}%"
+                table.add_row(labels.get(key, key.title()), f"[{color}]{pct}[/{color}]")
+
+        panel = Panel(
+            table,
+            title=f"[bold {theme_manager.get_color('accent')}]Health Breakdown[/bold {theme_manager.get_color('accent')}]",
+            title_align="center",
+            border_style=theme_manager.get_color("table.border"),
             box=box.ROUNDED,
             padding=(1, 2),
         )
@@ -444,13 +733,8 @@ class HealthRenderer:
         if not node.health_states:
             return
 
-        table = Table(
-            title="[bold cyan]Component Health States[/bold cyan]",
-            box=box.ROUNDED,
-            show_header=True,
-            header_style="bold",
-            border_style="cyan",
-            title_style="bold cyan",
+        table = create_flow_table(
+            title="Component Health States", show_borders=True, padding=1, expand=False
         )
 
         table.add_column("Component", style="white", header_style="bold white", justify="center")
@@ -475,14 +759,7 @@ class HealthRenderer:
         # Only show last 10 events
         recent_events = sorted(node.events, key=lambda e: e.timestamp, reverse=True)[:10]
 
-        table = Table(
-            title="[bold cyan]Recent Events[/bold cyan]",
-            box=box.ROUNDED,
-            show_header=True,
-            header_style="bold",
-            border_style="cyan",
-            title_style="bold cyan",
-        )
+        table = create_flow_table(title="Recent Events", show_borders=True, padding=1, expand=False)
 
         table.add_column("Time", style="dim white", header_style="bold white", justify="center")
         table.add_column("Level", justify="center", header_style="bold white")
@@ -516,12 +793,7 @@ class HealthRenderer:
         if not summary.critical_issues:
             return
 
-        table = Table(
-            box=None,
-            show_header=True,
-            header_style="bold",
-            expand=False,
-        )
+        table = create_flow_table(show_borders=False, padding=1, expand=False)
 
         table.add_column("Node", style="cyan")
         table.add_column("Component", style="yellow")
@@ -537,9 +809,9 @@ class HealthRenderer:
         # Wrap in panel with centered title
         panel = Panel(
             table,
-            title="[bold red]Critical Issues[/bold red]",
+            title=f"[bold {theme_manager.get_color('error')}]Critical Issues[/bold {theme_manager.get_color('error')}]",
             title_align="center",
-            border_style="red",
+            border_style=theme_manager.get_color("error"),
             box=box.ROUNDED,
             padding=(0, 1),
         )
@@ -552,12 +824,12 @@ class HealthRenderer:
         lines = []
 
         # Task info
-        lines.append(f"[bold cyan]Task:[/bold cyan] {node.task_name} ({node.task_id})")
+        lines.append(f"[bold accent]Task:[/bold accent] {node.task_name} ({node.task_id})")
         lines.append(f"[bold]Instance:[/bold] {node.instance_id} ({node.instance_type})")
 
         # Health status
         status_text, status_style = self._format_health_status(node.health_status)
-        score_color = self._get_percentage_color(node.health_score * 100)
+        score_color = self._get_score_color(node.health_score * 100)
         lines.append(
             f"[bold]Health:[/bold] [{status_style}]{status_text}[/{status_style}] "
             f"[{score_color}]{node.health_score * 100:.0f}%[/{score_color}]"
@@ -581,14 +853,14 @@ class HealthRenderer:
 
         return Panel(
             content,
-            title="[bold cyan]Node Health Details[/bold cyan]",
+            title=f"[bold {theme_manager.get_color('accent')}]Node Health Details[/bold {theme_manager.get_color('accent')}]",
             title_align="center",
             border_style=border_style,
             box=box.ROUNDED,
             padding=(1, 2),
         )
 
-    def _format_health_status(self, status: HealthStatus) -> Tuple[str, str]:
+    def _format_health_status(self, status: HealthStatus) -> tuple[str, str]:
         """Format health status with appropriate icon and color."""
         status_map = {
             HealthStatus.HEALTHY: ("● Healthy", "green"),
@@ -608,8 +880,8 @@ class HealthRenderer:
         }
         return icon_map.get(health.lower(), Text("?", style="dim"))
 
-    def _get_percentage_color(self, percentage: float) -> str:
-        """Get color based on percentage value."""
+    def _get_utilization_color(self, percentage: float) -> str:
+        """Color for higher-is-worse utilization metrics (e.g., temp, usage)."""
         if percentage >= 90:
             return "red"
         elif percentage >= 75:
@@ -619,6 +891,17 @@ class HealthRenderer:
         else:
             return "green"
 
+    def _get_score_color(self, percentage: float) -> str:
+        """Color for higher-is-better scores (e.g., health score)."""
+        if percentage >= 90:
+            return "green"
+        elif percentage >= 75:
+            return "white"
+        elif percentage >= 50:
+            return "yellow"
+        else:
+            return "red"
+
     def _get_temperature_color(self, temp_c: float) -> str:
         """Get color based on temperature."""
         if temp_c >= 85:
