@@ -1,16 +1,12 @@
 """Centralized name generation utilities for CLI commands.
 
-Provides consistent unique name generation across all Flow commands
-to avoid repetition and maintain DRY principles.
+Provides consistent unique name generation across Flow commands.
 """
 
 import uuid
-from typing import Optional
 
 
-def generate_unique_name(
-    prefix: str, base_name: Optional[str] = None, add_unique: bool = True
-) -> str:
+def generate_unique_name(prefix: str, base_name: str | None = None, add_unique: bool = True) -> str:
     """Generate a task name with optional unique suffix.
 
     Args:
