@@ -1,7 +1,7 @@
 """Task rendering utilities for CLI output."""
 
 import os
-from typing import Any, Dict, List, Optional, Union
+from typing import Any
 
 from rich import box
 from rich.align import Align
@@ -11,18 +11,17 @@ from rich.table import Table
 from rich.text import Text
 
 from flow.api.models import Task, TaskStatus
-
-from .gpu_formatter import GPUFormatter
-from .task_formatter import TaskFormatter
-from .terminal_adapter import TerminalAdapter
-from .time_formatter import TimeFormatter
-from .theme_manager import theme_manager
+from flow.cli.utils.gpu_formatter import GPUFormatter
+from flow.cli.utils.task_formatter import TaskFormatter
+from flow.cli.utils.terminal_adapter import TerminalAdapter
+from flow.cli.utils.theme_manager import theme_manager
+from flow.cli.utils.time_formatter import TimeFormatter
 
 
 class TaskTableRenderer:
     """Renders task lists as formatted tables with responsive column handling."""
 
-    def __init__(self, console: Optional[Console] = None):
+    def __init__(self, console: Console | None = None):
         """Initialize renderer with optional console override."""
         self.console = console or Console()
         self.terminal = TerminalAdapter()
@@ -32,16 +31,15 @@ class TaskTableRenderer:
 
     def render_task_list(
         self,
-        tasks: List[Task],
-        title: Optional[str] = None,
+        tasks: list[Task],
+        title: str | None = None,
         show_all: bool = False,
         limit: int = 20,
         return_renderable: bool = False,
     ):
-        """Render a list of tasks using the new core table columns.
+        """Render a list of tasks using the unified core table columns.
 
-        This method now delegates to the unified StatusTableRenderer to keep
-        the column definitions and styling DRY across the codebase.
+        Delegates to the shared StatusTableRenderer for consistent columns and styling.
         """
         from flow.cli.utils.status_table_renderer import StatusTableRenderer as CoreStatusTable
 
@@ -49,7 +47,9 @@ class TaskTableRenderer:
         panel = renderer.render(
             tasks,
             me=None,
-            title=(title or f"Tasks (showing up to {limit}{', last 24 hours' if not show_all else ''})"),
+            title=(
+                title or f"Tasks (showing up to {limit}{', last 24 hours' if not show_all else ''})"
+            ),
             wide=False,
             return_renderable=True,
         )
@@ -57,8 +57,20 @@ class TaskTableRenderer:
             return panel
         self.console.print(panel)
 
+    # Public wrappers to provide a stable external API for building tables
+    def create_table(
+        self, title: str | None, layout: dict[str, Any], density_config: dict[str, Any]
+    ) -> Table:
+        return self._create_professional_table(title, layout, density_config)
+
+    def add_responsive_columns(self, table: Table, layout: dict[str, Any]) -> None:
+        self._add_responsive_columns(table, layout)
+
+    def build_row_data(self, task: Task, layout: dict[str, Any]) -> list[str | Align]:
+        return self._build_row_data(task, layout)
+
     def _create_professional_table(
-        self, title: Optional[str], layout: Dict[str, Any], density_config: Dict[str, Any]
+        self, title: str | None, layout: dict[str, Any], density_config: dict[str, Any]
     ) -> Table:
         """Create a professionally styled table.
 
@@ -92,7 +104,7 @@ class TaskTableRenderer:
 
         return table
 
-    def _add_responsive_columns(self, table: Table, layout: Dict[str, Any]) -> None:
+    def _add_responsive_columns(self, table: Table, layout: dict[str, Any]) -> None:
         """Add columns to table based on responsive layout.
 
         Args:
@@ -181,7 +193,6 @@ class TaskTableRenderer:
             Text padded with spaces to center within width
         """
         from rich.console import Console
-        from rich.text import Text
 
         # Create a temporary console to measure text properly
         temp_console = Console(file=None, width=1000)  # Wide console for accurate measurement
@@ -211,7 +222,7 @@ class TaskTableRenderer:
 
         return centered
 
-    def _build_row_data(self, task: Task, layout: Dict[str, Any]) -> List[Union[str, Align]]:
+    def _build_row_data(self, task: Task, layout: dict[str, Any]) -> list[str | Align]:
         """Build row data based on responsive layout configuration.
 
         Args:
@@ -233,22 +244,9 @@ class TaskTableRenderer:
             row.append(name)
 
         if "status" in columns:
-            # Check if we have a more specific instance status to show
-            instance_status = getattr(task, "instance_status", None)
+            from flow.cli.utils.task_formatter import TaskFormatter
 
-            # Determine display status using instance metadata when available
-            if instance_status in ["STATUS_STARTING", "STATUS_INITIALIZING"]:
-                display_status = "starting"
-            elif instance_status == "STATUS_SCHEDULED":
-                display_status = "pending"
-            elif task.status.value == "pending":
-                display_status = "pending"
-            elif task.status.value == "running" and not task.ssh_host:
-                # Only show starting if running but no IP yet
-                display_status = "starting"
-            else:
-                # Default to the task's actual status
-                display_status = task.status.value
+            display_status = TaskFormatter.get_display_status(task)
 
             # Decide compact vs full word based on BOTH layout preference and width budget
             status_col_width = layout.get("status_width", 12)
@@ -342,7 +340,7 @@ class TaskTableRenderer:
 class TaskDetailRenderer:
     """Renders detailed task information in a formatted panel."""
 
-    def __init__(self, console: Optional[Console] = None):
+    def __init__(self, console: Console | None = None):
         """Initialize renderer with optional console override."""
         self.console = console or Console()
         self.time_fmt = TimeFormatter()
@@ -364,22 +362,10 @@ class TaskDetailRenderer:
         if os.environ.get("FLOW_DEBUG") or (task.task_id and not task.task_id.startswith("bid_")):
             lines.append(f"[bold]ID:[/bold] {task.task_id}")
 
-        # Status with color - check if we should show "starting" status
-        instance_status = getattr(task, "instance_status", None)
+        # Status with unified display mapping
+        from flow.cli.utils.task_formatter import TaskFormatter
 
-        # Determine display status based on instance state and task state
-        if instance_status in ["STATUS_STARTING", "STATUS_INITIALIZING"]:
-            display_status = "starting"
-        elif instance_status == "STATUS_SCHEDULED":
-            display_status = "pending"
-        elif task.status.value == "pending":
-            display_status = "pending"
-        elif task.status.value == "running" and not task.ssh_host:
-            # Only show starting if running but no IP yet
-            display_status = "starting"
-        else:
-            # Default to the task's actual status
-            display_status = task.status.value
+        display_status = TaskFormatter.get_display_status(task)
 
         status_text = self.task_fmt.format_status_with_color(display_status)
         lines.append(f"[bold]Status:[/bold] {status_text}")
@@ -399,6 +385,69 @@ class TaskDetailRenderer:
         if task.region:
             lines.append(f"[bold]Region:[/bold] {task.region}")
 
+        # Connection info (show IP/port when available)
+        if getattr(task, "ssh_host", None):
+            if getattr(task, "ssh_port", None):
+                lines.append(f"[bold]IP:[/bold] {task.ssh_host}:{task.ssh_port}")
+            else:
+                lines.append(f"[bold]IP:[/bold] {task.ssh_host}")
+
+        # Endpoints and ports (best-effort)
+        try:
+            # Prefer provider-populated endpoints when available
+            endpoints = getattr(task, "endpoints", None) or {}
+            if isinstance(endpoints, dict) and endpoints:
+                lines.append("")
+                lines.append("[bold]Endpoints:[/bold]")
+                for name, url in endpoints.items():
+                    # Render as link when feasible
+                    if isinstance(url, str) and url.startswith("http"):
+                        lines.append(f"  • {name}: [link]{url}[/link]")
+                    else:
+                        lines.append(f"  • {name}: {url}")
+            else:
+                # Fallback: derive from declared ports if config is attached
+                cfg = getattr(task, "config", None)
+                declared_ports = []
+                if cfg is not None:
+                    declared_ports = list(getattr(cfg, "ports", []) or [])
+
+                if declared_ports:
+                    # Unique, int, sorted
+                    ports_clean: list[int] = []
+                    seen: set[int] = set()
+                    for p in declared_ports:
+                        try:
+                            pi = int(p)
+                        except Exception:
+                            continue
+                        if pi not in seen:
+                            seen.add(pi)
+                            ports_clean.append(pi)
+                    ports_clean.sort()
+
+                    lines.append("")
+                    lines.append("[bold]Ports:[/bold] " + ", ".join(str(p) for p in ports_clean))
+
+                    # If IP is known, show public URLs and an SSH tunnel example
+                    host = getattr(task, "ssh_host", None)
+                    if host:
+                        # Public URLs
+                        url_items = [f"http://{host}:{p}" for p in ports_clean]
+                        if url_items:
+                            lines.append("[bold]Public URLs:[/bold]")
+                            for url in url_items:
+                                lines.append(f"  • [link]{url}[/link]")
+
+                        # Tunnel example (use first port for brevity)
+                        first_port = ports_clean[0]
+                        ssh_user = getattr(task, "ssh_user", "ubuntu")
+                        lines.append("[bold]Tunnel:[/bold] ssh -N -L "
+                                     f"{first_port}:localhost:{first_port} {ssh_user}@{host}")
+        except Exception:
+            # Never fail details rendering because of endpoint/port formatting
+            pass
+
         # Timing info
         if task.created_at:
             lines.append(f"[bold]Created:[/bold] {self.time_fmt.format_time_ago(task.created_at)}")
@@ -467,7 +516,7 @@ class TaskDetailRenderer:
             if metadata.get("web_console_url") and metadata.get("provider") == "mithril":
                 lines.append("")  # Add separator
                 lines.append(
-                    f"[bold]Mithril Console:[/bold] [cyan]{metadata['web_console_url']}[/cyan]"
+                    f"[bold]Mithril Console:[/bold] [link]{metadata['web_console_url']}[/link]"
                 )
 
         # Create panel
