@@ -1,22 +1,25 @@
 """Task selector mixin for commands that operate on tasks.
 
-Provides a clean abstraction for commands that need to select tasks,
-following SOLID principles and avoiding leaky abstractions.
+Provides a small abstraction for commands that need to resolve or select tasks
+interactively, and utilities for common task filters.
 """
 
-from typing import Optional, List, Callable, TypeVar
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
+from collections.abc import Callable
+from typing import TypeVar
 
-import click
-
-from flow import Flow
+try:
+    # Prefer the Flow symbol from status module so tests that patch it are respected consistently
+    from flow.cli.commands.status import Flow  # type: ignore
+except Exception:
+    from flow import Flow
 from flow.api.models import Task
+from flow.cli.utils.interactive_selector import select_task
+from flow.cli.utils.task_resolver import resolve_task_identifier
 from flow.errors import AuthenticationError
 
-from .interactive_selector import select_task
-from .task_resolver import resolve_task_identifier
-
-
 T = TypeVar("T")
 
 
@@ -24,21 +27,21 @@ class TaskFilter:
     """Composable task filters following the Strategy pattern."""
 
     @staticmethod
-    def running_only(tasks: List[Task]) -> List[Task]:
+    def running_only(tasks: list[Task]) -> list[Task]:
         """Filter for running tasks only."""
         from flow.api.models import TaskStatus
 
         return [t for t in tasks if t.status == TaskStatus.RUNNING]
 
     @staticmethod
-    def cancellable(tasks: List[Task]) -> List[Task]:
+    def cancellable(tasks: list[Task]) -> list[Task]:
         """Filter for tasks that can be cancelled."""
         from flow.api.models import TaskStatus
 
         return [t for t in tasks if t.status in [TaskStatus.PENDING, TaskStatus.RUNNING]]
 
     @staticmethod
-    def with_logs(tasks: List[Task]) -> List[Task]:
+    def with_logs(tasks: list[Task]) -> list[Task]:
         """Filter for tasks that have logs available."""
         from flow.api.models import TaskStatus
 
@@ -47,9 +50,27 @@ class TaskFilter:
         return [t for t in tasks if t.status in [TaskStatus.RUNNING, TaskStatus.COMPLETED]]
 
     @staticmethod
-    def with_ssh(tasks: List[Task]) -> List[Task]:
+    def with_ssh(tasks: list[Task]) -> list[Task]:
         """Filter for tasks with SSH access."""
-        return [t for t in tasks if t.status == "RUNNING" and t.ssh_host]
+        try:
+            from flow.api.models import TaskStatus
+
+            return [
+                t
+                for t in tasks
+                if getattr(t, "ssh_host", None) and getattr(t, "status", None) == TaskStatus.RUNNING
+            ]
+        except Exception:
+            # Fallback: compare by value string
+            return [
+                t
+                for t in tasks
+                if getattr(t, "ssh_host", None)
+                and getattr(
+                    getattr(t, "status", None), "value", str(getattr(t, "status", "")).lower()
+                )
+                == "running"
+            ]
 
 
 class TaskSelectorMixin(ABC):
@@ -60,7 +81,7 @@ class TaskSelectorMixin(ABC):
     """
 
     @abstractmethod
-    def get_task_filter(self) -> Optional[Callable[[List[Task]], List[Task]]]:
+    def get_task_filter(self) -> Callable[[list[Task]], list[Task]] | None:
         """Return the filter to apply to tasks, or None for all tasks."""
         pass
 
@@ -75,8 +96,8 @@ class TaskSelectorMixin(ABC):
         pass
 
     def resolve_task(
-        self, task_identifier: Optional[str], client: Flow, allow_multiple: bool = False
-    ) -> Optional[Task] | List[Task]:
+        self, task_identifier: str | None, client: Flow, allow_multiple: bool = False
+    ) -> Task | None | list[Task]:
         """Resolve a task from identifier or interactive selection.
 
         This method encapsulates the common pattern of:
@@ -95,7 +116,7 @@ class TaskSelectorMixin(ABC):
         Raises:
             SystemExit: If task not found or selection cancelled
         """
-        from ..commands.base import console
+        from flow.cli.commands.base import console
 
         # If identifier provided, resolve it directly
         if task_identifier:
@@ -107,7 +128,7 @@ class TaskSelectorMixin(ABC):
 
         # Interactive selection
         # Use TaskFetcher to ensure we find all active tasks
-        from .task_fetcher import TaskFetcher
+        from flow.cli.utils.task_fetcher import TaskFetcher
 
         fetcher = TaskFetcher(client)
         all_tasks = fetcher.fetch_for_resolution(limit=1000)
@@ -149,7 +170,7 @@ class TaskOperationCommand(TaskSelectorMixin):
         """Execute the command logic on the selected task."""
         pass
 
-    def execute_with_selection(self, task_identifier: Optional[str], **kwargs) -> None:
+    def execute_with_selection(self, task_identifier: str | None, **kwargs) -> None:
         """Execute command with task selection.
 
         This is the main entry point that:
@@ -157,8 +178,8 @@ class TaskOperationCommand(TaskSelectorMixin):
         2. Resolves/selects the task
         3. Executes the command logic
         """
-        from ..commands.base import console
-        from .animated_progress import AnimatedEllipsisProgress
+        from flow.cli.commands.base import console
+        from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
 
         try:
             # Check if command manages its own progress
@@ -174,9 +195,10 @@ class TaskOperationCommand(TaskSelectorMixin):
                     with AnimatedEllipsisProgress(
                         console, "Loading tasks", transient=True, start_immediately=True
                     ) as progress:
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Fetch tasks within animation context to show progress
-                        from .task_fetcher import TaskFetcher
+                        from flow.cli.utils.task_fetcher import TaskFetcher
 
                         fetcher = TaskFetcher(client)
                         all_tasks = fetcher.fetch_for_resolution(limit=1000)
@@ -194,7 +216,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                         raise SystemExit(0)
 
                     # Show selector
-                    from .interactive_selector import select_task
+                    from flow.cli.utils.interactive_selector import select_task
 
                     selected = select_task(
                         filtered_tasks, title=self.get_selection_title(), allow_multiple=False
@@ -211,7 +233,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                 display_msg = "Fetching logs"
                 if task_identifier:
                     # Quick cache lookup for better UX
-                    from .task_index_cache import TaskIndexCache
+                    from flow.cli.utils.task_index_cache import TaskIndexCache
 
                     cache = TaskIndexCache()
 
@@ -238,7 +260,8 @@ class TaskOperationCommand(TaskSelectorMixin):
                     with AnimatedEllipsisProgress(
                         console, display_msg, transient=True, start_immediately=True
                     ):
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Resolve task
                         task = self.resolve_task(task_identifier, client)
                         # Execute command logic
@@ -251,9 +274,10 @@ class TaskOperationCommand(TaskSelectorMixin):
                         transient=True,
                         start_immediately=True,
                     ) as progress:
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Fetch tasks within animation context to show progress
-                        from .task_fetcher import TaskFetcher
+                        from flow.cli.utils.task_fetcher import TaskFetcher
 
                         fetcher = TaskFetcher(client)
                         all_tasks = fetcher.fetch_for_resolution(limit=1000)
@@ -271,7 +295,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                         raise SystemExit(0)
 
                     # Show selector
-                    from .interactive_selector import select_task
+                    from flow.cli.utils.interactive_selector import select_task
 
                     selected = select_task(
                         filtered_tasks, title=self.get_selection_title(), allow_multiple=False
@@ -292,7 +316,8 @@ class TaskOperationCommand(TaskSelectorMixin):
                     with AnimatedEllipsisProgress(
                         console, display_msg, transient=True, start_immediately=True
                     ):
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Resolve task
                         task = self.resolve_task(task_identifier, client)
                     # Animation stopped, now execute (which shows confirmation)
@@ -302,9 +327,10 @@ class TaskOperationCommand(TaskSelectorMixin):
                     with AnimatedEllipsisProgress(
                         console, display_msg, transient=True, start_immediately=True
                     ) as progress:
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Fetch tasks within animation context to show progress
-                        from .task_fetcher import TaskFetcher
+                        from flow.cli.utils.task_fetcher import TaskFetcher
 
                         fetcher = TaskFetcher(client)
                         all_tasks = fetcher.fetch_for_resolution(limit=1000)
@@ -322,7 +348,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                         raise SystemExit(0)
 
                     # Show selector
-                    from .interactive_selector import select_task
+                    from flow.cli.utils.interactive_selector import select_task
 
                     selected = select_task(
                         filtered_tasks, title=self.get_selection_title(), allow_multiple=False
@@ -353,7 +379,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                     ) as progress:
                         client = Flow(auto_init=True)
                         # Fetch tasks within animation context to show progress
-                        from .task_fetcher import TaskFetcher
+                        from flow.cli.utils.task_fetcher import TaskFetcher
 
                         fetcher = TaskFetcher(client)
                         all_tasks = fetcher.fetch_for_resolution(limit=1000)
@@ -371,7 +397,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                         raise SystemExit(0)
 
                     # Show selector
-                    from .interactive_selector import select_task
+                    from flow.cli.utils.interactive_selector import select_task
 
                     selected = select_task(
                         filtered_tasks, title=self.get_selection_title(), allow_multiple=False
@@ -393,7 +419,8 @@ class TaskOperationCommand(TaskSelectorMixin):
                         transient=True,
                         start_immediately=True,
                     ):
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Resolve task (from arg or interactive)
                         task = self.resolve_task(task_identifier, client)
                         # Execute command logic
@@ -403,9 +430,10 @@ class TaskOperationCommand(TaskSelectorMixin):
                     with AnimatedEllipsisProgress(
                         console, "Loading tasks", transient=True, start_immediately=True
                     ) as progress:
-                        client = Flow(auto_init=True)
+                        flow_class = kwargs.pop("flow_class", Flow)
+                        client = flow_class(auto_init=True)
                         # Fetch tasks within animation context to show progress
-                        from .task_fetcher import TaskFetcher
+                        from flow.cli.utils.task_fetcher import TaskFetcher
 
                         fetcher = TaskFetcher(client)
                         all_tasks = fetcher.fetch_for_resolution(limit=1000)
@@ -423,7 +451,7 @@ class TaskOperationCommand(TaskSelectorMixin):
                         raise SystemExit(0)
 
                     # Show selector
-                    from .interactive_selector import select_task
+                    from flow.cli.utils.interactive_selector import select_task
 
                     selected = select_task(
                         filtered_tasks, title=self.get_selection_title(), allow_multiple=False
@@ -437,11 +465,30 @@ class TaskOperationCommand(TaskSelectorMixin):
                     self.execute_on_task(selected, client, **kwargs)
 
         except AuthenticationError:
-            console.print("[red]Authentication failed[/red]")
-            console.print("\nPlease run [cyan]flow init[/cyan] to configure your credentials")
-            raise SystemExit(1)
+            # Delegate to the current command's auth handler for consistent UX
+            # This will raise click.exceptions.Exit; do not print fallback to avoid duplication.
+            self.handle_auth_error()  # type: ignore[attr-defined]
+            return
         except SystemExit:
             raise
         except Exception as e:
-            console.print(f"[red]Error: {str(e)}[/red]")
+            # Route through command's centralized error handler when available
+            try:
+                # Detect common auth misconfig pattern that surfaces as ValueError
+                msg = str(e)
+                if (
+                    isinstance(e, ValueError)
+                    and (("Authentication not configured" in msg) or ("MITHRIL_API_KEY" in msg))
+                ) or ("Authentication not configured" in msg):
+                    if hasattr(self, "handle_auth_error"):
+                        self.handle_auth_error()
+                        raise SystemExit(1)
+            except Exception:
+                pass
+
+            if hasattr(self, "handle_error"):
+                # type: ignore[attr-defined]
+                self.handle_error(e)
+            else:
+                console.print(f"[red]Error: {str(e)}[/red]")
             raise SystemExit(1)
