@@ -1,23 +1,24 @@
+from __future__ import annotations
+
 """Provider interface protocols for Flow SDK.
 
-This module defines the core contracts for cloud provider implementations.
-All protocols use structural typing via Python's Protocol class, enabling
-duck typing with static analysis.
-
-The module provides three main protocol categories:
+Defines the core contracts for provider implementations using structural typing
+(`typing.Protocol`). The main categories are:
   - IComputeProvider: GPU instance lifecycle management
   - IStorageProvider: Persistent volume operations
   - IProvider: Combined compute + storage + user management
 
-These protocols enable dependency inversion where core depends on abstractions
-rather than implementations, allowing new providers to extend functionality
-without modifying core SDK code.
+These interfaces decouple the core from concrete providers and enable third
+parties to add implementations without changing the SDK.
 """
 
+from collections.abc import Iterator
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Protocol, Tuple
+from typing import TYPE_CHECKING, Any, Protocol
 
-from flow.api.models import Instance, Task, TaskConfig, TaskStatus, User, Volume
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from flow.api.models import Instance, Task, TaskConfig, TaskStatus, User, Volume
 
 if TYPE_CHECKING:
     from flow.providers.base import ProviderCapabilities
@@ -31,7 +32,7 @@ class IRemoteOperations(Protocol):
     cloud APIs, etc.) while maintaining a consistent interface.
     """
 
-    def execute_command(self, task_id: str, command: str, timeout: Optional[int] = None) -> str:
+    def execute_command(self, task_id: str, command: str, timeout: int | None = None) -> str:
         """Execute a command on a remote task.
 
         Args:
@@ -66,7 +67,7 @@ class IRemoteOperations(Protocol):
         """
         ...
 
-    def open_shell(self, task_id: str, command: Optional[str] = None) -> None:
+    def open_shell(self, task_id: str, command: str | None = None) -> None:
         """Open interactive shell or execute command with TTY.
 
         Args:
@@ -99,8 +100,8 @@ class IComputeProvider(Protocol):
     """
 
     def normalize_instance_request(
-        self, gpu_count: int, gpu_type: Optional[str] = None
-    ) -> Tuple[str, int, Optional[str]]:
+        self, gpu_count: int, gpu_type: str | None = None
+    ) -> tuple[str, int, str | None]:
         """Normalize GPU request to valid instance configuration.
 
         Handles provider-specific constraints like minimum node sizes.
@@ -136,9 +137,9 @@ class IComputeProvider(Protocol):
 
     def find_instances(
         self,
-        requirements: Dict[str, Any],
+        requirements: dict[str, Any],
         limit: int = 10,
-    ) -> List[Instance]:
+    ) -> list[Instance]:
         """Find available GPU instances matching requirements.
 
         Searches provider inventory for instances satisfying all specified
@@ -186,7 +187,7 @@ class IComputeProvider(Protocol):
         self,
         instance_type: str,
         config: TaskConfig,
-        volume_ids: Optional[List[str]] = None,
+        volume_ids: list[str] | None = None,
     ) -> Task:
         """Submit task with automatic instance and region selection.
 
@@ -281,21 +282,6 @@ class IComputeProvider(Protocol):
         """
         ...
 
-    def cancel_task(self, task_id: str) -> bool:
-        """Cancel task execution (deprecated).
-
-        Deprecated:
-            Use stop_task() instead. This method exists only
-            for backward compatibility and will be removed.
-
-        Args:
-            task_id: Task to cancel
-
-        Returns:
-            bool: Result from stop_task()
-        """
-        ...
-
     def get_task_logs(
         self,
         task_id: str,
@@ -359,9 +345,9 @@ class IComputeProvider(Protocol):
 
     def list_tasks(
         self,
-        status: Optional[TaskStatus] = None,
+        status: TaskStatus | list[TaskStatus] | None = None,
         limit: int = 100,
-    ) -> List[Task]:
+    ) -> list[Task]:
         """List tasks with optional status filter.
 
         Retrieves recent tasks for current project/user. Results
@@ -371,7 +357,7 @@ class IComputeProvider(Protocol):
         behavior.
 
         Args:
-            status: Filter by specific status. None returns all.
+            status: Filter by specific status or list of statuses. None returns all.
             limit: Maximum tasks to return. Range: 1-1000.
                 Large limits increase response time.
 
@@ -401,7 +387,7 @@ class IComputeProvider(Protocol):
         """
         ...
 
-    def get_task_instances(self, task_id: str) -> List[Instance]:
+    def get_task_instances(self, task_id: str) -> list[Instance]:
         """Fetch detailed instance information for task.
 
         Retrieves connection details for all instances in a multi-node
@@ -448,7 +434,7 @@ class IStorageProvider(Protocol):
     def create_volume(
         self,
         size_gb: int,
-        name: Optional[str] = None,
+        name: str | None = None,
         interface: str = "block",
     ) -> Volume:
         """Create new persistent storage volume.
@@ -496,7 +482,7 @@ class IStorageProvider(Protocol):
     def list_volumes(
         self,
         limit: int = 100,
-    ) -> List[Volume]:
+    ) -> list[Volume]:
         """List storage volumes in project.
 
         Enumerates all volumes accessible to current credentials.
@@ -518,7 +504,7 @@ class IStorageProvider(Protocol):
         self,
         volume_id: str,
         local_path: Path,
-        remote_path: Optional[str] = None,
+        remote_path: str | None = None,
     ) -> bool:
         """Upload local file to volume.
 
@@ -551,7 +537,7 @@ class IStorageProvider(Protocol):
         self,
         volume_id: str,
         local_path: Path,
-        remote_path: Optional[str] = None,
+        remote_path: str | None = None,
     ) -> bool:
         """Upload directory tree to volume.
 
@@ -630,7 +616,7 @@ class IProvider(IComputeProvider, IStorageProvider):
         - Instance type resolution
     """
 
-    def get_remote_operations(self) -> Optional[IRemoteOperations]:
+    def get_remote_operations(self) -> IRemoteOperations | None:
         """Get remote operations handler if supported.
 
         Returns:
