@@ -231,23 +231,33 @@ class APIError(FlowError):
         message: Human-readable error description.
         status_code: HTTP status code from the failed request.
         response_body: Raw response body for debugging.
+        request_id: Provider request ID/correlation ID for support/debugging.
 
     Attributes:
         status_code: The HTTP status code (e.g., 400, 500).
         response_body: The raw response body text.
+        request_id: Request correlation identifier when available.
 
     Example:
         >>> raise APIError(
         ...     "Server error while creating task",
         ...     status_code=500,
-        ...     response_body='{"error": "Internal server error"}'
+        ...     response_body='{"error": "Internal server error"}',
+        ...     request_id="req_abc123"
         ... )
     """
 
-    def __init__(self, message: str, status_code: int = None, response_body: str = None):
+    def __init__(
+        self,
+        message: str,
+        status_code: int = None,
+        response_body: str = None,
+        request_id: str = None,
+    ):
         super().__init__(message)
         self.status_code = status_code
         self.response_body = response_body
+        self.request_id = request_id
 
 
 class RateLimitError(APIError):
@@ -257,7 +267,7 @@ class RateLimitError(APIError):
     Includes retry_after information when available.
     """
 
-    def __init__(self, message: str, retry_after=None, **kwargs):
+    def __init__(self, message: str, retry_after=None, request_id: str | None = None, **kwargs):
         """Initialize rate limit error.
 
         Args:
@@ -265,7 +275,7 @@ class RateLimitError(APIError):
             retry_after: Seconds to wait before retrying
             **kwargs: Additional error details
         """
-        super().__init__(message, **kwargs)
+        super().__init__(message, request_id=request_id, **kwargs)
         self.retry_after = retry_after
 
 
@@ -310,7 +320,18 @@ class ValidationAPIError(APIError):
             pass
 
         message = self._format_message()
-        super().__init__(message, status_code=422, response_body=response.text)
+        # Capture request/correlation ID when available
+        request_id = None
+        try:
+            request_id = response.headers.get("x-request-id") or response.headers.get(
+                "x-correlation-id"
+            )
+        except Exception:
+            request_id = None
+
+        super().__init__(
+            message, status_code=422, response_body=response.text, request_id=request_id
+        )
 
     def _format_message(self) -> str:
         """Format validation errors into a readable message."""
@@ -512,7 +533,7 @@ class ProviderError(FlowError):
         ...     suggestions=[
         ...         "Set project in flow.yaml configuration",
         ...         "Pass project parameter to Flow constructor",
-        ...         "Set MITHRIL_DEFAULT_PROJECT environment variable"
+        ...         "Set MITHRIL_PROJECT environment variable"
         ...     ]
         ... )
     """
@@ -870,7 +891,7 @@ class NameConflictError(FlowError):
             f"{resource_type.capitalize()} name '{name}' is already in use",
             suggestions=[
                 f"Choose a different name for your {resource_type}",
-                f"Use --force-new to create with a unique suffix",
+                "Use --force-new to create with a unique suffix",
                 f"Check existing {resource_type}s with 'flow list'",
             ],
             error_code="RESOURCE_002",
