@@ -1,18 +1,18 @@
 """Local metrics storage with JSONL format and rotation support.
 
-This module provides efficient local storage for health metrics using JSONL
-(JSON Lines) format, with automatic rotation and cleanup capabilities.
+Efficient local storage for health metrics using JSONL (JSON Lines) format,
+with automatic rotation and cleanup.
 """
 
 import gzip
 import json
 import logging
-import os
 import shutil
 import threading
+from collections.abc import Iterator
 from datetime import datetime, timedelta, timezone
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Tuple
+from typing import Any
 
 import requests
 
@@ -26,7 +26,7 @@ class MetricsStore:
 
     def __init__(
         self,
-        base_path: Optional[Path] = None,
+        base_path: Path | None = None,
         retention_days: int = 7,
         compress_after_days: int = 1,
     ):
@@ -64,7 +64,7 @@ class MetricsStore:
         except Exception as e:
             logger.error(f"Failed to write metrics: {e}")
 
-    def write_raw_metrics(self, metrics: Dict[str, Any]) -> None:
+    def write_raw_metrics(self, metrics: dict[str, Any]) -> None:
         """Write raw metrics dict to storage.
 
         Args:
@@ -90,9 +90,9 @@ class MetricsStore:
 
     def read_snapshots(
         self,
-        start_date: Optional[datetime] = None,
-        end_date: Optional[datetime] = None,
-        task_id: Optional[str] = None,
+        start_date: datetime | None = None,
+        end_date: datetime | None = None,
+        task_id: str | None = None,
     ) -> Iterator[NodeHealthSnapshot]:
         """Read health snapshots from storage.
 
@@ -123,7 +123,7 @@ class MetricsStore:
                                 yield self._dict_to_snapshot(snapshot_dict)
                 else:
                     # Read uncompressed file
-                    with open(file_path, "r") as f:
+                    with open(file_path) as f:
                         for line in f:
                             snapshot_dict = json.loads(line.strip())
                             if not task_id or snapshot_dict.get("task_id") == task_id:
@@ -131,7 +131,7 @@ class MetricsStore:
             except Exception as e:
                 logger.warning(f"Error reading file {file_path}: {e}")
 
-    def cleanup(self) -> Tuple[int, int]:
+    def cleanup(self) -> tuple[int, int]:
         """Clean up old metrics files.
 
         Returns:
@@ -165,7 +165,7 @@ class MetricsStore:
 
         return files_deleted, files_compressed
 
-    def get_storage_stats(self) -> Dict[str, Any]:
+    def get_storage_stats(self) -> dict[str, Any]:
         """Get storage statistics.
 
         Returns:
@@ -201,7 +201,7 @@ class MetricsStore:
         """
         return self.base_path / f"health-metrics-{date_str}.jsonl"
 
-    def _get_files_in_range(self, start_date: datetime, end_date: datetime) -> List[Path]:
+    def _get_files_in_range(self, start_date: datetime, end_date: datetime) -> list[Path]:
         """Get all metrics files in date range.
 
         Args:
@@ -243,7 +243,7 @@ class MetricsStore:
         # Remove original file
         file_path.unlink()
 
-    def _dict_to_snapshot(self, data: Dict[str, Any]) -> NodeHealthSnapshot:
+    def _dict_to_snapshot(self, data: dict[str, Any]) -> NodeHealthSnapshot:
         """Convert dictionary to NodeHealthSnapshot.
 
         This is a simplified conversion - in production you'd want
@@ -262,7 +262,119 @@ class MetricsStore:
         if timestamp.tzinfo is None:
             timestamp = timestamp.replace(tzinfo=timezone.utc)
 
-        # Create snapshot with basic fields
+        # Deserialize nested types
+        from flow.api.health_models import (
+            ComponentHealth,
+            GPUMetric,
+            GPUProcess,
+            HealthState,
+            SystemEvent,
+            SystemMetrics,
+        )
+
+        gpu_metrics = []
+        for g in data.get("gpu_metrics", []) or []:
+            try:
+                processes = []
+                for p in g.get("processes", []) or []:
+                    processes.append(
+                        GPUProcess(
+                            pid=int(p.get("pid", 0)),
+                            name=str(p.get("name", "")),
+                            memory_mb=int(p.get("memory_mb", 0)),
+                            gpu_index=int(p.get("gpu_index", g.get("gpu_index", 0))),
+                        )
+                    )
+                gpu_metrics.append(
+                    GPUMetric(
+                        gpu_index=int(g.get("gpu_index", g.get("index", 0))),
+                        uuid=str(g.get("uuid", "")),
+                        name=str(g.get("name", "Unknown")),
+                        temperature_c=float(g.get("temperature_c", g.get("temperature", 0))),
+                        power_draw_w=float(g.get("power_draw_w", g.get("power_draw", 0))),
+                        power_limit_w=float(g.get("power_limit_w", g.get("power_limit", 0))),
+                        memory_used_mb=int(g.get("memory_used_mb", 0)),
+                        memory_total_mb=int(g.get("memory_total_mb", 0)),
+                        gpu_utilization_pct=float(
+                            g.get("gpu_utilization_pct", g.get("gpu_utilization", 0))
+                        ),
+                        sm_occupancy_pct=float(g.get("sm_occupancy_pct", g.get("sm_occupancy", 0))),
+                        clock_mhz=int(g.get("clock_mhz", 0)),
+                        max_clock_mhz=int(g.get("max_clock_mhz", 0)),
+                        ecc_errors=int(g.get("ecc_errors", 0)),
+                        xid_events=list(g.get("xid_events", []) or []),
+                        nvlink_status=str(g.get("nvlink_status", "healthy")),
+                        processes=processes,
+                    )
+                )
+            except Exception:
+                continue
+
+        system_metrics = None
+        try:
+            sm = data.get("system_metrics")
+            if sm:
+                system_metrics = SystemMetrics(
+                    cpu_usage_pct=float(sm.get("cpu_usage_pct", 0)),
+                    memory_used_gb=float(sm.get("memory_used_gb", 0)),
+                    memory_total_gb=float(sm.get("memory_total_gb", 0)),
+                    disk_usage_pct=float(sm.get("disk_usage_pct", 0)),
+                    network_rx_mbps=float(sm.get("network_rx_mbps", 0)),
+                    network_tx_mbps=float(sm.get("network_tx_mbps", 0)),
+                    open_file_descriptors=int(sm.get("open_file_descriptors", 0)),
+                    load_average=list(sm.get("load_average", []) or []),
+                )
+        except Exception:
+            system_metrics = None
+
+        health_states = []
+        try:
+            for hs in data.get("health_states", []) or []:
+                try:
+                    ts = None
+                    ts_raw = hs.get("timestamp")
+                    if ts_raw:
+                        ts = datetime.fromisoformat(str(ts_raw).replace("Z", "+00:00"))
+                    health = str(hs.get("health", "unknown")).lower()
+                    health_enum = (
+                        ComponentHealth(health)
+                        if health in ComponentHealth._value2member_map_
+                        else ComponentHealth.UNKNOWN
+                    )
+                    health_states.append(
+                        HealthState(
+                            component=str(hs.get("component", "unknown")),
+                            health=health_enum,
+                            message=str(hs.get("message", "")),
+                            severity=str(hs.get("severity", "info")),
+                            timestamp=ts,
+                        )
+                    )
+                except Exception:
+                    continue
+        except Exception:
+            health_states = []
+
+        events = []
+        try:
+            for ev in data.get("events", []) or []:
+                try:
+                    ts = datetime.fromisoformat(str(ev.get("timestamp", "")).replace("Z", "+00:00"))
+                except Exception:
+                    ts = datetime.now(timezone.utc)
+                events.append(
+                    SystemEvent(
+                        timestamp=ts,
+                        component=str(ev.get("component", "unknown")),
+                        level=str(ev.get("level", "info")),
+                        message=str(ev.get("message", "")),
+                        details=dict(ev.get("details", {})),
+                    )
+                )
+        except Exception:
+            events = []
+
+        # Create snapshot with full fields
         snapshot = NodeHealthSnapshot(
             task_id=data.get("task_id", "unknown"),
             task_name=data.get("task_name", "unknown"),
@@ -272,14 +384,23 @@ class MetricsStore:
             gpud_healthy=data.get("gpud_healthy", False),
             gpud_version=data.get("gpud_version"),
             machine_info=data.get("machine_info", {}),
-            # Note: For full implementation, properly deserialize nested objects
-            gpu_metrics=[],  # Would need to convert from dict
-            system_metrics=None,  # Would need to convert from dict
-            health_states=[],  # Would need to convert from dict
-            events=[],  # Would need to convert from dict
+            gpu_metrics=gpu_metrics,
+            system_metrics=system_metrics,
+            health_states=health_states,
+            events=events,
             health_score=data.get("health_score", 0.0),
         )
 
+        # Restore health status if present
+        try:
+            from flow.api.health_models import HealthStatus
+
+            status_str = data.get("health_status")
+            if status_str:
+                snapshot.health_status = HealthStatus(status_str)
+        except Exception:
+            pass
+
         return snapshot
 
 
@@ -298,7 +419,7 @@ class MetricsAggregator:
         self,
         task_id: str,
         hours: int = 24,
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         """Get summary statistics for a specific task.
 
         Args:
@@ -345,7 +466,7 @@ class MetricsAggregator:
             "unhealthy_periods": self._count_unhealthy_periods(snapshots),
         }
 
-    def _count_unhealthy_periods(self, snapshots: List[NodeHealthSnapshot]) -> int:
+    def _count_unhealthy_periods(self, snapshots: list[NodeHealthSnapshot]) -> int:
         """Count number of unhealthy periods in snapshots.
 
         Args:
@@ -382,10 +503,10 @@ class MetricsBatcher:
     def __init__(
         self,
         store: MetricsStore,
-        endpoint: Optional[str] = None,
+        endpoint: str | None = None,
         batch_size: int = 100,
         flush_interval: int = 60,
-        api_key: Optional[str] = None,
+        api_key: str | None = None,
     ):
         """Initialize metrics batcher.
 
@@ -413,7 +534,7 @@ class MetricsBatcher:
             self.session.headers.update(
                 {
                     "Content-Type": "application/json",
-                    "User-Agent": "flow-sdk/1.0",
+                    "User-Agent": "flow-compute/1.0",
                 }
             )
             if self.api_key:
@@ -426,7 +547,7 @@ class MetricsBatcher:
         self.metrics_sent = 0
         self.metrics_failed = 0
 
-    def add(self, metrics: Dict[str, Any]) -> None:
+    def add(self, metrics: dict[str, Any]) -> None:
         """Add metrics to batch, flush if threshold reached.
 
         Args:
@@ -473,7 +594,7 @@ class MetricsBatcher:
             except Exception as e:
                 logger.error(f"Failed to write metric locally: {e}")
 
-    def _send_to_remote(self, metrics: List[Dict[str, Any]]) -> bool:
+    def _send_to_remote(self, metrics: list[dict[str, Any]]) -> bool:
         """Send metrics to remote endpoint.
 
         Args:
@@ -485,7 +606,7 @@ class MetricsBatcher:
         try:
             # Prepare payload
             payload = {
-                "source": "flow-sdk",
+                "source": "flow-compute",
                 "version": "1.0",
                 "metrics": metrics,
             }
@@ -542,7 +663,7 @@ class MetricsBatcher:
         timer.daemon = True
         timer.start()
 
-    def get_stats(self) -> Dict[str, Any]:
+    def get_stats(self) -> dict[str, Any]:
         """Get batcher statistics.
 
         Returns:
