@@ -7,9 +7,9 @@ import subprocess
 import threading
 import time
 from abc import ABC, abstractmethod
+from collections.abc import Callable
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Callable, Dict, Optional
 
 from flow.api.models import TaskConfig
 from flow.providers.local.config import LocalTestConfig
@@ -18,8 +18,8 @@ logger = logging.getLogger(__name__)
 
 # Try to import Mithril startup builder for production startup scripts
 try:
-    from flow.providers.mithril.runtime.startup.sections import ScriptContext
     from flow.providers.mithril.runtime.startup.builder import MithrilStartupScriptBuilder
+    from flow.providers.mithril.runtime.startup.sections import ScriptContext
 
     HAS_MITHRIL_STARTUP_BUILDER = True
 except ImportError:
@@ -32,17 +32,16 @@ class TaskExecution:
     """Represents a running task execution."""
 
     task_id: str
-    container_id: Optional[str] = None
-    process_id: Optional[str] = None
-    process: Optional[subprocess.Popen] = None
+    container_id: str | None = None
+    process_id: str | None = None
+    process: subprocess.Popen | None = None
 
     def wait(self) -> int:
         """Wait for execution to complete and return exit code."""
         if self.container_id:
             # Docker container
-            import docker
-
-            client = docker.from_env()
+            docker_mod = __import__("docker")  # type: ignore
+            client = docker_mod.from_env()
             container = client.containers.get(self.container_id)
             result = container.wait()
             return result["StatusCode"]
@@ -58,7 +57,7 @@ class TaskExecutor(ABC):
 
     def __init__(self, config: LocalTestConfig):
         self.config = config
-        self.executions: Dict[str, TaskExecution] = {}
+        self.executions: dict[str, TaskExecution] = {}
 
     @abstractmethod
     def execute_task(
@@ -66,7 +65,7 @@ class TaskExecutor(ABC):
         task_id: str,
         config: TaskConfig,
         resources: dict,
-        log_callback: Optional[Callable[[str], None]] = None,
+        log_callback: Callable[[str], None] | None = None,
     ) -> TaskExecution:
         """Execute a task with given configuration."""
         pass
@@ -89,23 +88,29 @@ class ContainerTaskExecutor(TaskExecutor):
         super().__init__(config)
 
         # Initialize Docker client
-        import docker
-
-        self.client = docker.from_env()
+        try:
+            docker_mod = __import__("docker")  # type: ignore
+            self.client = docker_mod.from_env()
+            self._docker_sdk_available = True
+        except Exception:
+            # Defer errors until first use to avoid import warnings during lint
+            self._docker_sdk_available = False
+            self.client = None  # type: ignore
 
         # Create test network
         self.network_name = "flow-test-network"
-        try:
-            self.network = self.client.networks.get(self.network_name)
-        except docker.errors.NotFound:
-            self.network = self.client.networks.create(self.network_name, driver="bridge")
+        if self._docker_sdk_available and self.client is not None:
+            try:
+                self.network = self.client.networks.get(self.network_name)
+            except Exception:
+                self.network = self.client.networks.create(self.network_name, driver="bridge")
 
     def execute_task(
         self,
         task_id: str,
         config: TaskConfig,
         resources: dict,
-        log_callback: Optional[Callable[[str], None]] = None,
+        log_callback: Callable[[str], None] | None = None,
     ) -> TaskExecution:
         """Execute task in Docker container."""
         # Choose image: use task's image if specified, otherwise default
@@ -127,7 +132,7 @@ class ContainerTaskExecutor(TaskExecutor):
                 "flow.task_id": task_id,
                 "flow.task_name": config.name,
             },
-            "environment": self._build_environment(config),
+            "environment": self._build_environment(config, task_id),
             "volumes": self._build_volumes(config),
         }
 
@@ -142,9 +147,14 @@ class ContainerTaskExecutor(TaskExecutor):
 
         # Add GPU support if needed
         if resources.get("gpu_count", 0) > 0:
-            container_config["device_requests"] = [
-                docker.types.DeviceRequest(count=resources["gpu_count"], capabilities=[["gpu"]])
-            ]
+            # Lazy import for type to avoid linter import resolution issues in environments without docker SDK
+            try:
+                docker_mod = __import__("docker")  # type: ignore
+                container_config["device_requests"] = [
+                    docker_mod.types.DeviceRequest(count=resources["gpu_count"], capabilities=[["gpu"]])
+                ]
+            except Exception:
+                pass
 
         # Handle command execution based on type
         needs_script = False
@@ -167,7 +177,9 @@ class ContainerTaskExecutor(TaskExecutor):
             # Use sh for alpine-based images, bash for others
             shell = "sh" if image and "alpine" in image else "bash"
             # Use a unique path inside the container to avoid conflicts
-            container_script_path = f"/tmp/flow-startup-{task_id}.sh"
+            from flow.core.paths import STARTUP_SCRIPT_PREFIX
+
+            container_script_path = f"{STARTUP_SCRIPT_PREFIX}{task_id}.sh"
             container_config["command"] = [shell, container_script_path]
             # Add script to volumes
             container_config["volumes"][str(script_path)] = {
@@ -192,10 +204,10 @@ class ContainerTaskExecutor(TaskExecutor):
             logger.error(f"Failed to start container for task {task_id}: {e}")
             raise
 
-    def _build_environment(self, config: TaskConfig) -> Dict[str, str]:
+    def _build_environment(self, config: TaskConfig, task_id: str) -> dict[str, str]:
         """Build environment variables for container."""
         env = {
-            "FLOW_TASK_ID": config.name,
+            "FLOW_TASK_ID": task_id,
             "FLOW_TASK_NAME": config.name,
             "FLOW_NODE_RANK": "0",
             "FLOW_NODE_COUNT": str(config.num_instances),
@@ -226,7 +238,7 @@ class ContainerTaskExecutor(TaskExecutor):
 
         return env
 
-    def _build_volumes(self, config: TaskConfig) -> Dict[str, dict]:
+    def _build_volumes(self, config: TaskConfig) -> dict[str, dict]:
         """Build volume mounts for container."""
         volumes = {}
 
@@ -289,6 +301,21 @@ class ContainerTaskExecutor(TaskExecutor):
             "echo 'Instance type: Local Docker'",
             "echo",
             "",
+            "# Resolve python interpreter for portability",
+            "PY_BIN=\"${PYTHON:-}\"",
+            "if [ -z \"$PY_BIN\" ]; then",
+            "  if command -v python3 >/dev/null 2>&1; then PY_BIN=python3;",
+            "  elif command -v python >/dev/null 2>&1; then PY_BIN=python;",
+            "  else PY_BIN=echo; fi",
+            "fi",
+            "export PYTHON=\"$PY_BIN\"",
+            "# Provide a python shim if python isn't on PATH",
+            "if ! command -v python >/dev/null 2>&1; then",
+            "  if [ -n \"$PYTHON\" ] && command -v \"$PYTHON\" >/dev/null 2>&1; then",
+            "    python() { command \"$PYTHON\" \"$@\"; }",
+            "  fi",
+            "fi",
+            "",
             "# User script",
         ]
 
@@ -298,8 +325,13 @@ class ContainerTaskExecutor(TaskExecutor):
                 # String command - add directly (it's a script)
                 script_lines.append(config.command)
             elif isinstance(config.command, list):
-                # List command - convert to shell command
-                script_lines.append(" ".join(shlex.quote(arg) for arg in config.command))
+                # List command - convert to shell command; map python→$PYTHON for portability
+                parts = [shlex.quote(str(arg)) for arg in config.command]
+                if parts:
+                    first_raw = config.command[0]
+                    if first_raw in ("python", "python3", "/usr/bin/python", "/usr/bin/python3"):
+                        parts[0] = "$PYTHON"
+                script_lines.append(" ".join(parts))
 
         script_lines.extend(
             [
@@ -368,7 +400,7 @@ class ProcessTaskExecutor(TaskExecutor):
         task_id: str,
         config: TaskConfig,
         resources: dict,
-        log_callback: Optional[Callable[[str], None]] = None,
+        log_callback: Callable[[str], None] | None = None,
     ) -> TaskExecution:
         """Execute task as local process."""
         # Create working directory
@@ -475,6 +507,21 @@ class ProcessTaskExecutor(TaskExecutor):
             "echo 'Instance type: Local Process'",
             "echo",
             "",
+            "# Resolve python interpreter for portability",
+            "PY_BIN=\"${PYTHON:-}\"",
+            "if [ -z \"$PY_BIN\" ]; then",
+            "  if command -v python3 >/dev/null 2>&1; then PY_BIN=python3;",
+            "  elif command -v python >/dev/null 2>&1; then PY_BIN=python;",
+            "  else PY_BIN=echo; fi",
+            "fi",
+            "export PYTHON=\"$PY_BIN\"",
+            "# Provide a python shim if python isn't on PATH",
+            "if ! command -v python >/dev/null 2>&1; then",
+            "  if [ -n \"$PYTHON\" ] && command -v \"$PYTHON\" >/dev/null 2>&1; then",
+            "    python() { command \"$PYTHON\" \"$@\"; }",
+            "  fi",
+            "fi",
+            "",
             "# User script",
         ]
 
