@@ -4,14 +4,16 @@ import logging
 import subprocess
 import time
 import uuid
+from collections.abc import Iterator
 from datetime import datetime, timezone
-from typing import Any, Dict, Iterator, List, Optional
+from pathlib import Path
+from typing import Any
 
 from flow._internal.config import Config
-from flow.api.models import AvailableInstance, Task, TaskConfig, TaskStatus, Volume, VolumeSpec
+from flow.api.models import AvailableInstance, Task, TaskConfig, TaskStatus, Volume
 from flow.core.provider_interfaces import IComputeProvider
-from flow.providers.interfaces import IProviderInit
 from flow.errors import FlowError, TaskNotFoundError, VolumeError
+from flow.providers.interfaces import IProviderInit
 from flow.providers.local.config import LocalTestConfig
 from flow.providers.local.executor import ContainerTaskExecutor, ProcessTaskExecutor, TaskExecutor
 from flow.providers.local.logs import LocalLogManager
@@ -42,7 +44,7 @@ class LocalProvider(IComputeProvider):
         # Extract local-specific configuration from provider_config
         # For now, use defaults - could be extended to read from config.provider_config
         self.local_config = LocalTestConfig.default()
-        self.tasks: Dict[str, Task] = {}
+        self.tasks: dict[str, Task] = {}
         self.storage = LocalStorage(self.local_config.storage_dir)
         self.log_manager = LocalLogManager(self.local_config.storage_dir)
 
@@ -87,9 +89,12 @@ class LocalProvider(IComputeProvider):
             try:
                 subprocess.run(["docker", "version"], capture_output=True, check=True)
             except (subprocess.CalledProcessError, FileNotFoundError) as e:
-                raise FlowError(
-                    "Docker not available. Install Docker or use use_docker=False"
-                ) from e
+                # Fall back to process executor if Docker is not available
+                logger.warning(
+                    "Docker not available (%s), falling back to process executor", e
+                )
+                self.local_config.use_docker = False
+                self.executor = ProcessTaskExecutor(self.local_config)
 
         logger.info(f"LocalProvider initialized with executor: {type(self.executor).__name__}")
 
@@ -97,7 +102,7 @@ class LocalProvider(IComputeProvider):
         self,
         instance_type: str,
         config: TaskConfig,
-        volume_ids: Optional[List[str]] = None,
+        volume_ids: list[str] | None = None,
     ) -> Task:
         """Submit a task for local execution.
 
@@ -222,7 +227,7 @@ class LocalProvider(IComputeProvider):
             raise TaskNotFoundError(f"Task {task_id} not found")
         return self.tasks[task_id]
 
-    def list_tasks(self, status: Optional[TaskStatus] = None, limit: int = 100) -> List[Task]:
+    def list_tasks(self, status: TaskStatus | list[TaskStatus] | None = None, limit: int = 100) -> list[Task]:
         """List tasks with optional filtering.
 
         Args:
@@ -234,9 +239,13 @@ class LocalProvider(IComputeProvider):
         """
         tasks = list(self.tasks.values())
 
-        # Filter by status
+        # Filter by status or list of statuses
         if status:
-            tasks = [t for t in tasks if t.status == status]
+            if isinstance(status, list):
+                allowed = set(status)
+                tasks = [t for t in tasks if t.status in allowed]
+            else:
+                tasks = [t for t in tasks if t.status == status]
 
         # Sort by creation time (newest first)
         tasks.sort(key=lambda t: t.created_at, reverse=True)
@@ -315,41 +324,38 @@ class LocalProvider(IComputeProvider):
                 # Yield any final logs
                 final_logs = self.log_manager.get_logs(task_id, tail=10, log_type=log_type)
                 for line in final_logs.splitlines()[-5:]:
-                    if line not in self.log_manager._streamed_lines.get(task_id, set()):
+                    if not self.log_manager.has_streamed_line(task_id, line):
                         yield line
                 break
 
-    def create_volume(self, spec: VolumeSpec) -> Volume:
-        """Create a local volume.
-
-        Args:
-            spec: Volume specification
-
-        Returns:
-            Volume object
-        """
+    def create_volume(
+        self, size_gb: int, name: str | None = None, interface: str = "block"
+    ) -> Volume:
+        """Create a local volume matching IStorageProvider signature."""
         volume_id = f"local-vol-{uuid.uuid4().hex[:8]}"
-        volume_path = self.storage.create_volume(volume_id, spec.size_gb)
+        volume_path = self.storage.create_volume(volume_id, size_gb)
 
         from flow.api.models import StorageInterface
 
+        iface = StorageInterface.BLOCK if interface == "block" else StorageInterface.FILE
+
         return Volume(
             volume_id=volume_id,
-            name=spec.name or volume_id,
-            size_gb=spec.size_gb,
+            name=name or volume_id,
+            size_gb=size_gb,
             region="local",
-            interface=spec.interface or StorageInterface.BLOCK,
+            interface=iface,
             created_at=datetime.now(timezone.utc),
-            provider_data={"path": str(volume_path), "mount_path": spec.mount_path},
+            provider_data={"path": str(volume_path)},
         )
 
-    def delete_volume(self, volume_id: str) -> None:
-        """Delete a volume.
-
-        Args:
-            volume_id: Volume to delete
-        """
-        self.storage.delete_volume(volume_id)
+    def delete_volume(self, volume_id: str) -> bool:
+        """Delete a volume and return operation status."""
+        try:
+            self.storage.delete_volume(volume_id)
+            return True
+        except Exception:
+            return False
 
     def resize_volume(self, volume_id: str, new_size_gb: int) -> None:
         """Resize a volume.
@@ -394,7 +400,7 @@ class LocalProvider(IComputeProvider):
             provider_data=volume_info,
         )
 
-    def list_volumes(self, limit: int = 100) -> List[Volume]:
+    def list_volumes(self, limit: int = 100) -> list[Volume]:
         """List all volumes.
 
         Args:
@@ -412,6 +418,75 @@ class LocalProvider(IComputeProvider):
 
         return volumes
 
+    # ---- File transfer helpers for local volumes ----
+
+    def upload_file(
+        self, volume_id: str, local_path: "Path", remote_path: str | None = None
+    ) -> bool:
+        try:
+            vol = self.get_volume(volume_id)
+            target = Path(vol.provider_data.get("path", "")) / (
+                remote_path or Path(local_path).name
+            )
+            target.parent.mkdir(parents=True, exist_ok=True)
+            from shutil import copy2
+
+            copy2(local_path, target)
+            return True
+        except Exception:
+            return False
+
+    def upload_directory(
+        self, volume_id: str, local_path: "Path", remote_path: str | None = None
+    ) -> bool:
+        try:
+            vol = self.get_volume(volume_id)
+            base = Path(vol.provider_data.get("path", "")) / (remote_path or "")
+            base.mkdir(parents=True, exist_ok=True)
+            from shutil import copy2
+
+            for f in Path(local_path).rglob("*"):
+                if f.is_file():
+                    rel = f.relative_to(local_path)
+                    dst = base / rel
+                    dst.parent.mkdir(parents=True, exist_ok=True)
+                    copy2(f, dst)
+            return True
+        except Exception:
+            return False
+
+    def download_file(self, volume_id: str, remote_path: str, local_path: "Path") -> bool:
+        try:
+            vol = self.get_volume(volume_id)
+            src = Path(vol.provider_data.get("path", "")) / remote_path
+            Path(local_path).parent.mkdir(parents=True, exist_ok=True)
+            from shutil import copy2
+
+            copy2(src, local_path)
+            return True
+        except Exception:
+            return False
+
+    def download_directory(self, volume_id: str, remote_path: str, local_path: "Path") -> bool:
+        try:
+            vol = self.get_volume(volume_id)
+            src_base = Path(vol.provider_data.get("path", "")) / (remote_path or "")
+            Path(local_path).mkdir(parents=True, exist_ok=True)
+            from shutil import copy2
+
+            for f in src_base.rglob("*"):
+                if f.is_file():
+                    rel = f.relative_to(src_base)
+                    dst = Path(local_path) / rel
+                    dst.parent.mkdir(parents=True, exist_ok=True)
+                    copy2(f, dst)
+            return True
+        except Exception:
+            return False
+
+    def is_volume_id(self, identifier: str) -> bool:
+        return identifier.startswith("local-vol-")
+
     def cleanup(self):
         """Clean up all local resources."""
         # Stop all running tasks
@@ -455,9 +530,9 @@ class LocalProvider(IComputeProvider):
 
     def find_instances(
         self,
-        requirements: Dict[str, Any],
+        requirements: dict[str, Any],
         limit: int = 10,
-    ) -> List["AvailableInstance"]:
+    ) -> list["AvailableInstance"]:
         """Find available instances matching requirements.
 
         For local provider, we always have one "instance" available - the local machine.
@@ -492,6 +567,6 @@ class LocalProvider(IComputeProvider):
         Returns:
             IProviderInit implementation for local provider
         """
-        from .init import LocalInit
+        from flow.providers.local.init import LocalInit
 
         return LocalInit()
