@@ -5,14 +5,13 @@ operations using the Mithril API. It supports market-based resource allocation
 through auctions.
 """
 
+from flow.providers.mithril.manifest import MITHRIL_MANIFEST
+from flow.providers.mithril.provider import MithrilProvider
 from flow.providers.registry import ProviderRegistry
 
-from .manifest import MITHRIL_MANIFEST
-from .provider import MithrilProvider
-
 # Import from the direct module, not the setup subpackage
 try:
-    from .setup import MithrilProviderSetup
+    from flow.providers.mithril.setup import MithrilProviderSetup
 except ImportError:
     # Fallback if setup module causes issues
     MithrilProviderSetup = None
