@@ -16,10 +16,18 @@ class MithrilError(FlowError):
 class MithrilAPIError(MithrilError):
     """Error communicating with Mithril API."""
 
-    def __init__(self, message: str, status_code: int = None, response_body: str = None):
-        super().__init__(message)
+    def __init__(
+        self,
+        message: str,
+        status_code: int | None = None,
+        response_body: str | None = None,
+        request_id: str | None = None,
+        suggestions: list[str] | None = None,
+    ):
+        super().__init__(message, suggestions=suggestions)
         self.status_code = status_code
         self.response_body = response_body
+        self.request_id = request_id
 
 
 class MithrilAuthenticationError(MithrilAPIError):
