@@ -1,29 +1,27 @@
 """Mithril-specific implementation of remote operations via SSH.
 
-This module provides SSH-based remote operations for Mithril tasks,
-implementing the IRemoteOperations protocol with Mithril-specific behavior.
+SSH-based remote operations for Mithril tasks implementing ``IRemoteOperations``.
 """
 
 import logging
 import subprocess
 import time
 from pathlib import Path
-from typing import TYPE_CHECKING, Optional
+import shlex
+from typing import TYPE_CHECKING
 
 from flow.core.provider_interfaces import IRemoteOperations
 from flow.errors import FlowError
-from .ssh_waiter import ExponentialBackoffSSHWaiter
-from .ssh_utils import wait_for_task_ssh_info, SSHNotReadyError
-from .core.constants import (
-    SSH_READY_WAIT_SECONDS,
-    SSH_CHECK_INTERVAL,
+from flow.providers.mithril.core.constants import (
+    EXPECTED_PROVISION_MINUTES,
     SSH_QUICK_RETRY_ATTEMPTS,
     SSH_QUICK_RETRY_MAX_SECONDS,
-    EXPECTED_PROVISION_MINUTES,
+    SSH_READY_WAIT_SECONDS,
 )
+from flow.providers.mithril.ssh_utils import SSHNotReadyError, wait_for_task_ssh_info
 
 if TYPE_CHECKING:
-    from .provider import MithrilProvider
+    from flow.providers.mithril.provider import MithrilProvider
 
 logger = logging.getLogger(__name__)
 
@@ -51,7 +49,7 @@ class MithrilRemoteOperations(IRemoteOperations):
         """
         self.provider = provider
 
-    def execute_command(self, task_id: str, command: str, timeout: Optional[int] = None) -> str:
+    def execute_command(self, task_id: str, command: str, timeout: int | None = None) -> str:
         """Execute command on remote task via SSH.
 
         Args:
@@ -115,8 +113,8 @@ class MithrilRemoteOperations(IRemoteOperations):
                 if instance_status == "STATUS_STARTING":
                     # Instance is explicitly in starting state
                     raise RemoteExecutionError(
-                        f"Instance is starting up. SSH will be available once startup completes. "
-                        f"Please try again in a moment or check 'flow status' for current state."
+                        "Instance is starting up. SSH will be available once startup completes. "
+                        "Please try again in a moment or check 'flow status' for current state."
                     )
                 elif (
                     instance_age_minutes is not None
@@ -164,8 +162,8 @@ class MithrilRemoteOperations(IRemoteOperations):
                 stderr = result.stderr.lower()
                 if "connection closed" in stderr or "connection reset" in stderr:
                     raise RemoteExecutionError(
-                        f"SSH connection was closed. The instance may still be starting up. "
-                        f"Please wait a moment and try again."
+                        "SSH connection was closed. The instance may still be starting up. "
+                        "Please wait a moment and try again."
                     )
                 raise RemoteExecutionError(f"Command failed: {result.stderr}")
             return result.stdout
@@ -191,16 +189,16 @@ class MithrilRemoteOperations(IRemoteOperations):
         """
         # Use SSH cat to retrieve file
         try:
-            output = self.execute_command(task_id, f"cat {remote_path}")
+            # Quote remote path to prevent remote shell injection
+            safe_remote_path = shlex.quote(str(remote_path))
+            output = self.execute_command(task_id, f"cat {safe_remote_path}")
             return output.encode("utf-8")
         except RemoteExecutionError as e:
             if "No such file" in str(e) or "cannot open" in str(e):
                 raise FileNotFoundError(f"Remote file not found: {remote_path}") from e
             raise
 
-    def open_shell(
-        self, task_id: str, command: Optional[str] = None, progress_context=None
-    ) -> None:
+    def open_shell(self, task_id: str, command: str | None = None, progress_context=None) -> None:
         """Open interactive SSH shell to remote task.
 
         Args:
@@ -212,8 +210,8 @@ class MithrilRemoteOperations(IRemoteOperations):
             RemoteExecutionError: Shell access failed
         """
         # Try cache first for instant response
-        from flow.cli.utils.task_index_cache import TaskIndexCache
         from flow.cli.utils.ssh_key_cache import SSHKeyCache
+        from flow.cli.utils.task_index_cache import TaskIndexCache
 
         task_cache = TaskIndexCache()
         ssh_cache = SSHKeyCache()
@@ -263,9 +261,10 @@ class MithrilRemoteOperations(IRemoteOperations):
             # Don't immediately close progress_context - let it show while we wait for SSH
             # We'll close it later, right before starting the SSH subprocess
 
+            import signal
+
             from rich.console import Console
             from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
-            import signal
 
             console = Console()
 
@@ -274,6 +273,7 @@ class MithrilRemoteOperations(IRemoteOperations):
             is_paused_or_resuming = False
             if hasattr(task, "created_at") and task.created_at:
                 from datetime import datetime, timezone
+
                 from flow.api.models import TaskStatus
 
                 age = task.instance_age_seconds
@@ -288,38 +288,35 @@ class MithrilRemoteOperations(IRemoteOperations):
                     if paused_age < 300:  # Within 5 minutes of being paused
                         is_paused_or_resuming = True
 
-            # Determine appropriate timeout and message based on context
+            # Determine appropriate timeout and message based on context (non-flickering)
             if is_paused_or_resuming:
-                # For paused/resuming instances
                 ssh_wait_timeout = 120  # 2 minutes for resumed instances
                 initial_message = (
-                    f"\n[yellow]Instance is resuming from paused state[/yellow] "
-                    f"(IP: {task.ssh_host})"
-                )
-                timeout_message = (
-                    "SSH service needs to restart after resume. This typically takes 1-2 minutes."
+                    f"\n[yellow]Instance resuming[/yellow] (IP: {task.ssh_host})"
                 )
+                timeout_message = "SSH restarts after resume; usually 1–2 minutes."
             elif task_age_minutes > 30:
-                # For long-running instances, use a shorter timeout since SSH should already be ready
-                # But still give it a chance in case SSH service is temporarily unavailable
                 ssh_wait_timeout = 60  # 1 minute for established instances
                 initial_message = (
-                    f"\n[yellow]Reconnecting to established instance[/yellow] "
-                    f"(IP: {task.ssh_host}, age: {task_age_minutes} minutes)"
+                    f"\n[yellow]Checking SSH on established instance[/yellow] (IP: {task.ssh_host})"
                 )
                 timeout_message = "Checking SSH availability..."
             else:
-                # For new instances, use the standard timeout
                 ssh_wait_timeout = SSH_READY_WAIT_SECONDS
                 initial_message = (
                     f"\n[yellow]Instance is starting up[/yellow] (IP: {task.ssh_host})"
                 )
-                timeout_message = f"This can take up to {EXPECTED_PROVISION_MINUTES} minutes for Mithril instances."
+                timeout_message = (
+                    f"This can take up to {EXPECTED_PROVISION_MINUTES} minutes for Mithril instances."
+                )
 
-            # Show initial status
-            console.print(initial_message)
-            console.print(timeout_message)
-            console.print("[dim]Press Ctrl+C to cancel[/dim]\n")
+            # Show initial status (only when no external progress handler is managing UI)
+            if progress_context is None:
+                console.print(initial_message)
+                console.print(timeout_message)
+                console.print(
+                    "[dim]Press Ctrl+C to cancel this SSH attempt (instance continues starting)[/dim]\n"
+                )
 
             # Flag for graceful shutdown
             interrupted = False
@@ -355,20 +352,20 @@ class MithrilRemoteOperations(IRemoteOperations):
                         if self._is_ssh_ready(task, ssh_key_path):
                             ssh_ready = True
                             break
-
-                        # Exponential backoff with cap
-                        wait_time = min(2 ** (attempts / 10), 10)  # Cap at 10 seconds
-                        wait_start = time.time()
-                        while time.time() - wait_start < wait_time and not interrupted:
-                            time.sleep(0.1)  # Check for interrupt every 100ms
-
+                        # Deterministic backoff with frequent interrupt checks
+                        wait_time = min(0.5 * (1 + attempts), 5.0)
+                        slices = int(wait_time / 0.1)
+                        for _ in range(max(1, slices)):
+                            if interrupted:
+                                break
+                            time.sleep(0.1)
                         attempts += 1
 
                     # Handle results after the loop
                     if not interrupted and not ssh_ready:
                         # Continue to timeout handling below
                         pass
-                else:
+                elif progress_context is None:
                     # Original Progress display for backward compatibility
                     with Progress(
                         SpinnerColumn(),
@@ -377,9 +374,7 @@ class MithrilRemoteOperations(IRemoteOperations):
                         console=console,
                         transient=True,
                     ) as progress:
-                        progress_task = progress.add_task(
-                            "Waiting for SSH to be ready...", total=None
-                        )
+                        progress_task = progress.add_task("Waiting for SSH to be ready...", total=None)
 
                         start_time = time.time()
                         attempts = 0
@@ -411,63 +406,72 @@ class MithrilRemoteOperations(IRemoteOperations):
                                 )
                                 break
 
-                            # Exponential backoff with cap, but check for interrupts
-                            wait_time = min(2 ** (attempts / 10), 10)  # Cap at 10 seconds
-                            wait_start = time.time()
-                            while time.time() - wait_start < wait_time and not interrupted:
-                                time.sleep(0.1)  # Check for interrupt every 100ms
+                            # Deterministic backoff with frequent interrupt checks
+                            wait_time = min(0.5 * (1 + attempts), 5.0)
+                            slices = int(wait_time / 0.1)
+                            for _ in range(max(1, slices)):
+                                if interrupted:
+                                    break
+                                time.sleep(0.1)
 
                     if interrupted:
-                        progress.stop()
-                        console.print("\n[yellow]SSH connection cancelled by user[/yellow]")
-                        console.print("\nYou can reconnect later with:")
-                        console.print(f"  [cyan]flow ssh {task.name or task_id}[/cyan]")
+                        try:
+                            progress.stop()
+                        except Exception:
+                            pass
+                        if progress_context is None:
+                            console.print("\n[yellow]SSH connection cancelled by user[/yellow]")
+                            console.print("\nYou can reconnect later with:")
+                            console.print(f"  [cyan]flow ssh {task.name or task_id}[/cyan]")
                         raise KeyboardInterrupt("User cancelled SSH connection")
 
                     if time.time() - start_time >= ssh_wait_timeout:
                         # Timeout
-                        console.print("\n[red]✗ SSH connection timed out[/red]")
+                        if progress_context is None:
+                            console.print("\n[red]✗ SSH connection timed out[/red]")
 
                         if is_paused_or_resuming:
                             # For paused/resuming instances
-                            console.print(
-                                f"\n[yellow]SSH service is still restarting after resume[/yellow]"
-                            )
-                            console.print("\nThis can happen when:")
-                            console.print("  • The instance was paused for an extended period")
-                            console.print("  • System services are still initializing")
-                            console.print("  • Container runtime is restarting")
-                            console.print("\nRecommended actions:")
-                            console.print(
-                                f"  • Wait another minute and retry: [cyan]flow ssh {task.name or task_id}[/cyan]"
-                            )
-                            console.print(
-                                f"  • Check instance status: [cyan]flow status {task.name or task_id}[/cyan]"
-                            )
-                            console.print(
-                                f"  • View logs: [cyan]flow logs {task.name or task_id}[/cyan]"
-                            )
+                            if progress_context is None:
+                                console.print(
+                                    "\n[yellow]SSH service is still restarting after resume[/yellow]"
+                                )
+                                console.print("\nThis can happen when:")
+                                console.print("  • The instance was paused for an extended period")
+                                console.print("  • System services are still initializing")
+                                console.print("  • Container runtime is restarting")
+                                console.print("\nRecommended actions:")
+                                console.print(
+                                    f"  • Wait another minute and retry: [cyan]flow ssh {task.name or task_id}[/cyan]"
+                                )
+                                console.print(
+                                    f"  • Check instance status: [cyan]flow status {task.name or task_id}[/cyan]"
+                                )
+                                console.print(
+                                    f"  • View logs: [cyan]flow logs {task.name or task_id}[/cyan]"
+                                )
                             error_msg = "SSH not ready yet after resume. Please wait a moment and try again."
                         elif task_age_minutes > 30:
                             # For long-running instances, show different message
-                            console.print(
-                                f"\n[red]SSH service is not responding on established instance[/red]"
-                            )
-                            console.print(f"Instance age: {task_age_minutes} minutes")
-                            console.print("\nThis is unexpected. Possible causes:")
-                            console.print("  • SSH service crashed or was stopped")
-                            console.print("  • Instance was preempted and recreated")
-                            console.print("  • Container or system issue")
-                            console.print("\nRecommended actions:")
-                            console.print(
-                                f"  • Check instance health: [cyan]flow health {task.name or task_id}[/cyan]"
-                            )
-                            console.print(
-                                f"  • View logs: [cyan]flow logs {task.name or task_id}[/cyan]"
-                            )
-                            console.print(
-                                f"  • Restart the instance: [cyan]flow cancel {task.name or task_id} && flow dev[/cyan]"
-                            )
+                            if progress_context is None:
+                                console.print(
+                                    "\n[red]SSH service is not responding on established instance[/red]"
+                                )
+                                console.print(f"Instance age: {task_age_minutes} minutes")
+                                console.print("\nThis is unexpected. Possible causes:")
+                                console.print("  • SSH service crashed or was stopped")
+                                console.print("  • Instance was preempted and recreated")
+                                console.print("  • Container or system issue")
+                                console.print("\nRecommended actions:")
+                                console.print(
+                                    f"  • Check instance health: [cyan]flow health {task.name or task_id}[/cyan]"
+                                )
+                                console.print(
+                                    f"  • View logs: [cyan]flow logs {task.name or task_id}[/cyan]"
+                                )
+                                console.print(
+                                    f"  • Restart the instance: [cyan]flow cancel {task.name or task_id} && flow dev[/cyan]"
+                                )
 
                             # Show manual connection command
                             console.print(
@@ -483,22 +487,33 @@ class MithrilRemoteOperations(IRemoteOperations):
                             )
                         else:
                             # For new instances, show standard timeout message
-                            console.print(
-                                f"\nThis instance has been starting for {int(elapsed / 60)} minutes."
-                            )
-                            console.print("You can:")
-                            console.print(
-                                f"  • Check startup logs: [cyan]flow logs {task.name or task_id}[/cyan]"
-                            )
-                            console.print(
-                                f"  • Try again: [cyan]flow ssh {task.name or task_id}[/cyan]"
-                            )
-                            console.print(
-                                f"  • Check status: [cyan]flow status {task.name or task_id}[/cyan]"
-                            )
+                            if progress_context is None:
+                                console.print(
+                                    f"\nThis instance has been starting for {int(elapsed / 60)} minutes."
+                                )
+                                console.print("You can:")
+                                console.print(
+                                    f"  • Check startup logs: [cyan]flow logs {task.name or task_id}[/cyan]"
+                                )
+                                console.print(
+                                    f"  • Try again: [cyan]flow ssh {task.name or task_id}[/cyan]"
+                                )
+                                console.print(
+                                    f"  • Check status: [cyan]flow status {task.name or task_id}[/cyan]"
+                                )
                             error_msg = "SSH connection timed out"
 
                         raise RemoteExecutionError(error_msg)
+                else:
+                    # When an external progress handler is used, perform a minimal wait without additional UI
+                    start_time = time.time()
+                    attempts = 0
+                    while time.time() - start_time < ssh_wait_timeout:
+                        if self._is_ssh_ready(task, ssh_key_path):
+                            break
+                        wait_time = min(0.5 * (1 + attempts), 5.0)
+                        time.sleep(wait_time)
+                        attempts += 1
             finally:
                 # Restore original signal handler
                 signal.signal(signal.SIGINT, old_handler)
