@@ -6,9 +6,9 @@ This package manages Mithril resources:
 - SSH key management
 """
 
-from .gpu import get_default_gpu_memory, GPU_SPECS
-from .projects import ProjectNotFoundError, ProjectResolver
-from .ssh import SSHKeyError, SSHKeyManager, SSHKeyNotFoundError
+from flow.providers.mithril.resources.gpu import GPU_SPECS, get_default_gpu_memory
+from flow.providers.mithril.resources.projects import ProjectNotFoundError, ProjectResolver
+from flow.providers.mithril.resources.ssh import SSHKeyError, SSHKeyManager, SSHKeyNotFoundError
 
 __all__ = [
     # GPU
