@@ -1,7 +1,6 @@
-"""SSH key management component for Mithril provider.
+"""SSH key management component for the Mithril provider.
 
-This module provides clean separation of concerns for SSH key operations,
-including automatic key provisioning and error handling.
+Provides SSH key operations including automatic provisioning and error handling.
 """
 
 import json
@@ -13,12 +12,10 @@ import subprocess
 import time
 from datetime import datetime, timezone
 from pathlib import Path
-from typing import List, Optional, Tuple
 
 from flow._internal.io.http_interfaces import IHttpClient
-from flow.errors import FlowError
-
-from ..api.types import SSHKeyModel as SSHKey
+from flow.errors import AuthenticationError, FlowError
+from flow.providers.mithril.api.types import SSHKeyModel as SSHKey
 
 logger = logging.getLogger(__name__)
 
@@ -38,7 +35,7 @@ class SSHKeyNotFoundError(SSHKeyError):
 class SSHKeyManager:
     """Manages SSH keys with automatic provisioning and caching."""
 
-    def __init__(self, http_client: IHttpClient, project_id: Optional[str] = None):
+    def __init__(self, http_client: IHttpClient, project_id: str | None = None):
         """Initialize SSH key manager.
 
         Args:
@@ -47,10 +44,10 @@ class SSHKeyManager:
         """
         self.http = http_client
         self.project_id = project_id
-        self._keys_cache: Optional[List[SSHKey]] = None
+        self._keys_cache: list[SSHKey] | None = None
         self._key_dir = Path.home() / ".flow" / "keys"
 
-    def ensure_keys(self, requested_keys: Optional[List[str]] = None) -> List[str]:
+    def ensure_keys(self, requested_keys: list[str] | None = None) -> list[str]:
         """Ensure SSH keys are available for use.
 
         This method follows a fallback strategy:
@@ -89,7 +86,7 @@ class SSHKeyManager:
         # Return empty list - let the caller decide if this is an error
         return []
 
-    def list_keys(self) -> List[SSHKey]:
+    def list_keys(self) -> list[SSHKey]:
         """List all SSH keys for the project.
 
         Returns:
@@ -112,16 +109,26 @@ class SSHKeyManager:
             # API returns list directly
             keys_data = response if isinstance(response, list) else []
 
-            self._keys_cache = [
-                SSHKey(
-                    fid=k["fid"],
-                    name=k["name"],
-                    public_key=k.get("public_key", ""),
-                    created_at=k.get("created_at"),
-                )
-                for k in keys_data
-                if "fid" in k and "name" in k
-            ]
+            # Normalize optional fields like `required` which some APIs expose
+            normalized_keys: list[SSHKey] = []
+            for k in keys_data:
+                if "fid" not in k or "name" not in k:
+                    continue
+                try:
+                    normalized_keys.append(SSHKey.from_api(k))
+                except Exception:
+                    # Fallback minimal mapping if shape drifts
+                    normalized_keys.append(
+                        SSHKey(
+                            fid=k.get("fid", ""),
+                            name=k.get("name", ""),
+                            public_key=k.get("public_key", ""),
+                            created_at=k.get("created_at"),
+                            required=k.get("required"),
+                        )
+                    )
+
+            self._keys_cache = normalized_keys
 
             logger.debug(f"Loaded {len(self._keys_cache)} SSH keys from API")
             return self._keys_cache
@@ -207,7 +214,7 @@ class SSHKeyManager:
                 raise SSHKeyNotFoundError(f"SSH key {key_id} not found") from e
             raise SSHKeyError(f"Failed to delete SSH key {key_id}: {error_msg}") from e
 
-    def get_key(self, key_id: str) -> Optional[SSHKey]:
+    def get_key(self, key_id: str) -> SSHKey | None:
         """Get a specific SSH key by ID.
 
         Args:
@@ -222,7 +229,7 @@ class SSHKeyManager:
                 return key
         return None
 
-    def find_keys_by_name(self, name: str) -> List[SSHKey]:
+    def find_keys_by_name(self, name: str) -> list[SSHKey]:
         """Find SSH keys by name.
 
         Args:
@@ -239,7 +246,7 @@ class SSHKeyManager:
         self._keys_cache = None
         logger.debug("SSH key cache invalidated")
 
-    def ensure_platform_keys(self, key_references: List[str]) -> List[str]:
+    def ensure_platform_keys(self, key_references: list[str]) -> list[str]:
         """Ensure local SSH keys are uploaded to platform.
 
         This method handles different key reference types:
@@ -247,6 +254,8 @@ class SSHKeyManager:
         - Key names: Resolved locally and uploaded if needed
         - Paths: Read and uploaded if needed
 
+        Note: _auto_ is handled at a higher level in the provider's _get_ssh_keys method.
+
         Args:
             key_references: List of key references (names, paths, or platform IDs)
 
@@ -259,6 +268,13 @@ class SSHKeyManager:
         platform_keys = []
 
         for key_ref in key_references:
+            # Skip _auto_ - it should be handled at provider level
+            if key_ref == "_auto_":
+                logger.debug(
+                    "Skipping '_auto_' in ensure_platform_keys - should be handled at provider level"
+                )
+                continue
+
             key_reference = SSHKeyReference.from_config_value(key_ref)
 
             # Platform SSH key IDs can be used directly
@@ -272,7 +288,7 @@ class SSHKeyManager:
                 logger.warning(
                     f"Could not resolve SSH key '{key_ref}'\n"
                     f"  - Not found locally in ~/.ssh/\n"
-                    f"  - Not found on platform (check 'flow list-keys')\n"
+                    f"  - Not found on platform (check 'flow ssh-keys list')\n"
                     f"  - May have different format (RSA vs ED25519)"
                 )
                 continue
@@ -318,7 +334,23 @@ class SSHKeyManager:
 
         return platform_keys
 
-    def _find_existing_key_by_content(self, public_key_content: str) -> Optional[str]:
+    def ensure_public_key(self, public_key: str, name: str | None = None) -> str:
+        """Ensure a given public key exists on the platform and return its ID.
+
+        Args:
+            public_key: SSH public key content
+            name: Optional display name to use if creating
+
+        Returns:
+            Platform SSH key ID
+        """
+        existing = self._find_existing_key_by_content(public_key)
+        if existing:
+            return existing
+        key_name = name or "flow-key"
+        return self.create_key(key_name, public_key)
+
+    def _find_existing_key_by_content(self, public_key_content: str) -> str | None:
         """Find platform key with matching public key content.
 
         Args:
@@ -339,7 +371,7 @@ class SSHKeyManager:
 
         return None
 
-    def _try_create_default_key(self) -> Optional[str]:
+    def _try_create_default_key(self) -> str | None:
         """Try to create a default SSH key for Mithril use.
 
         This method ONLY checks environment variables and auto-generation.
@@ -395,7 +427,7 @@ class SSHKeyManager:
         logger.info("No SSH keys found, auto-generating new Mithril-specific SSH key")
         return self.auto_generate_key()
 
-    def auto_generate_key(self) -> Optional[str]:
+    def auto_generate_key(self) -> str | None:
         """Auto-generate an SSH key using the best available method.
 
         Tries server-side generation first (no local dependencies),
@@ -413,7 +445,7 @@ class SSHKeyManager:
         logger.info("Server-side generation unavailable, trying local generation...")
         return self._generate_ssh_key()
 
-    def generate_server_key(self) -> Optional[str]:
+    def generate_server_key(self) -> str | None:
         """Generate SSH key server-side using Mithril API.
 
         This is simpler than local generation as it doesn't require ssh-keygen.
@@ -502,7 +534,7 @@ class SSHKeyManager:
         )
         return False
 
-    def generate_local_key(self) -> Optional[str]:
+    def generate_local_key(self) -> str | None:
         """Generate SSH key locally using ssh-keygen.
 
         Public method for local key generation when server-side isn't available.
@@ -512,7 +544,7 @@ class SSHKeyManager:
         """
         return self._generate_ssh_key()
 
-    def _generate_ssh_key(self) -> Optional[str]:
+    def _generate_ssh_key(self) -> str | None:
         """Generate SSH key pair locally and register with Mithril.
 
         Creates an Ed25519 key pair using ssh-keygen, stores it in ~/.flow/keys,
@@ -562,7 +594,7 @@ class SSHKeyManager:
             logger.debug(f"Failed to auto-generate SSH key: {e}")
             return None
 
-    def _create_key_pair(self, key_name: str) -> Tuple[Path, Path]:
+    def _create_key_pair(self, key_name: str) -> tuple[Path, Path]:
         """Create SSH key pair using ssh-keygen.
 
         Args:
@@ -637,7 +669,7 @@ class SSHKeyManager:
         metadata_path.write_text(json.dumps(existing, indent=2))
         self._set_key_permissions(metadata_path)
 
-    def _get_cached_auto_key(self) -> Optional[str]:
+    def _get_cached_auto_key(self) -> str | None:
         """Check for previously auto-generated keys.
 
         Searches metadata.json for auto-generated keys belonging to the current
@@ -684,7 +716,7 @@ class SSHKeyManager:
             # Continue execution - functional key with suboptimal permissions
             # is preferable to complete failure
 
-    def find_matching_local_key(self, api_key_id: str) -> Optional[Path]:
+    def find_matching_local_key(self, api_key_id: str) -> Path | None:
         """Find local private key that matches an API SSH key.
 
         Searches standard SSH locations and cached metadata to find
@@ -714,7 +746,7 @@ class SSHKeyManager:
         # First check MITHRIL_SSH_KEY environment variable
         if env_key := os.environ.get("MITHRIL_SSH_KEY"):
             env_path = Path(env_key).expanduser()
-            if not env_path.suffix == ".pub":
+            if env_path.suffix != ".pub":
                 key_paths.append(env_path)
 
         # Add standard key names
@@ -728,7 +760,7 @@ class SSHKeyManager:
             for key_file in ssh_dir.glob("*"):
                 # Skip if it's a public key, directory, or already in our list
                 if (
-                    not key_file.suffix == ".pub"
+                    key_file.suffix != ".pub"
                     and key_file.is_file()
                     and key_file not in key_paths
                     and key_file.name not in ["known_hosts", "authorized_keys", "config"]
@@ -756,7 +788,7 @@ class SSHKeyManager:
         logger.debug(f"No matching local key found for {api_key_id} ({api_key.name})")
         return None
 
-    def _check_metadata_for_key(self, api_key_id: str) -> Optional[Path]:
+    def _check_metadata_for_key(self, api_key_id: str) -> Path | None:
         """Check metadata cache for auto-generated key.
 
         Args:
@@ -799,6 +831,37 @@ class SSHKeyManager:
 
         return local_normalized == api_normalized
 
+    # --- Admin operations ---
+    def set_key_required(self, key_id: str, required: bool) -> bool:
+        """Set or clear the 'required' flag on an SSH key.
+
+        Requires project admin privileges on Mithril. When a key is marked as
+        required, the platform expects it to be included in all new launches
+        for the project. Flow also auto-includes required keys for convenience.
+
+        Args:
+            key_id: Platform SSH key ID (e.g., sshkey_abc123)
+            required: True to mark as required, False to clear required flag
+
+        Returns:
+            True if the update succeeded, False otherwise
+        """
+        try:
+            self.http.request(
+                method="PATCH",
+                url=f"/v2/ssh-keys/{key_id}",
+                json={"required": bool(required)},
+            )
+            # Invalidate cache so list_keys reflects latest state
+            self.invalidate_cache()
+            return True
+        except AuthenticationError:
+            # Bubble up explicit auth/permission errors so CLI can show actionable text
+            raise
+        except Exception as e:
+            logger.error(f"Failed to update required flag for SSH key {key_id}: {e}")
+            return False
+
     def _normalize_public_key(self, public_key: str) -> str:
         """Normalize SSH public key for comparison.
 
