@@ -5,7 +5,7 @@ This package handles runtime aspects:
 - Quota awareness and management
 """
 
-from .startup.builder import MithrilStartupScriptBuilder
+from flow.providers.mithril.runtime.startup.builder import MithrilStartupScriptBuilder
 
 __all__ = [
     # Startup script builder
