@@ -6,8 +6,11 @@ This package builds startup scripts for Mithril instances:
 - Template engine abstraction
 """
 
-from .builder import MithrilStartupScriptBuilder, StartupScript
-from .sections import (
+from flow.providers.mithril.runtime.startup.builder import (
+    MithrilStartupScriptBuilder,
+    StartupScript,
+)
+from flow.providers.mithril.runtime.startup.sections import (
     CodeUploadSection,
     DockerSection,
     HeaderSection,
@@ -16,7 +19,7 @@ from .sections import (
     UserScriptSection,
     VolumeSection,
 )
-from .templates import create_template_engine, ITemplateEngine
+from flow.providers.mithril.runtime.startup.templates import ITemplateEngine, create_template_engine
 
 __all__ = [
     # Builder
