@@ -3,41 +3,40 @@
 Handles Mithril-specific configuration including API keys, projects, SSH keys, and regions.
 """
 
-import os
 import configparser
-from typing import Dict, Any, Optional, List
+import os
 from pathlib import Path
+from typing import Any
 
 from rich.console import Console
-from rich.prompt import Prompt, Confirm
+from rich.prompt import Confirm, Prompt
 
-from flow.core.provider_setup import ProviderSetup, SetupResult
 from flow._internal.io.http import HttpClient
-from flow.errors import AuthenticationError
-from .core.constants import DEFAULT_REGION, VALID_REGIONS
+from flow.core.provider_setup import ProviderSetup, SetupResult
+from flow.providers.mithril.core.constants import VALID_REGIONS
 
 
 class MithrilProviderSetup(ProviderSetup):
     """Mithril-specific setup implementation."""
 
-    def __init__(self, console: Optional[Console] = None):
+    def __init__(self, console: Console | None = None):
         """Initialize Mithril setup.
 
         Args:
             console: Rich console for output (creates one if not provided)
         """
         self.console = console or Console()
-        self.api_url = os.environ.get("FLOW_API_URL", "https://api.mithril.ai")
+        self.api_url = os.environ.get("MITHRIL_API_URL", os.environ.get("FLOW_API_URL", "https://api.mithril.ai"))
 
     def get_provider_name(self) -> str:
         """Get the name of this provider."""
         return "mithril"
 
-    def get_required_fields(self) -> List[str]:
+    def get_required_fields(self) -> list[str]:
         """Get required configuration fields."""
         return ["api_key", "project"]
 
-    def get_optional_fields(self) -> List[str]:
+    def get_optional_fields(self) -> list[str]:
         """Get optional configuration fields."""
         return ["region", "default_ssh_key", "api_url"]
 
@@ -77,9 +76,9 @@ class MithrilProviderSetup(ProviderSetup):
 
     def setup_with_options(
         self,
-        api_key: Optional[str] = None,
-        project: Optional[str] = None,
-        region: Optional[str] = None,
+        api_key: str | None = None,
+        project: str | None = None,
+        region: str | None = None,
         **kwargs,
     ) -> SetupResult:
         """Configure with provided options (non-interactive).
@@ -112,7 +111,7 @@ class MithrilProviderSetup(ProviderSetup):
         else:
             return SetupResult(success=False, config=config, message="Invalid or missing API key")
 
-    def validate_credentials(self, credentials: Dict[str, Any]) -> bool:
+    def validate_credentials(self, credentials: dict[str, Any]) -> bool:
         """Validate Mithril credentials."""
         api_key = credentials.get("api_key", "").strip()
         if not api_key:
@@ -134,10 +133,11 @@ class MithrilProviderSetup(ProviderSetup):
         except Exception:
             return False
 
-    def _setup_api_key(self) -> Optional[str]:
+    def _setup_api_key(self) -> str | None:
         """Configure API key interactively."""
         self.console.print("\n[bold]Step 1: API Key Configuration[/bold]")
-        self.console.print("Get your API key from: https://mithril.ai/account/api-keys")
+        from flow.links import WebLinks
+        self.console.print(f"Get your API key from: {WebLinks.api_keys()}")
 
         api_key = Prompt.ask("\nEnter your Mithril API key").strip()
 
@@ -158,7 +158,7 @@ class MithrilProviderSetup(ProviderSetup):
                 return api_key
             return None
 
-    def _setup_project(self, api_key: str) -> Optional[str]:
+    def _setup_project(self, api_key: str) -> str | None:
         """Configure project selection."""
         self.console.print("\n[bold]Step 2: Project Selection[/bold]")
 
@@ -200,7 +200,7 @@ class MithrilProviderSetup(ProviderSetup):
             self.console.print(f"[yellow]Could not fetch projects: {e}[/yellow]")
             return Prompt.ask("Enter project name manually", default="default")
 
-    def _setup_ssh_keys(self, api_key: str, project: str) -> Optional[str]:
+    def _setup_ssh_keys(self, api_key: str, project: str) -> str | None:
         """Configure SSH keys (optional)."""
         self.console.print("\n[bold]Step 3: SSH Key Configuration (Optional)[/bold]")
 
@@ -377,7 +377,7 @@ class MithrilProviderSetup(ProviderSetup):
 
         return None
 
-    def _setup_region(self) -> Optional[str]:
+    def _setup_region(self) -> str | None:
         """Configure default region (optional)."""
         self.console.print("\n[bold]Step 4: Default Region (Optional)[/bold]")
 
