@@ -1,6 +1,6 @@
 """Mithril provider setup module."""
 
 # Only import the adapter to avoid circular imports
-from .adapter import MithrilSetupAdapter
+from flow.providers.mithril.setup.adapter import MithrilSetupAdapter
 
 __all__ = ["MithrilSetupAdapter"]
