@@ -1,36 +1,39 @@
 """Mithril provider setup adapter.
 
-Extracts Mithril-specific logic from the wizard to make it provider-agnostic
-while maintaining all the beautiful UI and functionality.
+Extracts Mithril-specific logic from the wizard to keep the wizard provider-agnostic
+while preserving the UI and functionality.
 """
 
-import asyncio
 import os
 from pathlib import Path
-from typing import Dict, Any, Optional, List, Tuple
+from typing import Any
 
 from rich.console import Console
+from rich.markup import escape
 
-from flow.core.setup_adapters import ProviderSetupAdapter, ConfigField, FieldType, ValidationResult
-from flow._internal.io.http import HttpClient
-from ..core.constants import DEFAULT_REGION, VALID_REGIONS
 from flow._internal.config_loader import ConfigLoader
-from flow._internal.init.writer import ConfigWriter
+from flow._internal.config_manager import ConfigManager
+from flow._internal.io.http import HttpClient
 from flow.api.models import ValidationResult as ApiValidationResult
+from flow.cli.utils.mask_utils import mask_api_key
 from flow.cli.utils.shell_completion import CompletionCommand
+from flow.core.setup_adapters import ConfigField, FieldType, ProviderSetupAdapter, ValidationResult
+from flow.links import WebLinks
+from flow.providers.mithril.core.constants import DEFAULT_REGION, VALID_REGIONS
 
 
 class MithrilSetupAdapter(ProviderSetupAdapter):
     """Mithril provider setup adapter."""
 
-    def __init__(self, console: Optional[Console] = None):
+    def __init__(self, console: Console | None = None):
         """Initialize Mithril setup adapter.
 
         Args:
             console: Rich console for output (creates one if not provided)
         """
         self.console = console or Console()
-        self.api_url = os.environ.get("FLOW_API_URL", "https://api.mithril.ai")
+        # Canonical API URL env var; allow FLOW_API_URL as a last-resort dev fallback
+        self.api_url = os.environ.get("MITHRIL_API_URL", os.environ.get("FLOW_API_URL", "https://api.mithril.ai"))
         self.config_path = Path.home() / ".flow" / "config.yaml"
         self._current_context = {}  # Store current wizard context
 
@@ -38,7 +41,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         """Get the provider name."""
         return "mithril"
 
-    def get_configuration_fields(self) -> List[ConfigField]:
+    def get_configuration_fields(self) -> list[ConfigField]:
         """Get Mithril configuration fields."""
         return [
             ConfigField(
@@ -46,7 +49,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                 field_type=FieldType.PASSWORD,
                 required=True,
                 mask_display=True,
-                help_url="https://app.mithril.ai/account/apikeys",
+                help_url=WebLinks.api_keys(),
                 help_text="Get your API key from Mithril",
                 default=None,
                 display_name="API Key",
@@ -57,15 +60,22 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                 required=True,
                 dynamic_choices=True,
                 help_text="Select your Mithril project",
+                depends_on=["api_key"],
+                empty_choices_hint="Requires API key to list projects",
             ),
             ConfigField(
                 name="default_ssh_key",
                 field_type=FieldType.CHOICE,
-                required=False,
+                required=True,
                 dynamic_choices=True,
-                help_url="https://app.mithril.ai/ssh-keys",
-                help_text="SSH key for accessing running instances",
+                help_url=WebLinks.ssh_keys(),
+                help_text=(
+                    "Choose how you'll SSH into instances. Pick an existing key from your "
+                    "Mithril account or select Auto‑generate per instance."
+                ),
                 display_name="Default SSH Key",
+                depends_on=["api_key", "project"],
+                empty_choices_hint="Requires API key and project to list SSH keys",
             ),
             ConfigField(
                 name="region",
@@ -79,7 +89,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         ]
 
     def validate_field(
-        self, field_name: str, value: str, context: Optional[Dict[str, Any]] = None
+        self, field_name: str, value: str, context: dict[str, Any] | None = None
     ) -> ValidationResult:
         """Validate a single field value."""
         # Update current context if provided
@@ -97,7 +107,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         else:
             return ValidationResult(is_valid=False, message=f"Unknown field: {field_name}")
 
-    def get_dynamic_choices(self, field_name: str, context: Dict[str, Any]) -> List[str]:
+    def get_dynamic_choices(self, field_name: str, context: dict[str, Any]) -> list[str]:
         """Get dynamic choices for a field."""
         # Store the current context for use in validation
         self._current_context = context
@@ -109,66 +119,29 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         else:
             return []
 
-    def detect_existing_config(self) -> Dict[str, Any]:
+    def detect_existing_config(self) -> dict[str, Any]:
         """Detect existing configuration from environment, files, etc."""
-        # Use the centralized ConfigLoader for proper precedence and consistency
-        loader = ConfigLoader(self.config_path)
-        sources = loader.load_all_sources()
+        # Use centralized manager for consistent detection/normalization
+        manager = ConfigManager(self.config_path)
+        detected = manager.detect_existing_config()
 
-        config = {}
+        # Always include region default if missing (UI shows something meaningful)
+        if "region" not in detected:
+            detected["region"] = DEFAULT_REGION
 
-        # API Key (env > credentials > config file)
-        api_key = sources.api_key
-        if api_key and not api_key.startswith("YOUR_"):
-            config["api_key"] = api_key
+        return detected
 
-        # Get Mithril-specific configuration
-        mithril_config = sources.get_mithril_config()
-
-        # Project
-        project = mithril_config.get("project")
-        if project and not project.startswith("YOUR_"):
-            config["project"] = project
-
-        # Region (always show default)
-        region = mithril_config.get("region", DEFAULT_REGION)
-        config["region"] = region
-
-        # SSH Keys (handle both list and single key formats)
-        ssh_keys = mithril_config.get("ssh_keys")
-        if ssh_keys:
-            if isinstance(ssh_keys, list):
-                # Check for platform auto-generation
-                if ssh_keys == ["_auto_"]:
-                    config["default_ssh_key"] = "_auto_"
-                else:
-                    config["default_ssh_key"] = ",".join(ssh_keys)
-            else:
-                config["default_ssh_key"] = str(ssh_keys)
-        else:
-            # Default to platform auto-generation for new users
-            config["default_ssh_key"] = "_auto_"
-
-        return config
-
-    def save_configuration(self, config: Dict[str, Any]) -> bool:
+    def save_configuration(self, config: dict[str, Any]) -> bool:
         """Save the final configuration using centralized ConfigWriter."""
         try:
-            # Get existing config and merge
-            existing_config = self._load_existing_config()
-            final_config = existing_config.copy()
-            final_config.update(config)
+            manager = ConfigManager(self.config_path)
+            # Normalize and save using centralized manager
+            payload = dict(config)
+            payload.setdefault("provider", "mithril")
+            saved = manager.save(payload)
 
-            # Ensure provider is set
-            if "provider" not in final_config:
-                final_config["provider"] = "mithril"
-
-            # Use centralized ConfigWriter for consistent, secure saving
-            writer = ConfigWriter(self.config_path)
-            writer.write(final_config, ApiValidationResult(is_valid=True, projects=[]))
-
-            # Update environment script with provider-specific variables
-            self._create_env_script(final_config)
+            # Write canonical env script
+            manager.write_env_script(saved)
 
             # Set up shell completion automatically
             try:
@@ -178,7 +151,6 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                     self.console.print(f"\n[dim]Setting up {shell} shell completion...[/dim]")
                     completion_cmd._install_completion(shell, None)
             except Exception:
-                # Don't fail the whole setup if completion install fails
                 pass
 
             return True
@@ -186,7 +158,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         except Exception:
             return False
 
-    def verify_configuration(self, config: Dict[str, Any]) -> Tuple[bool, Optional[str]]:
+    def verify_configuration(self, config: dict[str, Any]) -> tuple[bool, str | None]:
         """Verify that the configuration works end-to-end."""
         try:
             # Set environment from config
@@ -210,11 +182,13 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                 billing_status = http_client.request("GET", "/v2/account/billing")
                 if not billing_status.get("configured", False):
                     # Store billing status for completion message
-                    self._billing_not_configured = True
+                    # Expose as public attribute for UI to read without private access
+                    self.billing_not_configured = True
                     self.console.print(
                         "\n[yellow]Note: Billing not configured yet. Set it up at:[/yellow]"
                     )
-                    self.console.print("[cyan]https://app.mithril.ai/settings/billing[/cyan]")
+                    from flow.links import WebLinks
+                    self.console.print(f"[cyan]{WebLinks.billing_settings()}[/cyan]")
             except:
                 # Don't fail setup for billing check
                 pass
@@ -224,7 +198,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         except Exception as e:
             return False, str(e)
 
-    def get_welcome_message(self) -> Tuple[str, List[str]]:
+    def get_welcome_message(self) -> tuple[str, list[str]]:
         """Get Mithril-specific welcome message."""
         return (
             "Welcome to Flow SDK Setup",
@@ -257,8 +231,8 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                 base_url=self.api_url,
                 headers={"Authorization": f"Bearer {api_key}"},
             )
-            client.request("GET", "/v2/projects")
-            masked_key = f"{api_key[:8]}...{api_key[-4:]}" if len(api_key) > 10 else "[CONFIGURED]"
+            client.request("GET", "/v2/projects", timeout_seconds=10.0)
+            masked_key = mask_api_key(api_key)
             return ValidationResult(is_valid=True, display_value=masked_key)
         except Exception as e:
             return ValidationResult(is_valid=False, message=f"API validation failed: {e}")
@@ -337,7 +311,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
             )
         return ValidationResult(is_valid=True, display_value=region)
 
-    def _get_project_choices(self, api_key: Optional[str]) -> List[str]:
+    def _get_project_choices(self, api_key: str | None) -> list[str]:
         """Get available projects from API."""
         if not api_key:
             return []
@@ -347,19 +321,19 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                 base_url=self.api_url,
                 headers={"Authorization": f"Bearer {api_key}"},
             )
-            projects = client.request("GET", "/v2/projects")
+            projects = client.request("GET", "/v2/projects", timeout_seconds=10.0)
             return [proj["name"] for proj in projects if isinstance(projects, list)]
         except Exception:
             return []
 
-    def _get_ssh_key_choices(self, api_key: Optional[str], project: Optional[str]) -> List[str]:
+    def _get_ssh_key_choices(self, api_key: str | None, project: str | None) -> list[str]:
         """Get available SSH keys from API plus generation options."""
         choices = []
 
-        # Add generation options first - platform auto-generation is recommended
+            # Add generation options first
         choices.extend(
             [
-                "_auto_|Platform auto-generates per instance (recommended)",
+                    "_auto_|Platform auto‑generate per instance",
                 "GENERATE_SERVER|Generate new SSH key on Mithril platform",
                 "GENERATE_LOCAL|Generate new SSH key locally with ssh-keygen",
             ]
@@ -375,7 +349,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
             )
 
             # Get project ID
-            projects = client.request("GET", "/v2/projects")
+            projects = client.request("GET", "/v2/projects", timeout_seconds=10.0)
             project_id = None
             for proj in projects:
                 if proj.get("name") == project:
@@ -386,20 +360,25 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
                 return choices
 
             # Get existing SSH keys
-            ssh_keys = client.request("GET", "/v2/ssh-keys", params={"project": project_id})
+            ssh_keys = client.request("GET", "/v2/ssh-keys", params={"project": project_id}, timeout_seconds=10.0)
             if isinstance(ssh_keys, list):
                 for key in ssh_keys:
                     # Format: ID|Name|CreatedAt|Fingerprint for richer display
                     created_at = key.get("created_at", "")
                     # Extract fingerprint from public key if available
                     fingerprint = self._extract_fingerprint(key.get("public_key", ""))
-                    choices.append(f"{key['fid']}|{key['name']}|{created_at}|{fingerprint}")
+                    required_flag = (
+                        " (required)" if key.get("required") or key.get("is_required") else ""
+                    )
+                    choices.append(
+                        f"{key['fid']}|{key['name']}{required_flag}|{created_at}|{fingerprint}"
+                    )
 
             return choices
         except Exception:
             return choices
 
-    def _generate_server_side_key(self) -> Optional[str]:
+    def _generate_server_side_key(self) -> str | None:
         """Generate SSH key server-side."""
         try:
             # Get current config for API access
@@ -448,12 +427,16 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
             return key_id
 
         except Exception as e:
-            self.console.print(f"[red]Error generating SSH key: {type(e).__name__}: {e}[/red]")
+            self.console.print(
+                f"[red]Error generating SSH key: {escape(type(e).__name__)}: {escape(str(e))}[/red]"
+            )
             if hasattr(e, "response"):
-                self.console.print(f"[red]API Response: {getattr(e, 'response', 'N/A')}[/red]")
+                self.console.print(
+                    f"[red]API Response: {escape(str(getattr(e, 'response', 'N/A')))}[/red]"
+                )
             return None
 
-    def _generate_local_key(self) -> Optional[str]:
+    def _generate_local_key(self) -> str | None:
         """Generate SSH key locally."""
         try:
             # Get current config for API access
@@ -502,12 +485,16 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
             return key_id
 
         except Exception as e:
-            self.console.print(f"[red]Error generating SSH key: {type(e).__name__}: {e}[/red]")
+            self.console.print(
+                f"[red]Error generating SSH key: {escape(type(e).__name__)}: {escape(str(e))}[/red]"
+            )
             if hasattr(e, "response"):
-                self.console.print(f"[red]API Response: {getattr(e, 'response', 'N/A')}[/red]")
+                self.console.print(
+                    f"[red]API Response: {escape(str(getattr(e, 'response', 'N/A')))}[/red]"
+                )
             return None
 
-    def _create_env_script(self, config: Dict[str, Any]):
+    def _create_env_script(self, config: dict[str, Any]):
         """Create shell script with clean provider-specific environment variables.
 
         Clean, decisive approach: MITHRIL_* variables only.
@@ -520,13 +507,13 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
             f.write("# Flow SDK Mithril provider environment variables\n")
             f.write("# Source this file: source ~/.flow/env.sh\n\n")
 
-            # Project - provider-specific naming only
+            # Project - provider-specific canonical naming only
             if "project" in config:
-                f.write(f'export MITHRIL_DEFAULT_PROJECT="{config["project"]}"\n')
+                f.write(f'export MITHRIL_PROJECT="{config["project"]}"\n')
 
-            # Region - provider-specific naming only
+            # Region - provider-specific canonical naming only
             if "region" in config:
-                f.write(f'export MITHRIL_DEFAULT_REGION="{config["region"]}"\n')
+                f.write(f'export MITHRIL_REGION="{config["region"]}"\n')
 
             # SSH keys - provider-specific naming only
             if "default_ssh_key" in config:
@@ -547,8 +534,8 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
             return ""
 
         try:
-            import hashlib
             import base64
+            import hashlib
 
             # SSH public keys format: <type> <base64-data> [comment]
             parts = public_key.strip().split()
@@ -565,7 +552,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         except Exception:
             return ""
 
-    def _load_existing_config(self) -> Dict[str, Any]:
+    def _load_existing_config(self) -> dict[str, Any]:
         """Load existing configuration from file."""
         if not self.config_path.exists():
             return {}
@@ -573,7 +560,7 @@ class MithrilSetupAdapter(ProviderSetupAdapter):
         try:
             import yaml
 
-            with open(self.config_path, "r") as f:
+            with open(self.config_path) as f:
                 return yaml.safe_load(f) or {}
         except Exception:
             return {}
