@@ -1,7 +1,6 @@
-"""Shared SSH utilities for Mithril provider.
+"""Shared SSH utilities for the Mithril provider.
 
-This module provides common SSH-related functionality used across
-the Mithril provider, ensuring consistent behavior for SSH operations.
+Common SSH-related functionality used across the provider for consistent behavior.
 """
 
 import atexit
@@ -11,16 +10,17 @@ import signal
 import socket
 import subprocess
 import time
+from collections.abc import Callable
 from contextlib import contextmanager
 from dataclasses import dataclass
-from typing import Optional, Callable, TYPE_CHECKING, Dict, List
+from typing import TYPE_CHECKING, Optional
 
 from flow.api.models import Task
 from flow.errors import FlowError
 from flow.utils.circuit_breaker import CircuitBreaker
 
 if TYPE_CHECKING:
-    from .provider import MithrilProvider
+    from flow.providers.mithril.provider import MithrilProvider
 
 logger = logging.getLogger(__name__)
 
@@ -73,7 +73,7 @@ class SSHTunnelManager:
     """
 
     # Global registry of active tunnels for cleanup
-    _active_tunnels: Dict[str, SSHTunnel] = {}
+    _active_tunnels: dict[str, SSHTunnel] = {}
     _circuit_breaker = CircuitBreaker(
         failure_threshold=3,
         recovery_timeout=30.0,
@@ -108,7 +108,7 @@ class SSHTunnelManager:
         local_port: int = 0,
         remote_port: int = 22,
         remote_host: str = "localhost",
-        ssh_options: Optional[List[str]] = None,
+        ssh_options: list[str] | None = None,
     ) -> SSHTunnel:
         """Create an SSH tunnel with circuit breaker protection.
 
@@ -142,7 +142,7 @@ class SSHTunnelManager:
         local_port: int = 0,
         remote_port: int = 22,
         remote_host: str = "localhost",
-        ssh_options: Optional[List[str]] = None,
+        ssh_options: list[str] | None = None,
     ) -> SSHTunnel:
         """Internal implementation of tunnel creation."""
         if not task.ssh_host:
@@ -237,8 +237,8 @@ class SSHTunnelManager:
         local_port: int,
         remote_port: int,
         remote_host: str,
-        ssh_options: Optional[List[str]] = None,
-    ) -> List[str]:
+        ssh_options: list[str] | None = None,
+    ) -> list[str]:
         """Build SSH command for tunnel."""
         cmd = [
             "ssh",
@@ -275,6 +275,7 @@ class SSHTunnelManager:
         if not ssh_key:
             # Try to get from config file if available
             from pathlib import Path
+
             import yaml
 
             try:
@@ -306,7 +307,7 @@ class SSHTunnelManager:
                     s.settimeout(1)
                     s.connect(("localhost", port))
                     return True
-            except (socket.error, socket.timeout):
+            except (TimeoutError, OSError):
                 time.sleep(0.1)
         return False
 
@@ -318,7 +319,7 @@ class SSHTunnelManager:
         local_port: int = 0,
         remote_port: int = 22,
         remote_host: str = "localhost",
-        ssh_options: Optional[List[str]] = None,
+        ssh_options: list[str] | None = None,
     ):
         """Context manager for SSH tunnels with automatic cleanup.
 
@@ -363,9 +364,9 @@ if hasattr(signal, "SIGINT"):
 def wait_for_task_ssh_info(
     task: Task,
     provider: Optional["MithrilProvider"] = None,
-    timeout: Optional[int] = None,
-    check_interval: Optional[int] = None,
-    progress_callback: Optional[Callable[[str], None]] = None,
+    timeout: int | None = None,
+    check_interval: int | None = None,
+    progress_callback: Callable[[str], None] | None = None,
     show_progress: bool = True,
 ) -> Task:
     """Wait for a task to have SSH connection information.
@@ -388,9 +389,8 @@ def wait_for_task_ssh_info(
     Raises:
         SSHNotReadyError: If SSH info not available within timeout
     """
-    from .core.constants import (
+    from flow.providers.mithril.core.constants import (
         EXPECTED_PROVISION_MINUTES,
-        INSTANCE_IP_WAIT_SECONDS,
         INSTANCE_IP_CHECK_INTERVAL,
     )
 
@@ -415,9 +415,10 @@ def wait_for_task_ssh_info(
     if not progress_callback and show_progress:
         # No callback provided - use AnimatedEllipsisProgress
         try:
-            from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
             from rich.console import Console
 
+            from flow.cli.utils.animated_progress import AnimatedEllipsisProgress
+
             console = Console()
             progress_context = AnimatedEllipsisProgress(
                 console,
@@ -486,9 +487,9 @@ def wait_for_task_ssh_info(
             except KeyboardInterrupt:
                 # Allow graceful interruption
                 raise SSHNotReadyError(
-                    f"Wait for SSH interrupted by user",
+                    "Wait for SSH interrupted by user",
                     suggestions=[
-                        f"The task may still be provisioning",
+                        "The task may still be provisioning",
                         f"Check later with: flow ssh {task.task_id}",
                         f"Check status with: flow status {task.task_id}",
                     ],
@@ -502,7 +503,7 @@ def wait_for_task_ssh_info(
                 pass  # Ignore cleanup errors
 
 
-def check_task_age_for_ssh(task: Task) -> Optional[str]:
+def check_task_age_for_ssh(task: Task) -> str | None:
     """Check task age and return appropriate message for SSH issues.
 
     Args:
@@ -540,7 +541,7 @@ def check_task_age_for_ssh(task: Task) -> Optional[str]:
                 "SSH is only available for running instances."
             )
     elif age_minutes < 20 and not task.ssh_host:
-        from .core.constants import EXPECTED_PROVISION_MINUTES
+        from flow.providers.mithril.core.constants import EXPECTED_PROVISION_MINUTES
 
         if status_str == "pending":
             return (
