@@ -1,7 +1,7 @@
-"""Storage backend implementation for Mithril provider.
+"""Storage backend implementation for the Mithril provider.
 
-This module provides a local HTTP server backend for handling large files
-like startup scripts during development and testing.
+Local HTTP server backend for handling large files like startup scripts during
+development and testing.
 """
 
 import atexit
@@ -18,10 +18,10 @@ from contextlib import contextmanager
 from datetime import datetime, timedelta, timezone
 from http.server import HTTPServer, SimpleHTTPRequestHandler
 from pathlib import Path
-from typing import Any, Dict, Optional, Protocol
+from typing import Any, Protocol
 from urllib.parse import quote
 
-from .models import StorageMetadata, StorageUrl
+from flow.providers.mithril.storage.models import StorageMetadata, StorageUrl
 
 logger = logging.getLogger(__name__)
 
@@ -34,12 +34,12 @@ class IStorageBackend(Protocol):
     """Interface for storage backend implementations."""
 
     def store(
-        self, key: str, data: bytes = None, content: bytes = None, metadata: Dict[str, Any] = None
+        self, key: str, data: bytes = None, content: bytes = None, metadata: dict[str, Any] = None
     ) -> StorageUrl:
         """Store content and return a URL for retrieval."""
         ...
 
-    def retrieve(self, key: str) -> Optional[bytes]:
+    def retrieve(self, key: str) -> bytes | None:
         """Retrieve content by key."""
         ...
 
@@ -55,11 +55,11 @@ class IStorageBackend(Protocol):
         """Check if content exists."""
         ...
 
-    def get_metadata(self, key: str) -> Optional[StorageMetadata]:
+    def get_metadata(self, key: str) -> StorageMetadata | None:
         """Get metadata for stored content."""
         ...
 
-    def health_check(self) -> Dict[str, Any]:
+    def health_check(self) -> dict[str, Any]:
         """Perform health check on the backend."""
         ...
 
@@ -85,7 +85,7 @@ class StorageConfig:
         backend_type: str = "local",
         local_port: int = 8080,
         local_bind: str = "127.0.0.1",
-        local_storage_path: Optional[str] = None,
+        local_storage_path: str | None = None,
         local_cleanup_on_exit: bool = True,
         url_expiry_seconds: int = 3600,
         max_file_size_mb: int = 100,
@@ -196,7 +196,7 @@ class BaseStorageBackend:
             if metric in self._metrics:
                 self._metrics[metric] += value
 
-    def get_metrics(self) -> Dict[str, int]:
+    def get_metrics(self) -> dict[str, int]:
         """Get current metrics.
 
         Returns:
@@ -427,7 +427,7 @@ class LocalHttpBackend(BaseStorageBackend):
         return StorageHandler
 
     def store(
-        self, key: str, data: bytes = None, content: bytes = None, metadata: Dict[str, Any] = None
+        self, key: str, data: bytes = None, content: bytes = None, metadata: dict[str, Any] = None
     ) -> StorageUrl:
         """Store content and return URL.
 
@@ -511,7 +511,7 @@ class LocalHttpBackend(BaseStorageBackend):
 
             return self.get_url(key)
 
-    def retrieve(self, key: str) -> Optional[bytes]:
+    def retrieve(self, key: str) -> bytes | None:
         """Retrieve content by key.
 
         Args:
@@ -589,7 +589,7 @@ class LocalHttpBackend(BaseStorageBackend):
         except StorageError:
             return False
 
-    def get_metadata(self, key: str) -> Optional[StorageMetadata]:
+    def get_metadata(self, key: str) -> StorageMetadata | None:
         """Get metadata for content.
 
         Args:
@@ -619,7 +619,7 @@ class LocalHttpBackend(BaseStorageBackend):
 
         return None
 
-    def health_check(self) -> Dict[str, Any]:
+    def health_check(self) -> dict[str, Any]:
         """Perform health check.
 
         Returns:
