@@ -1,7 +1,7 @@
 """File transfer strategies for uploading code to Mithril instances.
 
-This module provides various strategies for transferring files to remote
-instances, with rsync as the primary implementation for efficiency.
+Provides strategies for transferring files to remote instances, with rsync as
+the primary implementation for efficiency.
 """
 
 import logging
@@ -10,12 +10,13 @@ import re
 import subprocess
 import tempfile
 import time
+from collections.abc import Callable
 from dataclasses import dataclass
-from datetime import datetime, timezone
 from pathlib import Path
-from typing import Callable, List, Optional, Protocol
+from typing import Protocol
 
 from flow.errors import FlowError
+from flow.providers.mithril.ssh_waiter import SSHConnectionInfo
 
 logger = logging.getLogger(__name__)
 
@@ -31,11 +32,11 @@ class TransferProgress:
     """Progress information for ongoing transfer."""
 
     bytes_transferred: int
-    total_bytes: Optional[int]
-    percentage: Optional[float]
-    speed: Optional[str]  # e.g., "2.34MB/s"
-    eta: Optional[str]  # e.g., "0:00:03"
-    current_file: Optional[str]
+    total_bytes: int | None
+    percentage: float | None
+    speed: str | None  # e.g., "2.34MB/s"
+    eta: str | None  # e.g., "0:00:03"
+    current_file: str | None
 
     @property
     def is_complete(self) -> bool:
@@ -51,7 +52,7 @@ class TransferResult:
     bytes_transferred: int
     duration_seconds: float
     files_transferred: int
-    error_message: Optional[str] = None
+    error_message: str | None = None
 
     @property
     def transfer_rate(self) -> str:
@@ -70,7 +71,7 @@ class ITransferStrategy(Protocol):
         source: Path,
         target: str,
         connection: "SSHConnectionInfo",
-        progress_callback: Optional[Callable[[TransferProgress], None]] = None,
+        progress_callback: Callable[[TransferProgress], None] | None = None,
     ) -> TransferResult:
         """Transfer files from source to target.
 
@@ -108,7 +109,7 @@ class RsyncTransferStrategy:
         source: Path,
         target: str,
         connection: "SSHConnectionInfo",
-        progress_callback: Optional[Callable[[TransferProgress], None]] = None,
+        progress_callback: Callable[[TransferProgress], None] | None = None,
     ) -> TransferResult:
         """Transfer files using rsync.
 
@@ -134,7 +135,7 @@ class RsyncTransferStrategy:
         exclude_file = self._create_exclude_file(source)
 
         # Optional: prepare git-based incremental file list (auto-detected)
-        files_from_file = None  # type: Optional[Path]
+        files_from_file: Path | None = None
         files_from_is_from0 = False
         files_from_count = 0
         try:
@@ -253,7 +254,7 @@ class RsyncTransferStrategy:
             "  - RHEL/CentOS: yum install rsync"
         )
 
-    def _create_exclude_file(self, source: Path) -> Optional[Path]:
+    def _create_exclude_file(self, source: Path) -> Path | None:
         """Create exclude file from .flowignore patterns.
 
         Args:
@@ -300,7 +301,7 @@ class RsyncTransferStrategy:
             mode="w", delete=False, prefix="flow-rsync-exclude-"
         )
 
-        with open(flowignore, "r") as f:
+        with open(flowignore) as f:
             # Process .flowignore patterns
             for line in f:
                 line = line.strip()
@@ -311,7 +312,7 @@ class RsyncTransferStrategy:
         exclude_file.close()
         return Path(exclude_file.name)
 
-    def _get_git_changed_files(self, source: Path) -> Optional[dict]:
+    def _get_git_changed_files(self, source: Path) -> dict | None:
         """Build a NUL-separated list of changed files for rsync.
 
         Returns a dict with keys:
@@ -391,9 +392,9 @@ class RsyncTransferStrategy:
                 tmp.close()
 
             # Optionally enable deletion of missing args if requested
-            enable_delete_missing = (
-                os.environ.get("FLOW_RSYNC_DELETE_MISSING_ARGS", "0").lower() in {"1", "true", "yes"}
-            )
+            enable_delete_missing = os.environ.get(
+                "FLOW_RSYNC_DELETE_MISSING_ARGS", "0"
+            ).lower() in {"1", "true", "yes"}
 
             return {
                 "list_path": Path(tmp.name),
@@ -410,11 +411,11 @@ class RsyncTransferStrategy:
         source: Path,
         target: str,
         connection: "SSHConnectionInfo",
-        exclude_file: Optional[Path],
+        exclude_file: Path | None,
         *,
-        files_from: Optional[Path] = None,
+        files_from: Path | None = None,
         use_from0: bool = False,
-    ) -> List[str]:
+    ) -> list[str]:
         """Build rsync command with appropriate flags.
 
         Args:
@@ -462,7 +463,11 @@ class RsyncTransferStrategy:
             if use_from0:
                 cmd.append("--from0")
             # Deleting missing args can break on older remote rsync; guard via env
-            if os.environ.get("FLOW_RSYNC_DELETE_MISSING_ARGS", "0").lower() in {"1", "true", "yes"}:
+            if os.environ.get("FLOW_RSYNC_DELETE_MISSING_ARGS", "0").lower() in {
+                "1",
+                "true",
+                "yes",
+            }:
                 cmd.append("--delete-missing-args")
 
         # Add source and destination
@@ -474,9 +479,9 @@ class RsyncTransferStrategy:
 
     def _execute_with_progress(
         self,
-        cmd: List[str],
-        progress_callback: Optional[Callable[[TransferProgress], None]],
-        source_path: Optional[Path] = None,
+        cmd: list[str],
+        progress_callback: Callable[[TransferProgress], None] | None,
+        source_path: Path | None = None,
     ) -> dict:
         """Execute rsync with progress monitoring.
 
@@ -499,8 +504,16 @@ class RsyncTransferStrategy:
         current_file = None
 
         try:
+            # Important: rsync (especially openrsync on macOS) writes progress to stderr.
+            # If we don't drain stderr concurrently, the pipe buffer can fill and block
+            # the child process, causing the CLI to "hang" on Syncing code. To avoid
+            # that class of deadlocks, merge stderr into stdout and process a single stream.
             process = subprocess.Popen(
-                cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, bufsize=1
+                cmd,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.STDOUT,
+                text=True,
+                bufsize=1,
             )
 
             # Process output line by line
@@ -532,8 +545,8 @@ class RsyncTransferStrategy:
             process.wait()
 
             if process.returncode != 0:
-                stderr = process.stderr.read()
-                raise TransferError(f"rsync failed with code {process.returncode}: {stderr}")
+                # With stderr merged into stdout, we already streamed diagnostics above.
+                raise TransferError(f"rsync failed with code {process.returncode}")
 
             # Return collected stats
             return {
@@ -544,7 +557,7 @@ class RsyncTransferStrategy:
         except Exception as e:
             raise TransferError(f"rsync execution failed: {e}") from e
 
-    def _parse_rsync_progress(self, line: str) -> Optional[TransferProgress]:
+    def _parse_rsync_progress(self, line: str) -> TransferProgress | None:
         """Parse rsync progress line into structured data.
 
         Args:
