@@ -1,11 +1,9 @@
 """Simple region validation utilities."""
 
-from typing import List, Optional, Tuple
-
 from flow.providers.mithril.core.constants import VALID_REGIONS
 
 
-def validate_region(region: Optional[str]) -> Tuple[bool, Optional[str]]:
+def validate_region(region: str | None) -> tuple[bool, str | None]:
     """Validate region and suggest correction for common mistakes.
 
     Args:
@@ -27,6 +25,6 @@ def validate_region(region: Optional[str]) -> Tuple[bool, Optional[str]]:
     return False, None
 
 
-def list_regions() -> List[str]:
+def list_regions() -> list[str]:
     """Get list of valid regions."""
     return VALID_REGIONS.copy()
